@@ -17,5 +17,10 @@ GtkWidget *create_title(const char *name, const char *comment);
 const char *get_selected_font(struct font_prm *prm);
 GtkWidget *create_font_frame(struct font_prm *prm);
 void get_font_parameter(struct font_prm *prm, int *pt, int *spc, int *script, int *style, int *r, int *g, int *b);
+GtkWidget *dialog_new(const char *title, GCallback cancel_cb, GCallback ok_cb, gpointer user_data);
+GtkWidget *columnview_create(GType item_type);
+GtkColumnViewColumn *columnview_create_column(GtkWidget *columnview, const char *header, GCallback setup, GCallback bind, gpointer user_data);
+GListStore *columnview_get_list(GtkWidget *columnview);
+void use_dark_theme(void);
 
 #endif
