@@ -376,7 +376,7 @@ GRAaddlist(int GC,struct objlist *obj,N_VALUE *inst,
 }
 
 static int
-add_draw_obj(struct objlist *parent, char const **objects, int index)
+add_draw_obj(const struct objlist *parent, char const **objects, int index)
 {
   struct objlist *ocur;
   const char *objname;
@@ -871,7 +871,7 @@ GRAview(int GC,int x1,int y1,int x2,int y2,int clip)
 }
 
 void
-GRAlinestyle(int GC,int num,int *type,int width,enum GRA_LINE_CAP cap,enum GRA_LINE_JOIN join,
+GRAlinestyle(int GC,int num,const int *type,int width,enum GRA_LINE_CAP cap,enum GRA_LINE_JOIN join,
                   int miter)
 {
   char code;
@@ -1118,7 +1118,7 @@ GRAputpixel(int GC,int x,int y)
 }
 
 void
-GRAdrawpoly(int GC,int num,int *point,enum GRA_FILL_MODE fil)
+GRAdrawpoly(int GC,int num,const int *point,enum GRA_FILL_MODE fil)
 {
   char code;
   int i,*cpar,num2;
@@ -1141,7 +1141,7 @@ GRAdrawpoly(int GC,int num,int *point,enum GRA_FILL_MODE fil)
 }
 
 void
-GRAlines(int GC,int num,int *point)
+GRAlines(int GC,int num,const int *point)
 {
   char code;
   int i,*cpar;
@@ -1156,7 +1156,7 @@ GRAlines(int GC,int num,int *point)
 }
 
 void
-GRArotate(int x0, int y0, int *pos, int *rpos, int n, double dx, double dy)
+GRArotate(int x0, int y0, const int *pos, int *rpos, int n, double dx, double dy)
 {
   int i;
   for (i = 0; i < n; i++) {
@@ -3013,7 +3013,7 @@ GRAtextextentraw(char *s,char *font, int style,
 
 
 static int
-getintpar(char *s,int num,int cpar[])
+getintpar(const char *s,int num,int cpar[])
 {
   int i,pos1;
   char s2[256];
@@ -3595,7 +3595,7 @@ GRAinview(int GC,int x,int y)
 }
 
 void
-GRAcurvefirst(int GC,int num,int *dashlist,
+GRAcurvefirst(int GC,int num,const int *dashlist,
 	      clipfunc clipf,transfunc transf,diffunc diff,intpfunc intpf,void *local,
 	      double x0,double y0)
 {
@@ -3744,7 +3744,6 @@ GRAcmatchtod(double x,double y,struct cmatchtype *data)
 {
   double x1,y1,x2,y2;
   int gx1,gy1,gx2,gy2;
-  double r,r2,r3,ip;
 
   x1=data->x0;
   y1=data->y0;
@@ -3772,6 +3771,7 @@ GRAcmatchtod(double x,double y,struct cmatchtype *data)
   } else {
     if ((data->gclipf==NULL)
      || (data->gclipf(&x1,&y1,&x2,&y2,data->gflocal)==0)) {
+      double r, r2, r3;
       if (data->gtransf==NULL) {
         gx1=nround(x1);
         gy1=nround(y1);
@@ -3791,6 +3791,7 @@ GRAcmatchtod(double x,double y,struct cmatchtype *data)
       if ((r<=data->err) || (r3<data->err)) {
         data->match=TRUE;
       } else if (r2!=0) {
+        double ip;
         ip=((x2-x1)*(data->pointx-x1)+(y2-y1)*(data->pointy-y1))/r2;
         if ((0<=ip) && (ip<=r2)) {
           x2=x1+(x2-x1)*ip/r2;
