@@ -15,7 +15,7 @@ char *combo_box_get_active_text(GtkWidget *cbox);
 void combo_box_set_active(GtkWidget *cbox, int i);
 void combo_box_clear(GtkWidget *cbox);
 int combo_box_get_num(GtkWidget *cbox);
-void combo_box_append_text(GtkWidget *cbox, char *str);
+void combo_box_append_text(GtkWidget *cbox, const char *str);
 int combo_box_get_selected_row(GtkWidget *view, gchar *path, GtkTreeIter *iter, int col);
 
 #endif
