@@ -10,11 +10,15 @@
 #include "odraw.h"
 #include "oaxis.h"
 #include "odata.h"
+#include "ox11menu.h"
 #include "x11view.h"
 #include "x11menu.h"
 #include "x11dialg.h"
 #include "x11gui.h"
 #include "x11file.h"
+#include "x11lgnd.h"
+
+#define USE_OPACITY 0
 
 #define SETTING_PANEL_MARGIN 4
 #define LINE_WIDTH_ICON_NUM 7
@@ -24,7 +28,6 @@
 #define DEFAULT_JOIN_STR  "'miter'"
 
 #define DEFAULT_MARKER_TYPE MARKER_TYPE_NONE
-#define DEFAULT_MARKER_STR  "'none'"
 
 #define DEFAULT_STROKE_FILL_TYPE 1
 #define STROKE_FILL_ICON_NUM 8
@@ -40,20 +43,17 @@ struct presetting_widgets
   struct widget_info color1, color2;
   struct widget_info path_type;
   struct widget_info join_type;
-  GtkWidget *join_icon[JOIN_TYPE_NUM];
+  char *join_icon[JOIN_TYPE_NUM];
   struct widget_info marker_type_begin;
-  GtkWidget *marker_begin_icon[MARKER_TYPE_NUM];
   struct widget_info marker_type_end;
-  GtkWidget *marker_end_icon[MARKER_TYPE_NUM];
   struct widget_info mark_type_begin, mark_type_end;
   struct widget_info stroke_fill;
-  GtkWidget *stroke_fill_icon[STROKE_FILL_ICON_NUM];
+  char *stroke_fill_icon[STROKE_FILL_ICON_NUM];
   struct widget_info font, bold, italic, pt;
   struct widget_info mark_type, mark_size;
   enum JOIN_TYPE join;
   enum MARKER_TYPE marker_begin, marker_end;
-  struct MarkDialog mark, mark_begin, mark_end;
-  int lw, fill, stroke, close_path;
+  int fill, stroke, close_path;
 };
 
 static struct presetting_widgets Widgets = {{NULL}};
@@ -61,8 +61,17 @@ static int UpdateFieldsLock = TRUE;
 
 static void update_focused_obj(GtkWidget *widget, gpointer user_data);
 
+static void
+button_set_child(GtkWidget *button, const char *icon)
+{
+  GtkWidget *img;
+  img = gtk_image_new_from_icon_name(icon);
+  gtk_image_set_icon_size(GTK_IMAGE(img), Menulocal.icon_size);
+  gtk_menu_button_set_child(GTK_MENU_BUTTON(button), img);
+}
+
 static int
-check_selected_item(GSimpleAction *action, GVariant *parameter, char **item, GtkWidget *button, GtkWidget **icon)
+check_selected_item(GSimpleAction *action, GVariant *parameter, char **item, GtkWidget *button, char **icon)
 {
   const char *state;
   int i, selected;
@@ -70,7 +79,7 @@ check_selected_item(GSimpleAction *action, GVariant *parameter, char **item, Gtk
   state = g_variant_get_string(parameter, NULL);
   for (i = 0; item[i]; i++) {
     if (g_strcmp0(state, item[i]) == 0) {
-      gtk_button_set_image(GTK_BUTTON(button), icon[i]);
+      button_set_child(button, icon[i]);
       selected = i;
       break;
     }
@@ -87,17 +96,17 @@ JoinTypeAction_activated(GSimpleAction *action, GVariant *parameter, gpointer ap
 }
 
 static void
-MarkerTypeBeginAction_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
+marker_type_begin_selected(void)
 {
-  Widgets.marker_begin = check_selected_item(action, parameter, marker_type_char, Widgets.marker_type_begin.widget, Widgets.marker_begin_icon);
+  Widgets.marker_begin = combo_box_get_active (Widgets.marker_type_begin.widget);
   gtk_widget_set_sensitive(Widgets.mark_type_begin.widget, Widgets.marker_begin == MARKER_TYPE_MARK);
   update_focused_obj(Widgets.marker_type_begin.widget, GINT_TO_POINTER(Widgets.marker_begin));
 }
 
 static void
-MarkerTypeEndAction_activated(GSimpleAction *action, GVariant *parameter, gpointer app)
+marker_type_end_selected(void)
 {
-  Widgets.marker_end = check_selected_item(action, parameter, marker_type_char, Widgets.marker_type_end.widget, Widgets.marker_end_icon);
+  Widgets.marker_end = combo_box_get_active(Widgets.marker_type_end.widget);
   gtk_widget_set_sensitive(Widgets.mark_type_end.widget, Widgets.marker_end == MARKER_TYPE_MARK);
   update_focused_obj(Widgets.marker_type_end.widget, GINT_TO_POINTER(Widgets.marker_end));
 }
@@ -120,7 +129,7 @@ set_stroke_fill_icon(void)
   if (Widgets.close_path) {
     i |= PATH_TYPE_CLOSE;
   }
-  gtk_button_set_image(GTK_BUTTON(Widgets.stroke_fill.widget), Widgets.stroke_fill_icon[i]);
+  button_set_child(Widgets.stroke_fill.widget, Widgets.stroke_fill_icon[i]);
   update_focused_obj(Widgets.stroke_fill.widget, GINT_TO_POINTER(i));
 }
 
@@ -156,40 +165,22 @@ StrokeFillStrokeAction_activated(GSimpleAction *action, GVariant *parameter, gpo
 
 static GActionEntry ToolMenuEntries[] = {
   {"JoinTypeAction",            NULL, "s",  DEFAULT_JOIN_STR,  JoinTypeAction_activated},
-  {"MarkerTypeBeginAction",     NULL, "s",  DEFAULT_MARKER_STR, MarkerTypeBeginAction_activated},
-  {"MarkerTypeEndAction",       NULL, "s",  DEFAULT_MARKER_STR, MarkerTypeEndAction_activated},
   {"StrokeFillStrokeAction",    NULL, NULL, "true",            StrokeFillStrokeAction_activated},
   {"StrokeFillFillAction",      NULL, NULL, "false",           StrokeFillFillAction_activated},
   {"StrokeFillClosePathAction", NULL, NULL, "false",           StrokeFillClosePathAction_activated},
 };
 
 static void
-create_images_sub(const char *prefix, char **item, GtkWidget **icon)
+create_images_sub(const char *prefix, char **item, char **icon)
 {
   int i;
-  GtkWidget *img;
+  char *img;
   char img_file[256];
 
   for (i = 0; item[i]; i++) {
     snprintf(img_file, sizeof(img_file), "%s_%s-symbolic", prefix, item[i]);
-    img = gtk_image_new_from_icon_name(img_file, GTK_ICON_SIZE_LARGE_TOOLBAR);
+    img = g_strdup(img_file);
     icon[i] = img;
-    g_object_ref(img);
-  }
-}
-
-static void
-create_marker_images_sub(const char *postfix, char **item, GtkWidget **icon)
-{
-  int i;
-  GtkWidget *img;
-  char img_file[256];
-
-  for (i = 0; item[i]; i++) {
-    snprintf(img_file, sizeof(img_file), "%s_%s-symbolic", item[i], postfix);
-    img = gtk_image_new_from_icon_name(img_file, GTK_ICON_SIZE_LARGE_TOOLBAR);
-    icon[i] = img;
-    g_object_ref(img);
   }
 }
 
@@ -197,16 +188,13 @@ static void
 create_images(struct presetting_widgets *widgets)
 {
   int i;
-  create_marker_images_sub("begin", marker_type_char, widgets->marker_begin_icon);
-  create_marker_images_sub("end", marker_type_char, widgets->marker_end_icon);
   create_images_sub("join", joinchar, widgets->join_icon);
   for (i = 0; i < STROKE_FILL_ICON_NUM; i++) {
-    GtkWidget *img;
+    char *img;
     char img_file[256];
     snprintf(img_file, sizeof(img_file), "stroke_fill_%d-symbolic", i);
-    img = gtk_image_new_from_icon_name(img_file, GTK_ICON_SIZE_LARGE_TOOLBAR);
+    img = g_strdup(img_file);
     widgets->stroke_fill_icon[i] = img;
-    g_object_ref(img);
   }
 }
 
@@ -227,11 +215,15 @@ get_rgba(struct objlist *obj, int id, int r1, int g1, int b1, int a1, int r2, in
   putobj(obj, "stroke_R", id, &r1);
   putobj(obj, "stroke_G", id, &g1);
   putobj(obj, "stroke_B", id, &b1);
+#if USE_OPACITY
   putobj(obj, "stroke_A", id, &a1);
+#endif
   putobj(obj, "fill_R", id, &r2);
   putobj(obj, "fill_G", id, &g2);
   putobj(obj, "fill_B", id, &b2);
+#if USE_OPACITY
   putobj(obj, "fill_A", id, &a2);
+#endif
 }
 
 static void
@@ -289,7 +281,9 @@ set_text_obj(struct objlist *obj, int id)
   putobj(obj, "R", id, &r);
   putobj(obj, "G", id, &g);
   putobj(obj, "B", id, &b);
+#if USE_OPACITY
   putobj(obj, "A", id, &a);
+#endif
   pt = gtk_spin_button_get_value(GTK_SPIN_BUTTON(Widgets.pt.widget)) * 100;
   putobj(obj, "pt", id, &pt);
 }
@@ -299,6 +293,9 @@ set_path_type(struct objlist *obj, int id)
 {
   int type, interpolation;
   type = combo_box_get_active(Widgets.path_type.widget);
+  if (type < 0) {
+    return;
+  }
   if (type == 0) {
     putobj(obj, "type", id, &type);
   } else {
@@ -312,14 +309,19 @@ set_path_type(struct objlist *obj, int id)
 static int
 get_line_width_setting(void)
 {
-  return (2 << combo_box_get_active(Widgets.line_width.widget)) * 10;
+  int width;
+  width = combo_box_get_active(Widgets.line_width.widget);
+  if (width < 0) {
+    return 40;
+  }
+  return (2 << width) * 10;
 }
 
 void
 presetting_set_obj_field(struct objlist *obj, int id)
 {
   const char *name;
-  int ival, r1, g1, b1, a1, r2, g2, b2, a2, width;
+  int ival, r1, g1, b1, a1, r2, g2, b2, a2, width, mark;
 
   if (obj == NULL) {
     return;
@@ -342,22 +344,30 @@ presetting_set_obj_field(struct objlist *obj, int id)
     putobj(obj, "R", id, &r1);
     putobj(obj, "G", id, &g1);
     putobj(obj, "B", id, &b1);
+#if USE_OPACITY
     putobj(obj, "A", id, &a1);
+#endif
     putobj(obj, "gauge_R", id, &r1);
     putobj(obj, "gauge_G", id, &g1);
     putobj(obj, "gauge_B", id, &b1);
+#if USE_OPACITY
     putobj(obj, "gauge_A", id, &a1);
+#endif
     putobj(obj, "num_R", id, &r1);
     putobj(obj, "num_G", id, &g1);
     putobj(obj, "num_B", id, &b1);
+#if USE_OPACITY
     putobj(obj, "num_A", id, &a1);
+#endif
     set_font_style(obj, id, "num_font_style");
     set_font(obj, id, "num_font");
     ival = gtk_spin_button_get_value(GTK_SPIN_BUTTON(Widgets.pt.widget)) * 100;
     putobj(obj, "num_pt", id, &ival);
     ival = combo_box_get_active(Widgets.line_style.widget);
-    sputobjfield(obj, id, "style", FwLineStyle[ival].list);
-    sputobjfield(obj, id, "gauge_style", FwLineStyle[ival].list);
+    if (ival >= 0) {
+      sputobjfield(obj, id, "style", FwLineStyle[ival].list);
+      sputobjfield(obj, id, "gauge_style", FwLineStyle[ival].list);
+    }
   } else if (strcmp(name, "axisgrid") == 0) {
     ival = width / 2;
     putobj(obj, "width3", id, &ival);
@@ -375,12 +385,16 @@ presetting_set_obj_field(struct objlist *obj, int id)
     putobj(obj, "marker_begin", id, &ival);
     ival = Widgets.marker_end;
     putobj(obj, "marker_end", id, &ival);
-    putobj(obj, "mark_type_begin", id, &(Widgets.mark_begin.Type));
-    putobj(obj, "mark_type_end", id, &(Widgets.mark_end.Type));
+    mark = get_mark_type_from_widget (Widgets.mark_type_begin.widget);
+    putobj(obj, "mark_type_begin", id, &mark);
+    mark = get_mark_type_from_widget (Widgets.mark_type_end.widget);
+    putobj(obj, "mark_type_end", id, &mark);
     putobj(obj, "width", id, &width);
     get_rgba(obj, id, r1, g1, b1, a1, r2, g2, b2, a2);
     ival = combo_box_get_active(Widgets.line_style.widget);
-    sputobjfield(obj, id, "style", FwLineStyle[ival].list);
+    if (ival >= 0) {
+      sputobjfield(obj, id, "style", FwLineStyle[ival].list);
+    }
     set_path_type(obj, id);
   } else if (strcmp(name, "rectangle") == 0) {
     putobj(obj, "stroke", id, &(Widgets.stroke));
@@ -388,7 +402,9 @@ presetting_set_obj_field(struct objlist *obj, int id)
     putobj(obj, "width", id, &width);
     get_rgba(obj, id, r1, g1, b1, a1, r2, g2, b2, a2);
     ival = combo_box_get_active(Widgets.line_style.widget);
-    sputobjfield(obj, id, "style", FwLineStyle[ival].list);
+    if (ival >= 0) {
+      sputobjfield(obj, id, "style", FwLineStyle[ival].list);
+    }
   } else if (strcmp(name, "arc") == 0) {
     putobj(obj, "stroke", id, &(Widgets.stroke));
     putobj(obj, "fill", id, &(Widgets.fill));
@@ -398,28 +414,39 @@ presetting_set_obj_field(struct objlist *obj, int id)
     putobj(obj, "width", id, &width);
     get_rgba(obj, id, r1, g1, b1, a1, r2, g2, b2, a2);
     ival = combo_box_get_active(Widgets.line_style.widget);
-    sputobjfield(obj, id, "style", FwLineStyle[ival].list);
+    if (ival >= 0) {
+      sputobjfield(obj, id, "style", FwLineStyle[ival].list);
+    }
     ival = Widgets.marker_begin;
     putobj(obj, "marker_begin", id, &ival);
     ival = Widgets.marker_end;
     putobj(obj, "marker_end", id, &ival);
-    putobj(obj, "mark_type_begin", id, &(Widgets.mark_begin.Type));
-    putobj(obj, "mark_type_end", id, &(Widgets.mark_end.Type));
+    mark = get_mark_type_from_widget (Widgets.mark_type_begin.widget);
+    putobj(obj, "mark_type_begin", id, &mark);
+    mark = get_mark_type_from_widget (Widgets.mark_type_end.widget);
+    putobj(obj, "mark_type_end", id, &mark);
   } else if (strcmp(name, "mark") == 0) {
     putobj(obj, "width", id, &width);
     putobj(obj, "R", id, &r1);
     putobj(obj, "G", id, &g1);
     putobj(obj, "B", id, &b1);
+#if USE_OPACITY
     putobj(obj, "A", id, &a1);
+#endif
     putobj(obj, "R2", id, &r2);
     putobj(obj, "G2", id, &g2);
     putobj(obj, "B2", id, &b2);
+#if USE_OPACITY
     putobj(obj, "A2", id, &a2);
+#endif
     ival = gtk_spin_button_get_value(GTK_SPIN_BUTTON(Widgets.mark_size.widget)) * 100;
     putobj(obj, "size", id, &ival);
-    putobj(obj, "type", id, &(Widgets.mark.Type));
+    mark = get_mark_type_from_widget (Widgets.mark_type.widget);
+    putobj(obj, "type", id, &mark);
     ival = combo_box_get_active(Widgets.line_style.widget);
-    sputobjfield(obj, id, "style", FwLineStyle[ival].list);
+    if (ival >= 0) {
+      sputobjfield(obj, id, "style", FwLineStyle[ival].list);
+    }
   } else if (strcmp(name, "text") == 0) {
     set_text_obj(obj, id);
   }
@@ -443,8 +470,8 @@ create_toggle_button(GtkWidget *box, GtkWidget *img, const char *tooltip, int st
 {
   GtkWidget *w;
   w = gtk_toggle_button_new();
-  gtk_container_add(GTK_CONTAINER(w), img);
-  gtk_box_pack_start(GTK_BOX(box), w, FALSE, FALSE, 0);
+  gtk_button_set_child(GTK_BUTTON(w), img);
+  gtk_box_append(GTK_BOX(box), w);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), state);
   gtk_widget_set_tooltip_text(w, tooltip);
   g_signal_connect(w, "toggled", G_CALLBACK(update_focused_obj), NULL);
@@ -511,13 +538,14 @@ widget_set_line_width(struct objlist *obj, N_VALUE *inst)
   } else if (index < 0) {
     index = 0;
   }
-  gtk_combo_box_set_active(GTK_COMBO_BOX(Widgets.line_width.widget), index);
+  combo_box_set_active(Widgets.line_width.widget, index);
 }
 
 static void
 widget_set_rgba_color(struct objlist *obj, N_VALUE *inst, GtkWidget *button, const char *prefix, const char *postfix)
 {
-  char field[256], *color_str[] = {"R", "G", "B", "A"};
+  char field[256];
+  const char *color_str[] = {"R", "G", "B", "A"};
   int color[4], i;
   GdkRGBA gcolor;
 
@@ -530,20 +558,32 @@ widget_set_rgba_color(struct objlist *obj, N_VALUE *inst, GtkWidget *button, con
   gcolor.red   = color[0] / 255.0;
   gcolor.green = color[1] / 255.0;
   gcolor.blue  = color[2] / 255.0;
+#if USE_OPACITY
   gcolor.alpha = color[3] / 255.0;
+#else
+  gcolor.alpha = 1.0;
+#endif
   gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(button), &gcolor);
 }
 
 static void
 widget_set_marker_type_begin(struct objlist *obj, N_VALUE *inst)
 {
-  set_radio_action(obj, inst, "marker_begin", marker_type_char, "MarkerTypeBeginAction");
+  int type;
+  if (_getobj(obj, "marker_begin", inst, &type)) {
+    return;
+  }
+  combo_box_set_active (Widgets.marker_type_begin.widget, type);
 }
 
 static void
 widget_set_marker_type_end(struct objlist *obj, N_VALUE *inst)
 {
-  set_radio_action(obj, inst, "marker_end", marker_type_char, "MarkerTypeEndAction");
+  int type;
+  if (_getobj(obj, "marker_end", inst, &type)) {
+    return;
+  }
+  combo_box_set_active (Widgets.marker_type_end.widget, type);
 }
 
 static void
@@ -554,14 +594,13 @@ widget_set_marker_type(struct objlist *obj, N_VALUE *inst)
 }
 
 static void
-widget_set_mark_type(struct objlist *obj, N_VALUE *inst, GtkWidget *button, const char *field, struct MarkDialog *d)
+widget_set_mark_type(struct objlist *obj, N_VALUE *inst, GtkWidget *button, const char *field)
 {
   int type;
   if (_getobj(obj, field, inst, &type)) {
     return;
   }
   button_set_mark_image(button, type);
-  d->Type = type;
 }
 
 static void
@@ -573,7 +612,7 @@ widget_set_line_style(struct objlist *obj, N_VALUE *inst, char *field)
   if (style < 0) {
     style = 0;
   }
-  gtk_combo_box_set_active(GTK_COMBO_BOX(Widgets.line_style.widget), style);
+  combo_box_set_active(Widgets.line_style.widget, style);
 }
 
 static void
@@ -586,7 +625,7 @@ widget_set_path_type(struct objlist *obj, N_VALUE *inst)
     _getobj(obj, "interpolation", inst, &interpolation);
     type = interpolation + 1;
   }
-  gtk_combo_box_set_active(GTK_COMBO_BOX(Widgets.path_type.widget), type);
+  combo_box_set_active(Widgets.path_type.widget, type);
 }
 
 static void
@@ -612,7 +651,7 @@ static int
 check_font_index(const char *font)
 {
   int i;
-  struct fontmap *fcur;
+  const struct fontmap *fcur;
 
   if (font == NULL) {
     return 0;
@@ -638,7 +677,7 @@ widget_set_font(struct objlist *obj, N_VALUE *inst, const char *field)
     return;
   }
   index = check_font_index(font);
-  gtk_combo_box_set_active(GTK_COMBO_BOX(Widgets.font.widget), index);
+  combo_box_set_active(Widgets.font.widget, index);
 }
 
 static int
@@ -716,8 +755,8 @@ presetting_set_parameters(struct Viewer *d)
       widget_set_rgba_color(obj, inst, Widgets.color1.widget, "stroke_", NULL);
       widget_set_rgba_color(obj, inst, Widgets.color2.widget, "fill_", NULL);
       widget_set_marker_type(obj, inst);
-      widget_set_mark_type(obj, inst, Widgets.mark_type_begin.widget, "mark_type_begin", &(Widgets.mark_begin));
-      widget_set_mark_type(obj, inst, Widgets.mark_type_end.widget, "mark_type_end", &(Widgets.mark_end));
+      widget_set_mark_type(obj, inst, Widgets.mark_type_begin.widget, "mark_type_begin");
+      widget_set_mark_type(obj, inst, Widgets.mark_type_end.widget, "mark_type_end");
       widget_set_line_style(obj, inst, "style");
       widget_set_path_type(obj, inst);
     } else if (obj == rect_obj) {
@@ -738,14 +777,14 @@ presetting_set_parameters(struct Viewer *d)
       widget_set_rgba_color(obj, inst, Widgets.color1.widget, "stroke_", NULL);
       widget_set_rgba_color(obj, inst, Widgets.color2.widget, "fill_", NULL);
       widget_set_marker_type(obj, inst);
-      widget_set_mark_type(obj, inst, Widgets.mark_type_begin.widget, "mark_type_begin", &(Widgets.mark_begin));
-      widget_set_mark_type(obj, inst, Widgets.mark_type_end.widget, "mark_type_end", &(Widgets.mark_end));
+      widget_set_mark_type(obj, inst, Widgets.mark_type_begin.widget, "mark_type_begin");
+      widget_set_mark_type(obj, inst, Widgets.mark_type_end.widget, "mark_type_end");
       widget_set_line_style(obj, inst, "style");
     } else if (obj == mark_obj) {
       widget_set_line_width(obj, inst);
       widget_set_rgba_color(obj, inst, Widgets.color1.widget, NULL, NULL);
       widget_set_rgba_color(obj, inst, Widgets.color2.widget, NULL, "2");
-      widget_set_mark_type(obj, inst, Widgets.mark_type.widget, "type", &(Widgets.mark));
+      widget_set_mark_type(obj, inst, Widgets.mark_type.widget, "type");
       widget_set_line_style(obj, inst, "style");
       widget_set_spin_value(obj, inst, Widgets.mark_size.widget, "size");
     }
@@ -963,7 +1002,7 @@ check_focused_obj(struct FOCUS_OBJ_INFO *info)
   }
   num = get_focused_obj_type_array(NgraphApp.Viewer.focusobj, objs);
   for (i = 0; i < num; i++) {
-    struct FocusObj *focus;
+    const struct FocusObj *focus;
     focus = &objs[i];
     for (j = 0; j < FOCUS_OBJ_N; j++) {
       if (info->info[j].obj == focus->obj) {
@@ -1052,7 +1091,7 @@ chk_update_field(struct objlist *obj, N_VALUE *inst, const char *field, int new_
 typedef int (* AXISGRID_CALLBACK)(struct objlist *, N_VALUE *, void *);
 
 static int
-check_axis_id(struct AxisGroupInfo *info, int id)
+check_axis_id(const struct AxisGroupInfo *info, int id)
 {
   int i;
   for (i = 0; i < info->num; i++) {
@@ -1064,7 +1103,7 @@ check_axis_id(struct AxisGroupInfo *info, int id)
 }
 
 static int
-check_axisgrid(struct objlist *aobj, struct AxisGroupInfo *info, AXISGRID_CALLBACK cb, void *usr_data)
+check_axisgrid(struct objlist *aobj, const struct AxisGroupInfo *info, AXISGRID_CALLBACK cb, void *usr_data)
 {
   int i, n, aid, r;
   struct objlist *obj;
@@ -1219,6 +1258,9 @@ update_focused_obj_line_style(GtkWidget *widget, struct Viewer *d, int num)
   axis_obj = chkobject("axis");
   text_obj = chkobject("text");
   style = combo_box_get_active(widget);
+  if (style < 0) {
+    return modified;
+  }
   style_str = FwLineStyle[style].list;
   for (i = 0; i < num; i++) {
     focus = *(struct FocusObj **) arraynget(d->focusobj, i);
@@ -1348,13 +1390,17 @@ update_focused_obj_color2(GtkWidget *widget, struct Viewer *d, int num)
       _putobj(obj, "fill_R", inst, &r);
       _putobj(obj, "fill_G", inst, &g);
       _putobj(obj, "fill_B", inst, &b);
+#if USE_OPACITY
       _putobj(obj, "fill_A", inst, &a);
+#endif
       modified = TRUE;          /* really modified */
     } else if (obj == mark_obj) {
       _putobj(obj, "R2", inst, &r);
       _putobj(obj, "G2", inst, &g);
       _putobj(obj, "B2", inst, &b);
+#if USE_OPACITY
       _putobj(obj, "A2", inst, &a);
+#endif
       modified = TRUE;          /* really modified */
     }
   }
@@ -1618,6 +1664,12 @@ update_focused_obj_font_style(struct Viewer *d, int num, int style, int apply)
   return modified;
 }
 
+static void
+update_focused_obj_from_notify (GtkWidget *w, GParamSpec *pspec, gpointer user_data)
+{
+  update_focused_obj(w, user_data);
+}
+
 static void
 update_focused_obj(GtkWidget *widget, gpointer user_data)
 {
@@ -1690,38 +1742,124 @@ update_focused_obj(GtkWidget *widget, gpointer user_data)
   UpdateAll(objs);
 }
 
+static void
+factory_method_setup(GtkSignalListItemFactory *self, GtkListItem *list_item, gpointer user_data)
+{
+  GtkWidget *box, *label, *icon;
+  box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+
+  label = gtk_label_new(NULL);
+  gtk_box_append(GTK_BOX(box), label);
+
+  icon = gtk_image_new();
+  gtk_box_append(GTK_BOX(box), icon);
+
+  icon = gtk_image_new();
+  gtk_box_append(GTK_BOX(box), icon);
+
+  gtk_list_item_set_child(list_item, box);
+}
+
+static void
+selected_item_changed (GtkDropDown *self, GParamSpec *pspec, GtkListItem *list_item)
+{
+  GtkWidget *box;
+  GtkWidget *check;
+
+  box = gtk_list_item_get_child (list_item);
+  check = gtk_widget_get_last_child (box);
+
+  if (gtk_drop_down_get_selected_item (self) == gtk_list_item_get_item (list_item)) {
+    gtk_widget_set_opacity (check, 1.0);
+  } else {
+    gtk_widget_set_opacity (check, 0.0);
+  }
+}
+
+static void
+factory_method_bind(GtkSignalListItemFactory *factory, GtkListItem *list_item, gpointer user_data)
+{
+  GtkWidget *box, *label, *icon, *check, *combo;
+  gpointer item;
+  const char *string;
+  char buf[64], img_file[256];
+  int i, width;
+
+  combo = GTK_WIDGET(user_data);
+  item = gtk_list_item_get_item (list_item);
+  box = gtk_list_item_get_child(list_item);
+  label = gtk_widget_get_first_child (box);
+  icon = gtk_widget_get_next_sibling (label);
+  check = gtk_widget_get_next_sibling (icon);
+
+  string = gtk_string_object_get_string (GTK_STRING_OBJECT (item));
+  i = atoi(string);
+  width = 2 << i;
+
+  snprintf(img_file, sizeof(img_file), "linewidth_%03d-symbolic", width);
+  gtk_image_set_from_icon_name(GTK_IMAGE (icon), img_file);
+
+  gtk_image_set_from_icon_name(GTK_IMAGE (check), "object-select-symbolic");
+
+  snprintf(buf, sizeof(buf), "%4.1f", width / 10.0);
+  gtk_label_set_label (GTK_LABEL (label), buf);
+
+  if (gtk_widget_get_ancestor (box, GTK_TYPE_POPOVER)) {
+    g_signal_connect (combo, "notify::selected-item", G_CALLBACK (selected_item_changed), list_item);
+    selected_item_changed (GTK_DROP_DOWN(combo), NULL, list_item);
+    gtk_widget_set_visible (check, TRUE);
+  } else {
+    gtk_widget_set_visible (check, FALSE);
+  }
+}
+
+static void
+line_width_changed(GtkWidget *cbox, GParamSpec *pspec, GtkListItem *list_item)
+{
+  int selected;
+
+  selected = combo_box_get_active(cbox);
+  if (selected < 0) {
+    return;
+  }
+  update_focused_obj(cbox, NULL);
+}
+
+static void
+factory_method_unbind(GtkSignalListItemFactory *factory, GtkListItem *list_item, gpointer user_data)
+{
+  GtkDropDown *self = user_data;
+
+  g_signal_handlers_disconnect_by_func (self, selected_item_changed, list_item);
+}
+
 static GtkWidget *
 create_line_width_combo_box(void)
 {
-  GtkWidget *cbox;
-  GtkListStore *list;
-  GtkTreeIter iter;
-  int j;
-  GtkCellRenderer *rend;
+  GtkStringList *list;
+  GtkWidget *combo;
+  GtkListItemFactory* factory;
+  int i;
 
-  list = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT);
-  cbox = gtk_combo_box_new_with_model(GTK_TREE_MODEL(list));
-  rend = gtk_cell_renderer_text_new();
-  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(cbox), rend, FALSE);
-  gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(cbox), rend, "text", 0);
-  rend = gtk_cell_renderer_pixbuf_new();
-  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(cbox), rend, FALSE);
-  gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(cbox), rend, "icon-name", 1);
-  gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(cbox), rend, "stock-size", 2);
-  for (j = 0; j < LINE_WIDTH_ICON_NUM; j++) {
-    char buf[64], img_file[256];
-    snprintf(img_file, sizeof(img_file), "linewidth_%03d-symbolic", 2 << j);
-    gtk_list_store_append(list, &iter);
-    snprintf(buf, sizeof(buf), "%4.1f", (2 << j) / 10.0);
-    gtk_list_store_set(list, &iter,
-		       0, buf,
-		       1, img_file,
-		       2, GTK_ICON_SIZE_LARGE_TOOLBAR,
-		       -1);
+  list = gtk_string_list_new(NULL);
+
+  combo = gtk_drop_down_new(G_LIST_MODEL(list), NULL);
+
+  factory = gtk_signal_list_item_factory_new();
+  g_signal_connect(factory, "setup", G_CALLBACK(factory_method_setup), combo);
+  g_signal_connect(factory, "bind", G_CALLBACK(factory_method_bind), combo);
+  g_signal_connect (factory, "unbind", G_CALLBACK (factory_method_unbind), combo);
+
+  gtk_drop_down_set_factory(GTK_DROP_DOWN(combo), factory);
+  for (i = 0; i < LINE_WIDTH_ICON_NUM; i++) {
+    char buf[64];
+    snprintf(buf, sizeof(buf), "%d", i);
+    combo_box_append_text(combo, buf);
   }
-  gtk_combo_box_set_active(GTK_COMBO_BOX(cbox), 1);
-  g_signal_connect(cbox, "changed", G_CALLBACK(update_focused_obj), NULL);
-  return cbox;
+
+  combo_box_set_active(combo, 1);
+  g_signal_connect(combo, "notify::selected", G_CALLBACK(line_width_changed), NULL);
+  return combo;
 }
 
 static GtkWidget *
@@ -1730,12 +1868,12 @@ create_line_style_combo_box(void)
   GtkWidget *cbox;
   int j;
 
-  cbox = gtk_combo_box_text_new();
+  cbox = combo_box_create();
   for (j = 0; j < LINE_STYLE_ICON_NUM; j++) {
-    gtk_combo_box_text_append(GTK_COMBO_BOX_TEXT(cbox), NULL, _(FwLineStyle[j].name));
+    combo_box_append_text(cbox, _(FwLineStyle[j].name));
   }
-  gtk_combo_box_set_active(GTK_COMBO_BOX(cbox), 0);
-  g_signal_connect(cbox, "changed", G_CALLBACK(update_focused_obj), NULL);
+  combo_box_set_active(cbox, 0);
+  g_signal_connect(cbox, "notify::selected", G_CALLBACK(update_focused_obj_from_notify), NULL);
   return cbox;
 }
 
@@ -1744,13 +1882,13 @@ create_path_type_combo_box(void)
 {
   GtkWidget *cbox;
   int j;
-  cbox = gtk_combo_box_text_new();
-  gtk_combo_box_text_append(GTK_COMBO_BOX_TEXT(cbox), NULL, _("line"));
+  cbox = combo_box_create();
+  combo_box_append_text(cbox, _("line"));
   for (j = 0; intpchar[j]; j++) {
-    gtk_combo_box_text_append(GTK_COMBO_BOX_TEXT(cbox), NULL, _(intpchar[j]));
+    combo_box_append_text(cbox, _(intpchar[j]));
   }
-  gtk_combo_box_set_active(GTK_COMBO_BOX(cbox), 0);
-  g_signal_connect(cbox, "changed", G_CALLBACK(update_focused_obj), NULL);
+  combo_box_set_active(cbox, 0);
+  g_signal_connect(cbox, "notify::selected", G_CALLBACK(update_focused_obj_from_notify), NULL);
   return cbox;
 }
 
@@ -1767,30 +1905,51 @@ create_menu_button(GtkBuilder *builder, const char *menu_name, const char *toolt
 }
 
 static void
-select_mark(GtkWidget *w, gpointer client_data)
+select_mark(GtkWidget *w)
 {
-  struct MarkDialog *d;
-  int ret;
-
-  d = (struct MarkDialog *) client_data;
-  ret = DialogExecute(d->parent, d);
-  if (ret == IDOK) {
-    button_set_mark_image(w, d->Type);
-    update_focused_obj(w, GINT_TO_POINTER(d->Type));
-  }
+  int type;
+  type = get_mark_type_from_widget (w);
+  update_focused_obj(w, GINT_TO_POINTER(type));
 }
 
 static void
-setup_mark_type(GtkWidget *type, struct MarkDialog *mark)
+setup_mark_type(GtkWidget *type)
 {
+  mark_popover_new (type, select_mark);
   button_set_mark_image(type, 0);
-  MarkDialog(mark, TopLevel, 0);
+}
+
+void
+presetting_set_fonts(void)
+{
+  int lock, index;
+  char *font;
+  GtkWidget *cbox;
+
+  cbox = Widgets.font.widget;
+  if (cbox == NULL) {
+    return;
+  }
+
+  font = combo_box_get_active_text(cbox);
+  if (font == NULL) {
+    return;
+  }
+
+  lock = UpdateFieldsLock;
+  UpdateFieldsLock = TRUE;
+  set_font_family(GTK_WIDGET(cbox));
+  index = check_font_index(font);
+  combo_box_set_active(cbox, index);
+  g_free(font);
+  UpdateFieldsLock = lock;
 }
 
 GtkWidget *
 presetting_create_panel(GtkApplication *app)
 {
   GtkWidget *w, *box, *img;
+  GtkWidget *button_box;
   GtkBuilder *builder;
   GdkRGBA color;
 
@@ -1807,65 +1966,68 @@ presetting_create_panel(GtkApplication *app)
   w = combo_box_create();
   set_font_family(w);
   gtk_widget_set_tooltip_text(w, _("Font name"));
-  gtk_box_pack_start(GTK_BOX(box), w, FALSE, FALSE, 0);
-  g_signal_connect(w, "changed", G_CALLBACK(update_focused_obj), NULL);
+  gtk_box_append(GTK_BOX(box), w);
+  g_signal_connect(w, "notify::selected", G_CALLBACK(update_focused_obj_from_notify), NULL);
   Widgets.font.widget = w;
 
   w = create_spin_entry_type(SPIN_BUTTON_TYPE_POINT, FALSE, FALSE);
+  gtk_editable_set_enable_undo(GTK_EDITABLE(w), FALSE);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(w), DEFAULT_FONT_PT / 100.0);
-  gtk_entry_set_width_chars(GTK_ENTRY(w), 5);
+  gtk_editable_set_width_chars(GTK_EDITABLE(w), 5);
   gtk_widget_set_tooltip_text(w, _("Font size"));
-  gtk_box_pack_start(GTK_BOX(box), w, FALSE, FALSE, 0);
+  gtk_box_append(GTK_BOX(box), w);
   g_signal_connect(w, "value-changed", G_CALLBACK(update_focused_obj), NULL);
   Widgets.pt.widget = w;
 
-  img = gtk_image_new_from_icon_name("format-text-bold-symbolic", GTK_ICON_SIZE_BUTTON);
+  img = gtk_image_new_from_icon_name("format-text-bold-symbolic");
+  gtk_image_set_icon_size(GTK_IMAGE(img), Menulocal.icon_size);
   Widgets.bold.widget = create_toggle_button(box, img, _("Bold"), FALSE);
 
-  img = gtk_image_new_from_icon_name("format-text-italic-symbolic", GTK_ICON_SIZE_BUTTON);
+  img = gtk_image_new_from_icon_name("format-text-italic-symbolic");
+  gtk_image_set_icon_size(GTK_IMAGE(img), Menulocal.icon_size);
   Widgets.italic.widget = create_toggle_button(box, img,  _("Italic"), FALSE);
   gtk_widget_set_margin_end(Widgets.italic.widget, SETTING_PANEL_MARGIN * 4);
 
   w = create_path_type_combo_box();
   gtk_widget_set_margin_end(w, SETTING_PANEL_MARGIN * 4);
-  gtk_box_pack_start(GTK_BOX(box), w, FALSE, FALSE, 0);
+  gtk_box_append(GTK_BOX(box), w);
   Widgets.path_type.widget = w;
 
   w = gtk_button_new();
-  g_signal_connect(w, "clicked", G_CALLBACK(select_mark), &(Widgets.mark));
-  setup_mark_type(w, &(Widgets.mark));
-  gtk_box_pack_start(GTK_BOX(box), w, FALSE, FALSE, 0);
+  g_signal_connect(w, "clicked", G_CALLBACK(mark_popover_popup), NULL);
+  setup_mark_type(w);
+  gtk_box_append(GTK_BOX(box), w);
   Widgets.mark_type.widget = w;
 
   w = create_spin_entry_type(SPIN_BUTTON_TYPE_LENGTH, FALSE, FALSE);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(w), DEFAULT_MARK_SIZE / 100.0);
-  gtk_entry_set_width_chars(GTK_ENTRY(w), 5);
+  gtk_editable_set_width_chars(GTK_EDITABLE(w), 5);
   gtk_widget_set_tooltip_text(w, _("Mark size"));
-  gtk_box_pack_start(GTK_BOX(box), w, FALSE, FALSE, 0);
+  gtk_box_append(GTK_BOX(box), w);
   g_signal_connect(w, "value-changed", G_CALLBACK(update_focused_obj), NULL);
   Widgets.mark_size.widget = w;
 
   w = create_menu_button(builder, "stroke-fill-menu", _("stroke/fill"));
   Widgets.stroke_fill.widget = w;
   Widgets.stroke = TRUE;
-  gtk_box_pack_start(GTK_BOX(box), w, FALSE, FALSE, 0);
+  gtk_box_append(GTK_BOX(box), w);
   set_stroke_fill_icon();
 
   w = create_line_width_combo_box();
   gtk_widget_set_tooltip_text(w, _("Line Width"));
-  gtk_box_pack_start(GTK_BOX(box), w, FALSE, FALSE, 0);
+  gtk_box_append(GTK_BOX(box), w);
   Widgets.line_width.widget = w;
 
   w = create_line_style_combo_box();
   gtk_widget_set_tooltip_text(w, _("Line Style"));
-  gtk_box_pack_start(GTK_BOX(box), w, FALSE, FALSE, 0);
+  gtk_box_append(GTK_BOX(box), w);
   Widgets.line_style.widget = w;
 
   w = create_color_button(NULL);
   color.red = color.green = color.blue = 0;
   color.alpha = 1;
   gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(w), &color);
-  gtk_box_pack_start(GTK_BOX(box), w, FALSE, FALSE, 0);
+  gtk_box_append(GTK_BOX(box), w);
   Widgets.color1.widget = w;
   gtk_widget_set_name(Widgets.color1.widget, "StrokeColorButton");
   g_signal_connect(w, "color-set", G_CALLBACK(update_focused_obj), NULL);
@@ -1874,37 +2036,47 @@ presetting_create_panel(GtkApplication *app)
   color.red = color.green = color.blue = 1;
   color.alpha = 1;
   gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(w), &color);
-  gtk_box_pack_start(GTK_BOX(box), w, FALSE, FALSE, 0);
+  gtk_box_append(GTK_BOX(box), w);
   g_signal_connect(w, "color-set", G_CALLBACK(update_focused_obj), NULL);
   Widgets.color2.widget = w;
 
   w = create_menu_button(builder, "join-type-menu", _("Join"));
-  gtk_box_pack_start(GTK_BOX(box), w, FALSE, FALSE, 0);
+  gtk_box_append(GTK_BOX(box), w);
   Widgets.join_type.widget = w;
-  gtk_button_set_image(GTK_BUTTON(Widgets.join_type.widget), Widgets.join_icon[DEFAULT_JOIN_TYPE]);
+  button_set_child(Widgets.join_type.widget, Widgets.join_icon[DEFAULT_JOIN_TYPE]);
 
   w = gtk_button_new();
-  g_signal_connect(w, "clicked", G_CALLBACK(select_mark), &(Widgets.mark_begin));
-  setup_mark_type(w, &(Widgets.mark_begin));
-  gtk_box_pack_start(GTK_BOX(box), w, FALSE, FALSE, 0);
+  g_signal_connect(w, "clicked", G_CALLBACK(mark_popover_popup), NULL);
+  setup_mark_type(w);
+  gtk_widget_add_css_class(w, MENUBUTTON_CLASS);
+  button_box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+  gtk_box_append(GTK_BOX(button_box), w);
   Widgets.mark_type_begin.widget = w;
 
-  w = create_menu_button(builder, "marker-type-begin-menu", _("Marker begin"));
-  gtk_box_pack_start(GTK_BOX(box), w, FALSE, FALSE, 0);
+  w = create_marker_type_combo_box("begin", _("Marker begin"));
+  gtk_widget_add_css_class(w, MENUBUTTON_CLASS);
+  gtk_box_append(GTK_BOX(button_box), w);
+  gtk_box_append(GTK_BOX(box), button_box);
   Widgets.marker_type_begin.widget = w;
   Widgets.marker_begin = DEFAULT_MARKER_TYPE;
-  gtk_button_set_image(GTK_BUTTON(Widgets.marker_type_begin.widget), Widgets.marker_begin_icon[DEFAULT_MARKER_TYPE]);
+  combo_box_set_active (w, Widgets.marker_begin);
+  g_signal_connect(Widgets.marker_type_begin.widget, "notify::selected-item", marker_type_begin_selected, NULL);
 
-  w = create_menu_button(builder, "marker-type-end-menu", _("Marker end"));
-  gtk_box_pack_start(GTK_BOX(box), w, FALSE, FALSE, 0);
+  w = create_marker_type_combo_box("end", _("Marker end"));
+  gtk_widget_add_css_class(w, MARKERBUTTON_CLASS);
+  button_box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+  gtk_box_append(GTK_BOX(button_box), w);
   Widgets.marker_type_end.widget = w;
   Widgets.marker_end = DEFAULT_MARKER_TYPE;
-  gtk_button_set_image(GTK_BUTTON(Widgets.marker_type_end.widget), Widgets.marker_end_icon[DEFAULT_MARKER_TYPE]);
+  combo_box_set_active (w, Widgets.marker_end);
+  g_signal_connect(Widgets.marker_type_end.widget, "notify::selected-item", marker_type_end_selected, NULL);
 
   w = gtk_button_new();
-  g_signal_connect(w, "clicked", G_CALLBACK(select_mark), &(Widgets.mark_end));
-  setup_mark_type(w, &(Widgets.mark_end));
-  gtk_box_pack_start(GTK_BOX(box), w, FALSE, FALSE, 0);
+  g_signal_connect(w, "clicked", G_CALLBACK(mark_popover_popup), NULL);
+  setup_mark_type(w);
+  gtk_widget_add_css_class(w, MARKERBUTTON_CLASS);
+  gtk_box_append(GTK_BOX(button_box), w);
+  gtk_box_append(GTK_BOX(box), button_box);
   Widgets.mark_type_end.widget = w;
 
   gtk_widget_set_sensitive(Widgets.mark_type_begin.widget, Widgets.marker_begin == MARKER_TYPE_MARK);
@@ -1913,3 +2085,34 @@ presetting_create_panel(GtkApplication *app)
   g_object_unref(builder);
   return box;
 }
+
+void
+presetting_get(struct presettings *setting)
+{
+  int active;
+  setting->line_width = get_line_width_setting();
+  active = combo_box_get_active(Widgets.line_style.widget);
+  if (active >= 0) {
+    setting->line_style = active;
+  }
+  active = combo_box_get_active(Widgets.path_type.widget);
+  if (active >= 0) {
+    setting->type = active;
+  }
+  setting->interpolation = setting->type - 1;
+  if (setting->type) {
+    setting->type = 1;
+  }
+  setting->join = Widgets.join;
+  setting->mark_type_begin = get_mark_type_from_widget (Widgets.mark_type_begin.widget);
+  setting->mark_type_end = get_mark_type_from_widget (Widgets.mark_type_end.widget);
+  setting->mark_type = get_mark_type_from_widget (Widgets.mark_type.widget);
+  setting->mark_size = gtk_spin_button_get_value(GTK_SPIN_BUTTON(Widgets.mark_size.widget)) * 100;;
+  setting->marker_begin = Widgets.marker_begin;
+  setting->marker_end = Widgets.marker_end;
+  setting->fill = Widgets.fill;
+  setting->stroke = Widgets.stroke;
+  setting->close_path = Widgets.close_path;
+  set_rgba(Widgets.color1.widget, &setting->r1, &setting->g1, &setting->b1, &setting->a1);
+  set_rgba(Widgets.color2.widget, &setting->r2, &setting->g2, &setting->b2, &setting->a2);
+}
