@@ -45,8 +45,6 @@ typedef struct _Nruler {
   double lower, upper, position;
   double save_l, save_u;
   GtkWidget *widget, *parent;
-  GtkStyleContext *saved_style;
-  GdkRGBA saved_fg;
 } Nruler;
 
 struct _NrulerMetric
@@ -67,11 +65,17 @@ static const struct _NrulerMetric Metric = {
 static void nruler_make_pixmap(Nruler *ruler, GtkWidget *widget, GtkWidget *parent);
 static void nruler_draw_ticks(Nruler *ruler, GtkWidget *widget);
 static void nruler_realize(GtkWidget *widget, gpointer user_data);
-static void nruler_size_allocate(GtkWidget *widget, GtkAllocation *allocation, gpointer user_data);
+static void nruler_resize(GtkWidget *widget, int width, int hegiht, gpointer user_data);
 static gboolean nruler_destroy(GtkWidget *widget, gpointer user_data);
 static void nruler_draw_pos(Nruler *ruler, GtkWidget *widget, cairo_t *cr);
 static gboolean nruler_expose(GtkWidget *widget, cairo_t *cr, gpointer user_data);
-static GtkStyleContext *nruler_get_color(Nruler *ruler, GdkRGBA *fg);
+static void nruler_get_color(Nruler *ruler, GdkRGBA *fg, GdkRGBA *bg);
+
+static void
+draw_function(GtkDrawingArea* drawing_area, cairo_t* cr, int width, int height, gpointer user_data)
+{
+  nruler_expose(GTK_WIDGET(drawing_area), cr, user_data);
+}
 
 GtkWidget *
 nruler_new(GtkOrientation orientation)
@@ -92,19 +96,18 @@ nruler_new(GtkOrientation orientation)
   }
 
   frame = gtk_frame_new(NULL);
-  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);
-  gtk_container_add(GTK_CONTAINER(frame), w);
+  gtk_widget_add_css_class(frame, "ruler");
+  gtk_frame_set_child(GTK_FRAME(frame), w);
 
   ruler->orientation = orientation;
   ruler->widget = w;
   ruler->parent = frame;
-  ruler->saved_style = NULL;
 
   g_object_set_data(G_OBJECT(frame), RULER_DATA_KEY, ruler);
 
-  g_signal_connect(w, "draw", G_CALLBACK(nruler_expose), ruler);
+  gtk_drawing_area_set_draw_func(GTK_DRAWING_AREA(w), draw_function, ruler, NULL);
   g_signal_connect(w, "realize", G_CALLBACK(nruler_realize), ruler);
-  g_signal_connect(w, "size-allocate", G_CALLBACK(nruler_size_allocate), ruler);
+  g_signal_connect(w, "resize", G_CALLBACK(nruler_resize), ruler);
   g_signal_connect(frame, "unrealize", G_CALLBACK(nruler_destroy), ruler);
 
   return frame;
@@ -172,13 +175,9 @@ nruler_destroy(GtkWidget *widget, gpointer user_data)
 }
 
 static void
-nruler_size_allocate(GtkWidget *widget, GtkAllocation *allocation, gpointer user_data)
+nruler_resize(GtkWidget *widget, int width, int hegiht, gpointer user_data)
 {
-  Nruler *ruler;
-
-  ruler = (Nruler *) user_data;
-
-  nruler_make_pixmap(ruler, widget, ruler->parent);
+  nruler_realize(widget, user_data);
 }
 
 static void
@@ -215,8 +214,10 @@ nruler_make_pixmap(Nruler *ruler, GtkWidget *widget, GtkWidget *parent)
     return;
   }
 
-  gtk_widget_get_allocation(widget, &allocation);
-  gtk_widget_get_allocation(parent, &parent_allocation);
+  allocation.width = gtk_widget_get_width (widget);
+  allocation.height = gtk_widget_get_height (widget);
+  parent_allocation.width = gtk_widget_get_width (parent);
+  parent_allocation.height = gtk_widget_get_height (parent);
 
   if (ruler->orientation == GTK_ORIENTATION_HORIZONTAL) {
     ruler->length = parent_allocation.width;
@@ -267,8 +268,7 @@ nruler_draw_ticks(Nruler *ruler, GtkWidget *widget)
   PangoRectangle logical_rect, ink_rect;
   PangoFontDescription *fs;
   GtkAllocation allocation;
-  GdkRGBA fg;
-  GtkStyleContext *context;
+  GdkRGBA fg, bg;
 
   if (! gtk_widget_is_drawable(widget)) {
     return;
@@ -278,7 +278,8 @@ nruler_draw_ticks(Nruler *ruler, GtkWidget *widget)
     return;
   }
 
-  gtk_widget_get_allocation(widget, &allocation);
+  allocation.width = gtk_widget_get_width (widget);
+  allocation.height = gtk_widget_get_height (widget);
 
   layout = gtk_widget_create_pango_layout(widget, "012456789");
   fs = pango_font_description_new();
@@ -290,10 +291,11 @@ nruler_draw_ticks(Nruler *ruler, GtkWidget *widget)
 
   digit_height = PANGO_PIXELS(ink_rect.height) + 2;
 
-  context = nruler_get_color(ruler, &fg);
+  nruler_get_color(ruler, &fg, &bg);
   cr = cairo_create(ruler->backing_store);
-  gtk_render_background(context, cr,
-			0, 0, allocation.width, allocation.height);
+  gdk_cairo_set_source_rgba(cr, &bg);
+  cairo_rectangle(cr, 0, 0, allocation.width, allocation.height);
+  cairo_fill (cr);
   gdk_cairo_set_source_rgba(cr, &fg);
 
   if (ruler->orientation == GTK_ORIENTATION_HORIZONTAL) {
@@ -421,29 +423,18 @@ nruler_draw_ticks(Nruler *ruler, GtkWidget *widget)
 }
 
 static void
-style_changed(GtkStyleContext *stylecontext, gpointer user_data)
+nruler_get_color(Nruler *ruler, GdkRGBA *fg, GdkRGBA *bg)
 {
-  Nruler *ruler;
+  static GdkRGBA white = {0.9, 0.9, 0.9, 1.0}, black = {0.2, 0.2, 0.2, 1.0};
+  GdkRGBA theme;
 
-  ruler = user_data;
-  gtk_style_context_get_color(stylecontext, GTK_STATE_FLAG_NORMAL, &ruler->saved_fg);
-}
-
-static GtkStyleContext *
-nruler_get_color(Nruler *ruler, GdkRGBA *fg)
-{
-  if (ruler->saved_style == NULL) {
-    ruler->saved_style = gtk_widget_get_style_context(TopLevel);
-    gtk_style_context_get_color(ruler->saved_style, GTK_STATE_FLAG_NORMAL, &ruler->saved_fg);
-    g_signal_connect(ruler->saved_style, "changed", G_CALLBACK(style_changed), ruler);
+  gtk_widget_get_color(ruler->widget, &theme);
+  if (fg) {
+    *fg = (theme.red > 0.8) ? white : black;
   }
-
-  if (fg == NULL) {
-    return ruler->saved_style;
+  if (bg) {
+    *bg = (theme.red > 0.8) ? black : white;
   }
-
-  *fg = ruler->saved_fg;
-  return ruler->saved_style;
 }
 
 static void
@@ -456,7 +447,8 @@ nruler_draw_pos(Nruler *ruler, GtkWidget *widget, cairo_t *cr)
   GtkAllocation allocation;
   GdkRGBA fg;
 
-  gtk_widget_get_allocation(widget, &allocation);
+  allocation.width = gtk_widget_get_width (widget);
+  allocation.height = gtk_widget_get_height (widget);
 
   if (! gtk_widget_is_drawable(widget)) {
     return;
@@ -500,7 +492,7 @@ nruler_draw_pos(Nruler *ruler, GtkWidget *widget, cairo_t *cr)
     y = nround((ruler->position - ruler->lower) * increment) - bs_height / 2 - ruler->ofst;
   }
 
-  nruler_get_color(ruler, &fg);
+  nruler_get_color(ruler, &fg, NULL);
   gdk_cairo_set_source_rgba(cr, &fg);
 
   cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);
