@@ -8,7 +8,6 @@
 #include "common.h"
 
 #include "gtk_common.h"
-#include "gtk_liststore.h"
 
 #include "object.h"
 #include "ogra2cairo.h"
@@ -42,23 +41,19 @@ void set_combo_cell_renderer_cb(struct obj_list_data *d, int col, n_list_store *
 void set_obj_cell_renderer_cb(struct obj_list_data *d, int col, n_list_store *list, GCallback start);
 void set_cell_attribute_source(struct SubWin *d, const char *attr, int target_column, int source_column);
 
-gboolean list_sub_window_must_rebuild(struct obj_list_data *d);
-void list_sub_window_build(struct obj_list_data *d, list_sub_window_set_val_func func);
-void list_sub_window_set(struct obj_list_data *d, list_sub_window_set_val_func func);
+void list_sub_window_build(struct obj_list_data *d);
 
-void list_sub_window_object_name(GtkMenuItem *w, gpointer client_data);
-void list_sub_window_delete(GtkMenuItem *item, gpointer user_data);
-void list_sub_window_copy(GtkMenuItem *item, gpointer user_data);
-void list_sub_window_move_top(GtkMenuItem *item, gpointer user_data);
-void list_sub_window_move_last(GtkMenuItem *item, gpointer user_data);
-void list_sub_window_move_up(GtkMenuItem *item, gpointer user_data);
-void list_sub_window_move_down(GtkMenuItem *item, gpointer user_data);
-void list_sub_window_update(GtkMenuItem *item, gpointer user_data);
-void list_sub_window_focus(GtkMenuItem *item, gpointer user_data);
-void list_sub_window_focus_all(GtkMenuItem *item, gpointer user_data);
-void list_sub_window_add_focus(GtkMenuItem *item, gpointer user_data);
+void list_sub_window_object_name(GSimpleAction *action, GVariant *parameter, gpointer user_data);
+void list_sub_window_delete(GSimpleAction *action, GVariant *parameter, gpointer user_data);
+void list_sub_window_copy(GSimpleAction *action, GVariant *parameter, gpointer user_data);
+void list_sub_window_move_top(GSimpleAction *action, GVariant *parameter, gpointer user_data);
+void list_sub_window_move_last(GSimpleAction *action, GVariant *parameter, gpointer user_data);
+void list_sub_window_move_up(GSimpleAction *action, GVariant *parameter, gpointer user_data);
+void list_sub_window_move_down(GSimpleAction *action, GVariant *parameter, gpointer user_data);
+void list_sub_window_update(GSimpleAction *action, GVariant *parameter, gpointer user_data);
+void list_sub_window_focus(GSimpleAction *action, GVariant *parameter, gpointer user_data);
+void list_sub_window_focus_all(GSimpleAction *action, GVariant *parameter, gpointer user_data);
 void update_viewer(struct obj_list_data *d);
 
-GtkWidget *sub_win_create_popup_menu(struct obj_list_data *d, int n, struct subwin_popup_list *list, GCallback cb);
-
+void sub_win_create_popup_menu(struct obj_list_data *d, int n, GActionEntry *list, GCallback cb);
 #endif
