@@ -12,9 +12,89 @@
 #include "x11gui.h"
 #include "dir_defs.h"
 
-void
-set_button_icon(GtkWidget *w, const char *icon_name)
+GtkWidget *
+button_new_with_icon(const char *icon_name, int toggle)
 {
+  GtkWidget *img, *button;
+  img = gtk_image_new_from_icon_name(icon_name);
+  gtk_image_set_icon_size(GTK_IMAGE(img), Menulocal.icon_size);
+  if (toggle) {
+    button = gtk_toggle_button_new();
+  } else {
+    button = gtk_button_new();
+  }
+  gtk_button_set_child(GTK_BUTTON(button), img);
+  return button;
+}
+
+void
+editable_set_init_text(GtkWidget *w, const char *text)
+{
+  gtk_editable_set_enable_undo(GTK_EDITABLE(w), FALSE);
+  gtk_editable_set_text(GTK_EDITABLE(w), text);
+  gtk_editable_set_enable_undo(GTK_EDITABLE(w), TRUE);
+}
+
+void
+spin_button_set_activates_default(GtkWidget *w)
+{
+  GtkEditable *editable;
+  editable = gtk_editable_get_delegate(GTK_EDITABLE(w));
+  gtk_text_set_activates_default(GTK_TEXT(editable), TRUE);
+}
+
+void
+widget_set_parent(GtkWidget *widget, GtkWidget *parent) {
+  gtk_widget_set_parent(widget, parent);
+  g_signal_connect_swapped(parent, "destroy", G_CALLBACK(gtk_widget_unparent), widget);
+}
+
+gboolean
+gtk_true(void)
+{
+  return TRUE;
+}
+
+double
+scrollbar_get_value(GtkWidget *w)
+{
+  GtkAdjustment *adj;
+  adj = gtk_scrollbar_get_adjustment(GTK_SCROLLBAR(w));
+  return gtk_adjustment_get_value(adj);
+}
+
+void
+scrollbar_set_value(GtkWidget *w, double val)
+{
+  GtkAdjustment *adj;
+  adj = gtk_scrollbar_get_adjustment(GTK_SCROLLBAR(w));
+  gtk_adjustment_set_value(adj, val);
+}
+
+void
+scrollbar_set_range(GtkWidget *w, double min, double max)
+{
+  GtkAdjustment *adj;
+  adj = gtk_scrollbar_get_adjustment(GTK_SCROLLBAR(w));
+  gtk_adjustment_set_lower(adj, min);
+  gtk_adjustment_set_upper(adj, max);
+}
+
+double
+scrollbar_get_max(GtkWidget *w)
+{
+  GtkAdjustment *adj;
+  adj = gtk_scrollbar_get_adjustment(GTK_SCROLLBAR(w));
+  return gtk_adjustment_get_upper(adj);
+}
+
+void
+scrollbar_set_increment(GtkWidget *w, double step, double page)
+{
+  GtkAdjustment *adj;
+  adj = gtk_scrollbar_get_adjustment(GTK_SCROLLBAR(w));
+  gtk_adjustment_set_step_increment(adj, step);
+  gtk_adjustment_set_page_increment(adj, step);
 }
 
 void
@@ -37,6 +117,15 @@ set_widget_margin(GtkWidget *w, int margin_pos)
   }
 }
 
+void
+set_widget_margin_all(GtkWidget *w, int margin)
+{
+  gtk_widget_set_margin_bottom(w, margin);
+  gtk_widget_set_margin_end(w, margin);
+  gtk_widget_set_margin_start(w, margin);
+  gtk_widget_set_margin_top(w, margin);
+}
+
 void
 set_scale_mark(GtkWidget *scale, GtkPositionType pos, int start, int inc)
 {
@@ -139,7 +228,7 @@ set_widget_visibility_with_label(GtkWidget *w, gboolean state)
 }
 
 GtkWidget *
-add_widget_to_table_sub(GtkWidget *table, GtkWidget *w, char *title, int expand, int col, int width, int col_max, int n)
+add_widget_to_table_sub(GtkWidget *table, GtkWidget *w, const char *title, int expand, int col, int width, int col_max, int n)
 {
   GtkWidget *label;
 
@@ -149,7 +238,7 @@ add_widget_to_table_sub(GtkWidget *table, GtkWidget *w, char *title, int expand,
     label = gtk_label_new_with_mnemonic(title);
     gtk_label_set_mnemonic_widget(GTK_LABEL(label), w);
     gtk_widget_set_halign(label, GTK_ALIGN_START);
-    g_object_set(label, "margin", GINT_TO_POINTER(4), NULL);
+    set_widget_margin_all(label, 4);
     gtk_grid_attach(GTK_GRID(table), label, col, n, 1, 1);
     col++;
   }
@@ -161,7 +250,7 @@ add_widget_to_table_sub(GtkWidget *table, GtkWidget *w, char *title, int expand,
     } else {
       gtk_widget_set_halign(w, GTK_ALIGN_START);
     }
-    g_object_set(w, "margin", GINT_TO_POINTER(4), NULL);
+    set_widget_margin_all(w, 4);
     gtk_grid_attach(GTK_GRID(table), w, col, n, width, 1);
   }
 
@@ -169,7 +258,7 @@ add_widget_to_table_sub(GtkWidget *table, GtkWidget *w, char *title, int expand,
 }
 
 GtkWidget *
-add_widget_to_table(GtkWidget *table, GtkWidget *w, char *title, int expand, int n)
+add_widget_to_table(GtkWidget *table, GtkWidget *w, const char *title, int expand, int n)
 {
   return add_widget_to_table_sub(table, w, title, expand, 0, (title) ? 1 : 2, 2, n);
 }
@@ -182,9 +271,9 @@ add_copy_button_to_box(GtkWidget *parent_box, GCallback cb, gpointer d, char *ob
   hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
   w = gtk_button_new_with_mnemonic(_("_Copy Settings"));
   g_signal_connect(w, "map", G_CALLBACK(set_sensitivity_by_check_instance), obj_name);
-  g_signal_connect(w, "clicked", cb, d);
-  gtk_box_pack_start(GTK_BOX(hbox), w, FALSE, FALSE, 4);
-  gtk_box_pack_start(GTK_BOX(parent_box), hbox, FALSE, FALSE, 4);
+  g_signal_connect_swapped(w, "clicked", cb, d);
+  gtk_box_append(GTK_BOX(hbox), w);
+  gtk_box_append(GTK_BOX(parent_box), hbox);
 
   return hbox;
 }
@@ -197,8 +286,8 @@ item_setup(GtkWidget *box, GtkWidget *w, char *title, gboolean expand)
   hbox = gtk_grid_new();
   label = gtk_label_new_with_mnemonic(title);
   gtk_label_set_mnemonic_widget(GTK_LABEL(label), w);
-  g_object_set(label, "margin", GINT_TO_POINTER(2), NULL);
-  g_object_set(w, "margin", GINT_TO_POINTER(2), NULL);
+  set_widget_margin_all(label, 2);
+  set_widget_margin_all(w, 2);
   if (expand) {
     gtk_widget_set_hexpand(w, TRUE);
     gtk_widget_set_halign(w, GTK_ALIGN_FILL);
@@ -207,17 +296,15 @@ item_setup(GtkWidget *box, GtkWidget *w, char *title, gboolean expand)
   }
   gtk_grid_attach(GTK_GRID(hbox), label, 0, 0, 1, 1);
   gtk_grid_attach(GTK_GRID(hbox), w, 1, 0, 1, 1);
-  gtk_box_pack_start(GTK_BOX(box), hbox, expand, expand, 4);
+  gtk_box_append(GTK_BOX(box), hbox);
 
   return label;
 }
 
-int
-entry_set_filename(GtkWidget *w, char *filename)
+void
+entry_set_filename(GtkWidget *w, const char *filename)
 {
-  gtk_entry_set_text(GTK_ENTRY(w), filename);
-
-  return 0;
+  editable_set_init_text(w, filename);
 }
 
 char *
@@ -225,7 +312,7 @@ entry_get_filename(GtkWidget *w)
 {
   const char *utf8filename;
 
-  utf8filename = gtk_entry_get_text(GTK_ENTRY(w));
+  utf8filename = gtk_editable_get_text(GTK_EDITABLE(w));
   if (utf8filename == NULL) {
     return NULL;
   }
@@ -245,12 +332,24 @@ get_parent_window(GtkWidget *w)
   return (ptr) ? ptr : TopLevel;
 }
 
+static void
+entry_icon_file_select_response(char *file, gpointer user_data)
+{
+  GtkWidget *w;
+  w = GTK_WIDGET(user_data);
+  if (file) {
+    entry_set_filename(w, file);
+    g_free(file);
+  }
+}
 
 static void
-entry_icon_file_select(GtkEntry *w, GtkEntryIconPosition icon_pos, GdkEvent *event, gpointer user_data)
+entry_icon_file_select(GtkEntry *w, GtkEntryIconPosition icon_pos, gpointer user_data)
 {
   struct objlist *obj;
-  char *file, *ext;
+  char *ext;
+  const char *str;
+  int chd;
 
   obj = (struct objlist *) user_data;
   if (obj == NULL)
@@ -261,12 +360,9 @@ entry_icon_file_select(GtkEntry *w, GtkEntryIconPosition icon_pos, GdkEvent *eve
     getobj(obj, "ext", 0, 0, NULL, &ext);
   }
 
-  if (nGetOpenFileName(get_parent_window(GTK_WIDGET(w)), obj->name, ext, NULL,
-		       gtk_entry_get_text(w),
-		       &file, TRUE, Menulocal.changedirectory) == IDOK && file) {
-    entry_set_filename(GTK_WIDGET(w), file);
-    g_free(file);
-  }
+  chd = Menulocal.changedirectory;
+  str = gtk_editable_get_text(GTK_EDITABLE(w));
+  nGetOpenFileName(get_parent_window(GTK_WIDGET(w)), obj->name, ext, NULL, str, chd, entry_icon_file_select_response, w);
 }
 
 GtkWidget *
@@ -289,11 +385,11 @@ create_file_entry(struct objlist *obj)
 }
 
 static void
-direction_icon_released(GtkEntry *entry, GtkEntryIconPosition pos, GdkEvent *event, gpointer user_data)
+direction_icon_released(GtkSpinButton *entry, GtkEntryIconPosition pos, GdkEvent *event, gpointer user_data)
 {
   int angle, val;
 
-  angle = gtk_spin_button_get_value(GTK_SPIN_BUTTON(entry));
+  angle = gtk_spin_button_get_value(entry);
   val = angle % 360;
   val += (val < 0) ? 360 : 0;
 
@@ -323,20 +419,42 @@ direction_icon_released(GtkEntry *entry, GtkEntryIconPosition pos, GdkEvent *eve
 
   val += (val < 0) ? 360 : 0;
 
-  gtk_spin_button_set_value(GTK_SPIN_BUTTON(entry), val);
+  gtk_spin_button_set_value(entry, val);
+}
+
+static void
+direction_down(GtkWidget *button, GtkSpinButton *user_data)
+{
+  direction_icon_released(user_data, GTK_ENTRY_ICON_PRIMARY, NULL, NULL);
+}
+
+static void
+direction_up(GtkWidget *button, GtkSpinButton *user_data)
+{
+  direction_icon_released(user_data, GTK_ENTRY_ICON_SECONDARY, NULL, NULL);
 }
 
 GtkWidget *
-create_direction_entry(void)
+create_direction_entry(GtkWidget *table, const char *title, int row)
 {
-  GtkWidget *w;
+  GtkWidget *w, *ubtn, *dbtn, *box;
 
   w = create_spin_entry_type(SPIN_BUTTON_TYPE_ANGLE, FALSE, TRUE);
-  gtk_entry_set_width_chars(GTK_ENTRY(w), NUM_ENTRY_WIDTH);
-  gtk_entry_set_max_width_chars(GTK_ENTRY(w), NUM_ENTRY_WIDTH);
-  gtk_entry_set_icon_from_icon_name(GTK_ENTRY(w), GTK_ENTRY_ICON_SECONDARY, "go-up-symbolic");
-  gtk_entry_set_icon_from_icon_name(GTK_ENTRY(w), GTK_ENTRY_ICON_PRIMARY, "go-down-symbolic");
-  g_signal_connect(w, "icon-release", G_CALLBACK(direction_icon_released), NULL);
+  gtk_editable_set_width_chars(GTK_EDITABLE(w), NUM_ENTRY_WIDTH);
+  gtk_editable_set_max_width_chars(GTK_EDITABLE(w), NUM_ENTRY_WIDTH);
+
+  ubtn = button_new_with_icon("go-up-symbolic", FALSE);
+  g_signal_connect(ubtn, "clicked", G_CALLBACK(direction_up), w);
+
+  dbtn = button_new_with_icon("go-down-symbolic", FALSE);
+  g_signal_connect(dbtn, "clicked", G_CALLBACK(direction_down), w);
+
+  box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
+  gtk_box_append(GTK_BOX(box), w);
+  gtk_box_append(GTK_BOX(box), dbtn);
+  gtk_box_append(GTK_BOX(box), ubtn);
+
+  add_widget_to_table(table, box, title, FALSE, row);
 
   return w;
 }
@@ -348,7 +466,7 @@ create_text_entry(int set_default_size, int set_default_action)
 
   w = gtk_entry_new();
   if (set_default_size) {
-    gtk_entry_set_width_chars(GTK_ENTRY(w), NUM_ENTRY_WIDTH);
+    gtk_editable_set_width_chars(GTK_EDITABLE(w), NUM_ENTRY_WIDTH);
   }
 
   if (set_default_action) {
@@ -461,7 +579,9 @@ spin_entry_set_val(GtkWidget *entry, int ival)
     val = max;
   }
 
+  gtk_editable_set_enable_undo(GTK_EDITABLE(entry), FALSE);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(entry), val);
+  gtk_editable_set_enable_undo(GTK_EDITABLE(entry), TRUE);
 }
 
 int
@@ -483,7 +603,7 @@ spin_change_value_cb(GtkSpinButton *spinbutton, GtkScrollType arg1, gpointer use
   double oval, val;
   int ecode;
 
-  str = gtk_entry_get_text(GTK_ENTRY(spinbutton));
+  str = gtk_editable_get_text(GTK_EDITABLE(spinbutton));
   if (str == NULL)
     return 0;
 
@@ -507,7 +627,7 @@ _create_spin_entry(enum SPIN_BUTTON_TYPE type, double min, double max,
   GtkWidget *w;
 
   w = gtk_spin_button_new_with_range(min, max, inc);
-  gtk_entry_set_alignment(GTK_ENTRY(w), 1.0);
+  gtk_editable_set_alignment(GTK_EDITABLE(w), 1.0);
 
   gtk_spin_button_set_increments(GTK_SPIN_BUTTON(w), inc, page);
   gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(w), wrap);
@@ -515,12 +635,12 @@ _create_spin_entry(enum SPIN_BUTTON_TYPE type, double min, double max,
   gtk_spin_button_set_digits(GTK_SPIN_BUTTON(w), (numeric) ? 0 : 2);
 
   if (set_default_size) {
-    gtk_entry_set_width_chars(GTK_ENTRY(w), NUM_ENTRY_WIDTH);
-    gtk_entry_set_max_width_chars(GTK_ENTRY(w), NUM_ENTRY_WIDTH);
+    gtk_editable_set_width_chars(GTK_EDITABLE(w), NUM_ENTRY_WIDTH);
+    gtk_editable_set_max_width_chars(GTK_EDITABLE(w), NUM_ENTRY_WIDTH);
   }
 
   if (set_default_action) {
-    gtk_entry_set_activates_default(GTK_ENTRY(w), TRUE);
+    spin_button_set_activates_default(w);
   }
 
   g_object_set_data(G_OBJECT(w), "user-data", GINT_TO_POINTER(type));
@@ -606,6 +726,12 @@ create_spin_entry_type(enum SPIN_BUTTON_TYPE type,
     inc = 1;
     numeric = TRUE;
     break;
+  case SPIN_BUTTON_TYPE_ALPHA:
+    min = 0;
+    max = 255;
+    inc = 1;
+    numeric = TRUE;
+    break;
   default:
     type = SPIN_BUTTON_TYPE_CUSTOM;
     min = INT_MIN;
@@ -639,12 +765,7 @@ set_custom_palette(GtkWidget *dlg)
     return;
   }
   colors = arraydata(palette);
-  if (n >= 18) {
-    gtk_color_chooser_add_palette(GTK_COLOR_CHOOSER(dlg), GTK_ORIENTATION_HORIZONTAL, 9, n - 9, colors);
-    gtk_color_chooser_add_palette(GTK_COLOR_CHOOSER(dlg), GTK_ORIENTATION_HORIZONTAL, 9, 9, colors + (n - 9));
-  } else {
-    gtk_color_chooser_add_palette(GTK_COLOR_CHOOSER(dlg), GTK_ORIENTATION_HORIZONTAL, 9, n - 9, colors);
-  }
+  gtk_color_chooser_add_palette(GTK_COLOR_CHOOSER(dlg), GTK_ORIENTATION_HORIZONTAL, 9, n, colors);
 }
 
 static void
@@ -665,45 +786,19 @@ show_color_sel(GtkWidget *w, gpointer user_data)
 void
 add_default_color(struct narray *palette)
 {
-  const gchar *default_colors[9][3] = {
-    { "#ef2929", "#cc0000", "#a40000" }, /* Scarlet Red */
-    { "#fcaf3e", "#f57900", "#ce5c00" }, /* Orange */
-    { "#fce94f", "#edd400", "#c4a000" }, /* Butter */
-    { "#8ae234", "#73d216", "#4e9a06" }, /* Chameleon */
-    { "#729fcf", "#3465a4", "#204a87" }, /* Sky Blue */
-    { "#ad7fa8", "#75507b", "#5c3566" }, /* Plum */
-    { "#e9b96e", "#c17d11", "#8f5902" }, /* Chocolate */
-    { "#888a85", "#555753", "#2e3436" }, /* Aluminum 1 */
-    { "#eeeeec", "#d3d7cf", "#babdb6" }  /* Aluminum 2 */
+  const char * const default_colors[9*5] = {
+    "#99c1f1", "#8ff0a4", "#f9f06b", "#ffbe6f", "#f66151", "#dc8add", "#cdab8f", "#ffffff", "#77767b",
+    "#62a0ea", "#57e389", "#f8e45c", "#ffa348", "#ed333b", "#c061cb", "#b5835a", "#f6f5f4", "#5e5c64",
+    "#3584e4", "#33d17a", "#f6d32d", "#ff7800", "#e01b24", "#9141ac", "#986a44", "#deddda", "#3d3846",
+    "#1c71d8", "#2ec27e", "#f5c211", "#e66100", "#c01c28", "#813d9c", "#865e3c", "#c0bfbc", "#241f31",
+    "#1a5fb4", "#26a269", "#e5a50a", "#c64600", "#a51d2d", "#613583", "#63452c", "#9a9996", "#000000",
+    /* Blue     Green      Yellow     Orange     Red        Purple     Brown      Light      Dark */
   };
   GdkRGBA color;
-  int i, j;
-  for (j = 0; j < 3; j++) {
-    for (i = 0; i < 9; i++) {
-      gdk_rgba_parse(&color, default_colors[i][j]);
-      arrayadd(palette, &color);
-    }
-  }
-}
-
-void
-add_default_gray(struct narray *palette)
-{
-  const gchar *default_grays[9] = {
-    "#000000", /* black */
-    "#2e3436", /* very dark gray */
-    "#555753", /* darker gray */
-    "#888a85", /* dark gray */
-    "#babdb6", /* medium gray */
-    "#d3d7cf", /* light gray */
-    "#eeeeec", /* lighter gray */
-    "#f3f3f3", /* very light gray */
-    "#ffffff"  /* white */
-  };
-  GdkRGBA color;
-  int i;
-  for (i = 0; i < 9; i++) {
-    gdk_rgba_parse(&color, default_grays[i]);
+  int i, n;
+  n = sizeof(default_colors) / sizeof(*default_colors);
+  for (i = 0; i < n; i++) {
+    gdk_rgba_parse(&color, default_colors[i]);
     arrayadd(palette, &color);
   }
 }
@@ -718,37 +813,39 @@ set_default_palette(GtkWidget *cc)
   add_default_color(&palette);
   n = arraynum(&palette);
   gtk_color_chooser_add_palette(GTK_COLOR_CHOOSER(cc), GTK_ORIENTATION_HORIZONTAL, 9, n, arraydata(&palette));
-
-  arrayclear(&palette);
-  add_default_gray(&palette);
-  n = arraynum(&palette);
-  gtk_color_chooser_add_palette(GTK_COLOR_CHOOSER(cc), GTK_ORIENTATION_HORIZONTAL, n, n, arraydata(&palette));
   arraydel(&palette);
 }
 
 #define CUSTOM_PALETTE_KEY "custom_palette"
 
-static gboolean
-show_color_dialog(GtkButton *btn, GdkEvent *event, gpointer user_data)
+static void
+show_color_dialog(GtkGestureClick *gesture, gint n_press, gdouble x, gdouble y, gpointer user_data)
 {
+  GtkColorChooser *btn;
+  btn = GTK_COLOR_CHOOSER(user_data);
   gtk_color_chooser_add_palette(GTK_COLOR_CHOOSER(btn), GTK_ORIENTATION_HORIZONTAL, 0, 0, NULL);
   if (Menulocal.use_custom_palette) {
     set_custom_palette(GTK_WIDGET(btn));
   } else {
     set_default_palette(GTK_WIDGET(btn));
   }
-  return FALSE;
 }
 
 GtkWidget *
 create_color_button(GtkWidget *win)
 {
+  GtkGesture *gesture;
   GtkWidget *w;
 
   w = gtk_color_button_new();
+  gtk_color_chooser_set_use_alpha(GTK_COLOR_CHOOSER(w), FALSE);
   g_object_set_data(G_OBJECT(w), CUSTOM_PALETTE_KEY, GINT_TO_POINTER(0));
   g_signal_connect(w, "color-set", G_CALLBACK(show_color_sel), win);
-  g_signal_connect(w, "button-press-event", G_CALLBACK(show_color_dialog), win);
+  gesture = gtk_gesture_click_new();
+  gtk_widget_add_controller(w, GTK_EVENT_CONTROLLER(gesture));
+
+  gtk_gesture_single_set_button(GTK_GESTURE_SINGLE(gesture), 0);
+  g_signal_connect(gesture, "pressed", G_CALLBACK(show_color_dialog), w);
 
   return w;
 }
@@ -757,7 +854,6 @@ void
 set_widget_font(GtkWidget *w, const char *font)
 {
   GtkCssProvider *css_provider;
-  GError *error;
   char *css_str;
   PangoFontDescription *desc;
   const char *family, *style_str, *unit;
@@ -858,14 +954,11 @@ set_widget_font(GtkWidget *w, const char *font)
     return;
   }
 
-  error = NULL;
-  gtk_css_provider_load_from_data(css_provider, css_str, -1, &error);
+  gtk_css_provider_load_from_string(css_provider, css_str);
   g_free(css_str);
-  if (error == NULL) {
-    gtk_style_context_add_provider(gtk_widget_get_style_context(w),
-				   GTK_STYLE_PROVIDER(css_provider),
-				   GTK_STYLE_PROVIDER_PRIORITY_USER);
-  }
+  gtk_style_context_add_provider(gtk_widget_get_style_context(w),
+				 GTK_STYLE_PROVIDER(css_provider),
+				 GTK_STYLE_PROVIDER_PRIORITY_USER);
 }
 
 GtkWidget *
@@ -885,9 +978,9 @@ create_text_view_with_line_number(GtkWidget **v)
   gtk_source_buffer_set_highlight_syntax(GTK_SOURCE_BUFFER(buffer), FALSE);
   gtk_source_view_set_show_line_numbers(GTK_SOURCE_VIEW(source_view), TRUE);
 
-  swin = gtk_scrolled_window_new(NULL, NULL);
+  swin = gtk_scrolled_window_new();
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-  gtk_container_add(GTK_CONTAINER(swin), source_view);
+  gtk_scrolled_window_set_child(GTK_SCROLLED_WINDOW(swin), source_view);
   *v = source_view;
   return swin;
 }
@@ -907,157 +1000,6 @@ text_view_with_line_number_set_font(GtkWidget *view, const gchar *font)
   set_widget_font(view, font);
 }
 
-enum SELECT_OBJ_COLOR_RESULT
-select_obj_color(struct objlist *obj, int id, enum OBJ_FIELD_COLOR_TYPE type)
-{
-  GtkWidget *dlg;
-  int r, g, b, a, rr ,gg, bb, aa, response, modified, undo;
-  GdkRGBA color;
-  char *title;
-
-  switch (type) {
-  case OBJ_FIELD_COLOR_TYPE_STROKE:
-    title = _("Stroke Color");
-    getobj(obj, "stroke_R", id, 0, NULL, &r);
-    getobj(obj, "stroke_G", id, 0, NULL, &g);
-    getobj(obj, "stroke_B", id, 0, NULL, &b);
-    getobj(obj, "stroke_A", id, 0, NULL, &a);
-    break;
-  case OBJ_FIELD_COLOR_TYPE_FILL:
-    title = _("Fill Color");
-    getobj(obj, "fill_R", id, 0, NULL, &r);
-    getobj(obj, "fill_G", id, 0, NULL, &g);
-    getobj(obj, "fill_B", id, 0, NULL, &b);
-    getobj(obj, "fill_A", id, 0, NULL, &a);
-    break;
-  case OBJ_FIELD_COLOR_TYPE_0:
-    title = _("Color");
-    getobj(obj, "R", id, 0, NULL, &r);
-    getobj(obj, "G", id, 0, NULL, &g);
-    getobj(obj, "B", id, 0, NULL, &b);
-    getobj(obj, "A", id, 0, NULL, &a);
-    break;
-  case OBJ_FIELD_COLOR_TYPE_1:
-    title = _("Color 1");
-    getobj(obj, "R", id, 0, NULL, &r);
-    getobj(obj, "G", id, 0, NULL, &g);
-    getobj(obj, "B", id, 0, NULL, &b);
-    getobj(obj, "A", id, 0, NULL, &a);
-    break;
-  case OBJ_FIELD_COLOR_TYPE_2:
-    title = _("Color 2");
-    getobj(obj, "R2", id, 0, NULL, &r);
-    getobj(obj, "G2", id, 0, NULL, &g);
-    getobj(obj, "B2", id, 0, NULL, &b);
-    getobj(obj, "A2", id, 0, NULL, &a);
-    break;
-  case OBJ_FIELD_COLOR_TYPE_AXIS_BASE:
-    title = _("Axis baseline color");
-    getobj(obj, "R", id, 0, NULL, &r);
-    getobj(obj, "G", id, 0, NULL, &g);
-    getobj(obj, "B", id, 0, NULL, &b);
-    getobj(obj, "A", id, 0, NULL, &a);
-    break;
-  case OBJ_FIELD_COLOR_TYPE_AXIS_GAUGE:
-    title = _("Axis gauge color");
-    getobj(obj, "gauge_R", id, 0, NULL, &r);
-    getobj(obj, "gauge_G", id, 0, NULL, &g);
-    getobj(obj, "gauge_B", id, 0, NULL, &b);
-    getobj(obj, "gauge_A", id, 0, NULL, &a);
-    break;
-  case OBJ_FIELD_COLOR_TYPE_AXIS_NUM:
-    title = _("Axis numbering color");
-    getobj(obj, "num_R", id, 0, NULL, &r);
-    getobj(obj, "num_G", id, 0, NULL, &g);
-    getobj(obj, "num_B", id, 0, NULL, &b);
-    getobj(obj, "num_A", id, 0, NULL, &a);
-    break;
-  default:
-    return SELECT_OBJ_COLOR_ERROR;
-  }
-
-  if (! Menulocal.use_opacity) {
-    a = 255;
-  }
-
-  color.red = r / 255.0;
-  color.green = g / 255.0;
-  color.blue = b / 255.0;
-  color.alpha = a / 255.0;
-
-  dlg = gtk_color_chooser_dialog_new(title, GTK_WINDOW(TopLevel));
-  if (Menulocal.use_custom_palette) {
-    set_custom_palette(dlg);
-  }
-  gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(dlg), &color);
-  gtk_color_chooser_set_use_alpha(GTK_COLOR_CHOOSER(dlg), Menulocal.use_opacity);
-  response = ndialog_run(dlg);
-  gtk_color_chooser_get_rgba(GTK_COLOR_CHOOSER(dlg), &color);
-
-  gtk_widget_destroy(dlg);
-
-  if (response != GTK_RESPONSE_OK) {
-    return SELECT_OBJ_COLOR_CANCEL;
-  }
-
-  rr = nround(color.red * 255);
-  gg = nround(color.green * 255);
-  bb = nround(color.blue * 255);
-  aa = nround(color.alpha * 255);
-
-  undo = menu_save_undo_single(UNDO_TYPE_EDIT, obj->name);
-  switch (type) {
-  case OBJ_FIELD_COLOR_TYPE_STROKE:
-    putobj(obj, "stroke_R", id, &rr);
-    putobj(obj, "stroke_G", id, &gg);
-    putobj(obj, "stroke_B", id, &bb);
-    putobj(obj, "stroke_A", id, &aa);
-    break;
-  case OBJ_FIELD_COLOR_TYPE_FILL:
-    putobj(obj, "fill_R", id, &rr);
-    putobj(obj, "fill_G", id, &gg);
-    putobj(obj, "fill_B", id, &bb);
-    putobj(obj, "fill_A", id, &aa);
-    break;
-  case OBJ_FIELD_COLOR_TYPE_0:
-  case OBJ_FIELD_COLOR_TYPE_1:
-  case OBJ_FIELD_COLOR_TYPE_AXIS_BASE:
-    putobj(obj, "R", id, &rr);
-    putobj(obj, "G", id, &gg);
-    putobj(obj, "B", id, &bb);
-    putobj(obj, "A", id, &aa);
-    break;
-  case OBJ_FIELD_COLOR_TYPE_2:
-    putobj(obj, "R2", id, &rr);
-    putobj(obj, "G2", id, &gg);
-    putobj(obj, "B2", id, &bb);
-    putobj(obj, "A2", id, &aa);
-    break;
-  case OBJ_FIELD_COLOR_TYPE_AXIS_GAUGE:
-    putobj(obj, "gauge_R", id, &rr);
-    putobj(obj, "gauge_G", id, &gg);
-    putobj(obj, "gauge_B", id, &bb);
-    putobj(obj, "gauge_A", id, &aa);
-    break;
-  case OBJ_FIELD_COLOR_TYPE_AXIS_NUM:
-    putobj(obj, "num_R", id, &rr);
-    putobj(obj, "num_G", id, &gg);
-    putobj(obj, "num_B", id, &bb);
-    putobj(obj, "num_A", id, &aa);
-    break;
-  default:
-    return SELECT_OBJ_COLOR_ERROR;
-  }
-
-  if (rr == r && gg == g && bb == b && aa == a) {
-    modified = SELECT_OBJ_COLOR_SAME;
-    menu_delete_undo(undo);
-  } else {
-    modified = SELECT_OBJ_COLOR_DIFFERENT;
-  }
-  return modified;
-}
-
 gchar *
 get_text_from_buffer(GtkTextBuffer *buffer)
 {
@@ -1085,7 +1027,7 @@ GtkWidget *
 add_button(GtkWidget *grid, int row, int col, const char *icon, const char *tooltip, GCallback proc, gpointer data)
 {
   GtkWidget *w;
-  w = gtk_button_new_from_icon_name(icon, GTK_ICON_SIZE_BUTTON);
+  w = button_new_with_icon(icon, FALSE);
   add_button_common(w, grid, row, col, tooltip, proc, data);
   return w;
 }
@@ -1093,10 +1035,8 @@ add_button(GtkWidget *grid, int row, int col, const char *icon, const char *tool
 GtkWidget *
 add_toggle_button(GtkWidget *grid, int row, int col, const char *icon_name, const char *tooltip, GCallback proc, gpointer data)
 {
-  GtkWidget *w, *icon;
-  w = gtk_toggle_button_new();
-  icon = gtk_image_new_from_icon_name(icon_name, GTK_ICON_SIZE_BUTTON);
-  gtk_button_set_image(GTK_BUTTON(w), icon);
+  GtkWidget *w;
+  w = button_new_with_icon(icon_name, TRUE);
   add_button_common(w, grid, row, col, tooltip, NULL, NULL);
   if (proc) {
     g_signal_connect(w, "toggled", proc, data);
@@ -1104,16 +1044,27 @@ add_toggle_button(GtkWidget *grid, int row, int col, const char *icon_name, cons
   return w;
 }
 
-void
+GtkEventController *
 add_event_key(GtkWidget *widget, GCallback press_proc, GCallback release_proc, gpointer user_data)
 {
   GtkEventController *ev;
 
-  ev = gtk_event_controller_key_new(widget);
+  ev = gtk_event_controller_key_new();
+  gtk_widget_add_controller(widget, ev);
   if (press_proc) {
     g_signal_connect(ev, "key-pressed", press_proc, user_data);
   }
   if (release_proc) {
     g_signal_connect(ev, "key-released", release_proc, user_data);
   }
+  return ev;
+}
+
+void
+label_set_italic_text (GtkWidget *label, const char *text)
+{
+  char *str;
+  str = g_markup_printf_escaped ("<i>%s</i>", text);
+  gtk_label_set_markup (GTK_LABEL (label), str);
+  g_free (str);
 }
