@@ -37,11 +37,6 @@
 #include <locale.h>
 #include <signal.h>
 
-#if OSX
-#include <gtkosxapplication.h>
-static GtkosxApplication *GtkMacIntegration = NULL;
-#endif
-
 char *DOCDIR, *NDATADIR, *ADDINDIR, *LIBDIR, *PLUGINDIR, *CONFDIR, *NLOCALEDIR, *BINDIR;
 
 #include "dir_defs.h"
@@ -77,6 +72,7 @@ static char *systemname, *locale;
 static int consolefdout, consolefdin, ConsoleAc = FALSE;
 static int consolecol = 80, consolerow = 25;
 static int Initialized = FALSE;
+static GThread *MainThread;
 
 void *addobjectroot(void);
 void *addint(void);
@@ -119,13 +115,14 @@ void *addmenu(void);
 void *adddialog(void);
 #if WINDOWS
 void *addgra2emf(void);
+void *addgra2emfplus(void);
 #endif	/* WINDOWS */
 
 void resizeconsole(int col, int row);
 
 // XtAppContext Application=NULL;
 char *AppName = "Ngraph", *AppClass = "Ngraph", *Home;
-#if ! OSX
+#if ! OSX && ! WINDOWS
 static const char *Snap = NULL;
 #endif
 
@@ -148,6 +145,7 @@ char *Documenter[] = {
 
 
 static int OpenDisplay = FALSE;
+static GMainLoop *MainLoop = NULL;
 
 static void * ( * obj_add_func_ary[]) (void) = {
   addshell,
@@ -188,6 +186,7 @@ static void * ( * obj_add_func_ary[]) (void) = {
   adddialog,
 #if WINDOWS
   addgra2emf,
+  addgra2emfplus,
 #endif
 };
 
@@ -197,6 +196,24 @@ OpenApplication(void)
   return OpenDisplay;
 }
 
+GMainLoop *
+main_loop(void)
+{
+  return MainLoop;
+}
+
+int
+main_loop_is_running(void)
+{
+  return g_main_loop_is_running(MainLoop);
+}
+
+void
+main_loop_run(void)
+{
+  g_main_loop_run(main_loop());
+}
+
 int
 putconsole(const char *s)
 {
@@ -757,35 +774,22 @@ get_libexec_dir(const char *app_path)
 static int
 set_dir_defs(char *app)
 {
-  const char *app_contents;
-  char *app_path, *bin_path;
-
-  app_contents = g_getenv("NGRAPH_APP_CONTENTS");
-  app_path = gtkosx_application_get_resource_path();
-  if (app_contents) {
-    bin_path = gtkosx_application_get_executable_path();
-    LIBDIR = g_path_get_dirname(bin_path);
-    g_free(bin_path);
-  } else {
-    bin_path = g_path_get_dirname(app_path);
-    g_free(app_path);
-    app_path = bin_path;
-    LIBDIR = get_libexec_dir(app_path);
-  }
-
-  if (app_path == NULL) {
-    return 1;
-  }
-
-  BINDIR = NULL;
-  DOCDIR = g_strdup_printf("%s%c%s", app_path, DIRSEP, "share/doc/ngraph-gtk");
-  PLUGINDIR = g_strdup_printf("%s%c%s", app_path, DIRSEP, "lib/ngraph-gtk/plugins");
-  NDATADIR = g_strdup_printf("%s%c%s", app_path, DIRSEP, "share/ngraph-gtk");
-  ADDINDIR = g_strdup_printf("%s%c%s", app_path, DIRSEP, "share/ngraph-gtk/addin");
-  CONFDIR = g_strdup_printf("%s%c%s", app_path, DIRSEP, "etc/ngraph-gtk");
-  NLOCALEDIR = g_strdup_printf("%s%c%s", app_path, DIRSEP, "share/locale");
+  char  *app_path, *bin_path, *pkg_path;
 
+  app_path = g_find_program_in_path (app);
+  bin_path = g_path_get_dirname(app_path);
   g_free(app_path);
+  pkg_path = g_path_get_dirname(bin_path);
+  g_free(bin_path);
+  LIBDIR = get_libexec_dir(pkg_path);
+  g_free(pkg_path);
+  BINDIR = NULL;
+  DOCDIR = g_strdup (_DOCDIR);
+  PLUGINDIR = g_strdup (_PLUGINDIR);
+  NDATADIR = g_strdup (_NDATADIR);
+  ADDINDIR = g_strdup (_ADDINDIR);
+  CONFDIR = g_strdup (_CONFDIR);
+  NLOCALEDIR = g_strdup (LOCALEDIR);
 
   return 0;
 }
@@ -850,51 +854,43 @@ static void
 load_css(void)
 {
   GtkCssProvider *css_provider;
+  GdkDisplay *display;
 
   css_provider = gtk_css_provider_new();
   gtk_css_provider_load_from_resource(css_provider, CSS_PATH);
-  gtk_style_context_add_provider_for_screen(gdk_screen_get_default(), GTK_STYLE_PROVIDER(css_provider), GTK_STYLE_PROVIDER_PRIORITY_USER);
+  display = gdk_display_get_default();
+  if (display) {
+    gtk_style_context_add_provider_for_display(display, GTK_STYLE_PROVIDER(css_provider), GTK_STYLE_PROVIDER_PRIORITY_USER);
+  }
 }
 
 void
 n_application_ready(void)
 {
-#if OSX
-  if (GtkMacIntegration) {
-    gtkosx_application_ready(GtkMacIntegration);
-  }
-#endif
 }
 
-#if OSX
-static gboolean
-osx_open_file(GtkosxApplication *app, gchar *path, gpointer user_data)
+GtkApplication *
+n_get_gtk_application(void)
 {
-  char *dir;
-  if (path == NULL) {
-    return TRUE;
-  }
-  dir = getdirname(path);
-  if (dir) {
-    nchdir(dir);
-    g_free(dir);
-  }
-  LoadNgpFile(path, FALSE, "-f");
-  return TRUE;
+  return GtkApp;
 }
 
 static void
-create_app_menu(GtkApplication *app)
+set_prgname(int argc, char **argv)
 {
-  GtkBuilder *builder;
-  GObject *app_menu;
-  /* only for remove "Settings" menu item */
-  builder = gtk_builder_new_from_resource(RESOURCE_PATH "/gtk/menus-appmenu.ui");
-  app_menu = gtk_builder_get_object(builder, "app-menu");
-  gtk_application_set_app_menu(app, G_MENU_MODEL(app_menu));
-  g_object_unref(builder);
+  if (g_get_prgname() == NULL && argc > 0) {
+    gchar *prgname;
+    prgname = g_path_get_basename(argv[0]);
+    g_set_prgname(prgname);
+    g_free (prgname);
+  }
+}
+
+int
+is_main_thread(void)
+{
+  return MainThread == g_thread_self();
 }
-#endif
 
 int
 n_initialize(int *argc, char ***argv)
@@ -914,6 +910,8 @@ n_initialize(int *argc, char ***argv)
     return 0;
   }
 
+  MainThread = g_thread_self();
+
 #if EOF == -1
   char_type_buf_init();
 #endif
@@ -931,20 +929,15 @@ n_initialize(int *argc, char ***argv)
   textdomain(PACKAGE);
 #endif	/* HAVE_GETTEXT */
 
-  OpenDisplay = gtk_init_check(argc, argv);
+  MainLoop = g_main_loop_new (NULL, FALSE);
+  OpenDisplay = gtk_init_check();
   if (OpenDisplay) {
     GtkApp = gtk_application_new(APPLICATION_ID, G_APPLICATION_NON_UNIQUE);
     g_application_register(G_APPLICATION(GtkApp), NULL, NULL);
     setup_actions(GtkApp);
   }
+  set_prgname(*argc, *argv);  /* this is necessary to use GtkRecentManager */
   g_set_application_name(AppName);
-#if OSX
-  GtkMacIntegration = gtkosx_application_get();
-  g_signal_connect(GtkMacIntegration, "NSApplicationOpenFile", G_CALLBACK(osx_open_file), NULL);
-  if (OpenDisplay) {
-    create_app_menu(GtkApp);
-  }
-#endif
 
   if (init_cmd_tbl()) {
     exit(1);
@@ -1421,10 +1414,10 @@ my_completion_function(const char *text, int state, char **func(const char *))
 {
   static char **list = (char **) NULL;
   static int list_index = 0;
-  static int first_char_loc;
 
   /* If we don't have any state, make some. */
   if (!state) {
+    static int first_char_loc;
     if (list)
       g_free(list);
 
@@ -1585,12 +1578,12 @@ get_file_list(const char *path, int type, int mode)
   const char *ent;
   GStatBuf statbuf;
   struct mylist *list = NULL, *list_next = list;
-  char *full_path_name;
 
   if ((dir = g_dir_open(path, 0, NULL)) == NULL) {
     return NULL;
   }
   while ((ent = g_dir_read_name(dir)) != NULL) {
+    char *full_path_name;
     full_path_name = g_strdup_printf("%s/%s", path, ent);
     if (full_path_name == NULL) {
       if (list != NULL) {
