@@ -22,5 +22,11 @@ int n_initialize(int *argc, char ***argv);
 void n_save_shell_history(void);
 void n_finalize(void);
 void n_application_ready(void);
+GtkApplication *n_get_gtk_application(void);
+int is_main_thread(void);
+
+GMainLoop *main_loop(void);
+int main_loop_is_running(void);
+void main_loop_run(void);
 
 #endif
