@@ -68,17 +68,6 @@ static char *gtkerrorlist[] = {
 
 #define ERRNUM (sizeof(gtkerrorlist)/sizeof(*gtkerrorlist))
 
-struct action_type
-{
-  enum {
-    ACTION_TYPE_NONE,
-    ACTION_TYPE_KEY,
-    ACTION_TYPE_BUTTON,
-    ACTION_TYPE_SCROLL,
-  } type;
-  int val;
-};
-
 struct gtklocal
 {
   struct objlist *obj;
@@ -86,23 +75,20 @@ struct gtklocal
   N_VALUE *inst;
   GtkWidget *mainwin, *View;
   cairo_surface_t *surface;
-  GdkWindow *window;
   char *title;
   int redraw, fit, frame;
   unsigned int windpi;
   int PaperWidth, PaperHeight;
-  struct action_type action;
   double bg_r, bg_g, bg_b;
   struct gra2cairo_local *local;
+  int quit_main_loop;
 #if OSX
   int menulock;
 #endif
 };
 
 static void gtkMakeRuler(cairo_t *cr, struct gtklocal *gtklocal);
-static int gtk_evloop(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc,
-		      char **argv);
-static int gtkclose(GtkWidget *widget, GdkEvent  *event, gpointer user_data);
+static int gtkclose(GtkWidget *widget, gpointer user_data);
 static void gtkchangedpi(struct gtklocal *gtklocal);
 static gboolean gtkevpaint(GtkWidget * w, cairo_t * e,
 			   gpointer user_data);
@@ -117,6 +103,7 @@ static int gtkredraw(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc
 static int dot2pixel(struct gtklocal *gtklocal, int r);
 static int gtk_output(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc,
 		      char **argv);
+static void resized(GtkWidget *widget, int w, int h, gpointer user_data);
 
 static gboolean
 gtkevpaint(GtkWidget *w, cairo_t *cr, gpointer user_data)
@@ -141,8 +128,19 @@ gtkevpaint(GtkWidget *w, cairo_t *cr, gpointer user_data)
   return FALSE;
 }
 
+static void
+destroyed(GtkWidget *win, gpointer user_data)
+{
+  struct gtklocal *local;
+  local = (struct gtklocal *) user_data;
+  if (local->quit_main_loop) {
+    g_idle_add_once((GSourceOnceFunc) g_main_loop_quit, main_loop());
+  }
+  local->mainwin = NULL;
+}
+
 static int
-gtkclose(GtkWidget *widget, GdkEvent *event, gpointer user_data)
+gtkclose(GtkWidget *widget, gpointer user_data)
 {
   struct gtklocal *local;
   int i, id;
@@ -159,35 +157,23 @@ gtkclose(GtkWidget *widget, GdkEvent *event, gpointer user_data)
       break;
     }
   }
-  return TRUE;
-}
-
-static gboolean
-ev_key_down(GtkEventControllerKey *controller, guint keyval, guint keycode, GdkModifierType state, gpointer user_data)
-{
-  struct gtklocal *gtklocal;
-
-  gtklocal = (struct gtklocal *) user_data;
-
-  gtklocal->action.type = ACTION_TYPE_KEY;
-  gtklocal->action.val = keyval;
-
-  switch (keyval) {
-  case GDK_KEY_w:
-    if (state & GDK_CONTROL_MASK) {
-      gtkclose(gtklocal->mainwin, NULL, NULL);
-      return TRUE;
-    }
-    return FALSE;
-  }
   return FALSE;
 }
 
-void
+static void
 size_allocate(GtkWidget *widget, GdkRectangle *allocation, gpointer user_data)
+{
+  double w, h;
+  w = allocation->width;
+  h = allocation->height;
+  resized(widget, w, h, user_data);
+}
+
+static void
+resized(GtkWidget *widget, int w, int h, gpointer user_data)
 {
   struct gtklocal *local;
-  double w, h, rh, rw, ratio;
+  double rh, rw, ratio;
   int pw, ph, dpi;
 
   local = (struct gtklocal *) user_data;
@@ -199,9 +185,6 @@ size_allocate(GtkWidget *widget, GdkRectangle *allocation, gpointer user_data)
     return;
   }
 
-  w = allocation->width;
-  h = allocation->height;
-
   pw = local->PaperWidth;
   ph = local->PaperHeight;
 
@@ -209,8 +192,8 @@ size_allocate(GtkWidget *widget, GdkRectangle *allocation, gpointer user_data)
     return;
   }
 
-  rw = w / pw;
-  rh = h / ph;
+  rw = 1.0 * w / pw;
+  rh = 1.0 * h / ph;
 
   ratio = (rh > rw) ? rw : rh;
   dpi = ratio * DPI_MAX;
@@ -226,70 +209,9 @@ size_allocate(GtkWidget *widget, GdkRectangle *allocation, gpointer user_data)
 }
 
 static void
-cursor_moved(GtkEventControllerMotion *controller, gdouble x, gdouble y, gpointer user_data)
+draw_function(GtkDrawingArea* drawing_area, cairo_t* cr, int width, int height, gpointer user_data)
 {
-  struct gtklocal *gtklocal;
-
-  gtklocal = (struct gtklocal *) user_data;
-
-  if (gtklocal->blank_cursor) {
-    gdk_window_set_cursor(gtk_widget_get_window(gtklocal->mainwin), NULL);
-    g_object_unref(gtklocal->blank_cursor);
-    gtklocal->blank_cursor = NULL;
-  }
-}
-
-static void
-button_released(GtkGestureMultiPress *gesture, gint n_press, gdouble x, gdouble y, gpointer user_data)
-{
-  struct gtklocal *gtklocal;
-  guint button;
-
-  gtklocal = (struct gtklocal *) user_data;
-  button = gtk_gesture_single_get_current_button(GTK_GESTURE_SINGLE(gesture));
-
-  gtklocal->action.type = ACTION_TYPE_BUTTON;
-  gtklocal->action.val = button;
-}
-
-static gboolean
-scrolled(GtkWidget *widget, GdkEventScroll *event, gpointer user_data)
-{
-  struct gtklocal *gtklocal;
-
-  gtklocal = (struct gtklocal *) user_data;
-
-  switch (event->direction) {
-  case GDK_SCROLL_UP:
-  case GDK_SCROLL_DOWN:
-  case GDK_SCROLL_LEFT:
-  case GDK_SCROLL_RIGHT:
-    gtklocal->action.type = ACTION_TYPE_SCROLL;
-    gtklocal->action.val = event->direction;
-    break;
-  case GDK_SCROLL_SMOOTH:
-    break;
-  }
-
-  return FALSE;
-}
-
-static void
-add_event_button(GtkWidget *widget, struct gtklocal *gtklocal)
-{
-  GtkGesture *ev;
-  ev = gtk_gesture_multi_press_new(widget);
-  gtk_gesture_single_set_button(GTK_GESTURE_SINGLE(ev), 0);
-  g_signal_connect(ev, "released", G_CALLBACK(button_released), gtklocal);
-}
-
-static void
-add_event_motion(GtkWidget *widget, struct gtklocal *gtklocal)
-{
-  GtkEventController *ev;
-
-  ev = gtk_event_controller_motion_new(widget);
-  g_signal_connect(ev, "motion", G_CALLBACK(cursor_moved), gtklocal);
+  gtkevpaint(GTK_WIDGET(drawing_area), cr, user_data);
 }
 
 static int
@@ -297,8 +219,7 @@ gtkinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
 {
   struct gtklocal *gtklocal;
   struct gra2cairo_local *local;
-  struct objlist *robj;
-  int idn, oid, width, height;
+  int oid, width, height, delete_gra;
   GtkWidget *scrolled_window = NULL;
 
   if (_exeparent(obj, (char *) argv[1], inst, rval, argc, argv))
@@ -315,8 +236,7 @@ gtkinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
   gtklocal->fit = FALSE;
   gtklocal->frame = TRUE;
   gtklocal->blank_cursor = NULL;
-  gtklocal->action.type = ACTION_TYPE_NONE;
-  gtklocal->action.val = 0;
+  gtklocal->quit_main_loop = FALSE;
 
   if (_putobj(obj, "_gtklocal", inst, gtklocal))
     goto errexit;
@@ -359,6 +279,10 @@ gtkinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
   if (_putobj(obj, "dpiy", inst, &(gtklocal->windpi)))
     goto errexit;
 
+  delete_gra = TRUE;
+  if (_putobj(obj, "delete_gra", inst, &delete_gra))
+    goto errexit;
+
   local->pixel_dot_x =
   local->pixel_dot_y = gtklocal->windpi / (DPI_MAX * 1.0);
 
@@ -378,7 +302,7 @@ gtkinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
 
   gtklocal->title = mkobjlist(obj, NULL, oid, NULL, TRUE);
 
-  gtklocal->mainwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+  gtklocal->mainwin = gtk_window_new();
   if (TopLevel) {
     gtk_window_set_modal(GTK_WINDOW(gtklocal->mainwin), TRUE);
     gtk_window_set_transient_for(GTK_WINDOW(gtklocal->mainwin), GTK_WINDOW(TopLevel));
@@ -388,60 +312,37 @@ gtkinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
 #endif
   }
   gtk_window_set_default_size(GTK_WINDOW(gtklocal->mainwin), width, height);
-  g_signal_connect_swapped(gtklocal->mainwin,
-			   "delete-event",
-			   G_CALLBACK(gtkclose), gtklocal->mainwin);
-
-  add_event_key(gtklocal->mainwin, G_CALLBACK(ev_key_down), NULL,  gtklocal);
+  g_signal_connect(gtklocal->mainwin, "close_request", G_CALLBACK(gtkclose), gtklocal);
+  g_signal_connect(gtklocal->mainwin, "destroy", G_CALLBACK(destroyed), gtklocal);
 
   gtk_window_set_title((GtkWindow *) gtklocal->mainwin, gtklocal->title);
 
-  scrolled_window = gtk_scrolled_window_new(NULL, NULL);
+  scrolled_window = gtk_scrolled_window_new();
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-  gtk_container_add(GTK_CONTAINER(gtklocal->mainwin), scrolled_window);
+  gtk_window_set_child(GTK_WINDOW(gtklocal->mainwin), scrolled_window);
 
   gtklocal->View = gtk_drawing_area_new();
   gtk_widget_set_halign(gtklocal->View, GTK_ALIGN_CENTER);
   gtk_widget_set_valign(gtklocal->View, GTK_ALIGN_CENTER);
 
-  g_signal_connect(gtklocal->View, "draw",
-		   G_CALLBACK(gtkevpaint), gtklocal);
+  gtk_drawing_area_set_draw_func(GTK_DRAWING_AREA(gtklocal->View), draw_function, gtklocal, NULL);
+  gtk_widget_set_can_focus(gtklocal->View, TRUE);
+  g_signal_connect(gtklocal->View, "resize", G_CALLBACK(resized), gtklocal);
 
-  g_signal_connect(gtklocal->mainwin, "size-allocate",
-		   G_CALLBACK(size_allocate), gtklocal);
-
-  gtk_container_add(GTK_CONTAINER(scrolled_window), gtklocal->View);
-
-  gtk_widget_show_all(gtklocal->mainwin);
+  gtk_scrolled_window_set_child(GTK_SCROLLED_WINDOW(scrolled_window), gtklocal->View);
 
   gtklocal->surface= NULL;
   gtklocal->redraw = TRUE;
 
-  gtk_widget_add_events(gtklocal->mainwin, GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK);
-  add_event_motion(gtklocal->mainwin, gtklocal);
-  add_event_button(gtklocal->mainwin, gtklocal);
-  g_signal_connect(gtklocal->mainwin, "scroll-event", G_CALLBACK(scrolled), gtklocal);
-
-  if (chkobjfield(obj, "_evloop")) {
-    goto errexit;
-  }
-
-  idn = getobjtblpos(obj, "_evloop", &robj);
-  if (idn == -1) {
-    goto errexit;
-  }
-
-  registerevloop(chkobjectname(obj), "_evloop", robj, idn, inst, gtklocal);
-
   gtkchangedpi(gtklocal);
 
-  gtk_evloop(obj, inst, NULL, argc, argv);
   return 0;
 
 errexit:
   if (gtklocal) {
     if (gtklocal->mainwin) {
-      gtk_widget_destroy(gtklocal->mainwin);
+      gtk_window_destroy(GTK_WINDOW(gtklocal->mainwin));
+      gtklocal->mainwin = NULL;
     }
 
     if (gtklocal->mainwin) {
@@ -460,8 +361,6 @@ static int
 gtkdone(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   struct gtklocal *gtklocal;
-  int idn;
-  struct objlist *robj;
 
   if (_exeparent(obj, argv[1], inst, rval, argc, argv))
     return 1;
@@ -474,21 +373,14 @@ gtkdone(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
   }
 
   if (gtklocal->mainwin != NULL) {
-    gtk_widget_destroy(gtklocal->mainwin);
+    gtk_window_destroy(GTK_WINDOW(gtklocal->mainwin));
     gtklocal->mainwin = NULL;
-    while (gtk_events_pending()) {
-      gtk_main_iteration();
-    }
   }
 
   if (gtklocal->surface) {
     cairo_surface_destroy(gtklocal->surface);
   }
 
-  idn = getobjtblpos(obj, "_evloop", &robj);
-  if (idn != -1)
-    unregisterevloop(robj, idn, inst);
-
   g_free(gtklocal->title);
 
 #if OSX
@@ -513,18 +405,11 @@ clear_pixmap(struct gtklocal *local)
 static void
 redraw_window(struct gtklocal *local)
 {
-  GdkWindow *win;
-
   if (local->View == NULL) {
     return;
   }
 
-  win = gtk_widget_get_window(local->View);
-  if (win == NULL) {
-    return;
-  }
-
-  gdk_window_invalidate_rect(win, NULL, TRUE);
+  gtk_widget_queue_draw(local->View);
 }
 
 static int
@@ -555,10 +440,18 @@ gtkpresent(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **a
   if (_getobj(obj, "_gtklocal", inst, &local))
     return 1;
 
-  reset_event();
-  msleep(100);
+  if (local->mainwin == NULL)
+    return 1;
+
   gtk_window_present(GTK_WINDOW(local->mainwin));
 
+  if (main_loop_is_running()) {
+    local->quit_main_loop = FALSE;
+  } else {
+    local->quit_main_loop = TRUE;
+    main_loop_run();
+  }
+
   return 0;
 }
 
@@ -574,6 +467,9 @@ gtkfullscreen(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char
   if (_getobj(obj, "_gtklocal", inst, &local))
     return 1;
 
+  if (local->mainwin == NULL)
+    return 1;
+
   state = *(int *) argv[2];
 
   if (state) {
@@ -602,15 +498,13 @@ gtkflush(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
 }
 
 static double
-get_color(struct gtklocal *gtklocal, int argc, char **argv)
+get_color(const struct gtklocal *gtklocal, int argc, char **argv)
 {
   int c;
 
   c = abs(*(int *) argv[2]);
 
-  if (c < 0) {
-    c = 0;
-  } else if (c > 255) {
+  if (c > 255) {                /* c >= 0 */
     c = 255;
   }
 
@@ -653,15 +547,6 @@ gtkredraw(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **ar
   return 0;
 }
 
-static int
-gtk_evloop(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
-{
-  while (gtk_events_pending()) {
-    gtk_main_iteration();
-  }
-  return 0;
-}
-
 static int
 dot2pixel(struct gtklocal *gtklocal, int r)
 {
@@ -739,7 +624,7 @@ static int
 gtk_output(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   char code;
-  int *cpar;
+  const int *cpar;
   struct gtklocal *gtklocal;
 
   code = *(char *) (argv[3]);
@@ -805,7 +690,7 @@ gtk_set_fit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **
 
   local->fit = fit;
 
-  if (fit) {
+  if (fit && local->mainwin) {
     GtkAllocation allocation;
     gtk_widget_get_allocation(local->mainwin, &allocation);
     size_allocate(local->mainwin, &allocation, local);
@@ -839,76 +724,19 @@ gtk_set_size(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char *
     return 1;
   }
 
+  if (local->mainwin == NULL)
+    return 1;
+
   switch (argv[1][0]) {
   case 'h':
     height = size;
     _getobj(obj, "width", inst, &width);
-    gtk_window_resize(GTK_WINDOW(local->mainwin), width, height);
+    gtk_window_set_default_size(GTK_WINDOW(local->mainwin), width, height);
     break;
   case 'w':
     width = size;
     _getobj(obj, "height", inst, &height);
-    gtk_window_resize(GTK_WINDOW(local->mainwin), width, height);
-    break;
-  }
-
-  return 0;
-}
-
-static int
-gtkwait_action(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
-{
-  struct gtklocal *local;
-  char *name;
-
-  _getobj(obj, "_gtklocal", inst, &local);
-
-  g_free(rval->str);
-  rval->str = NULL;
-
-  if (local->mainwin == NULL) {
-    return 1;
-  }
-
-  if (local->blank_cursor == NULL) {
-    local->blank_cursor = gdk_cursor_new_for_display(gdk_display_get_default(), GDK_BLANK_CURSOR);
-    gdk_window_set_cursor(gtk_widget_get_window(local->mainwin), local->blank_cursor);
-  }
-
-  local->action.type = ACTION_TYPE_NONE;
-  local->action.val = 0;
-  while (local->action.type == ACTION_TYPE_NONE) {
-    gtk_evloop(NULL, NULL, NULL, 0, NULL);
-    msleep(100);
-  }
-
-  switch (local->action.type) {
-  case ACTION_TYPE_NONE:
-    break;
-  case ACTION_TYPE_KEY:
-    name = gdk_keyval_name(local->action.val);
-    if (name) {
-      rval->str = g_strdup(name);
-    }
-    break;
-  case ACTION_TYPE_BUTTON:
-    rval->str = g_strdup_printf("button%d", local->action.val);
-    break;
-  case ACTION_TYPE_SCROLL:
-    switch (local->action.val) {
-    case GDK_SCROLL_UP:
-      rval->str = g_strdup("scroll_up");
-      break;
-    case GDK_SCROLL_DOWN:
-      rval->str = g_strdup("scroll_down");
-      break;
-    case GDK_SCROLL_LEFT:
-      rval->str = g_strdup("scroll_left");
-      break;
-    case GDK_SCROLL_RIGHT:
-      rval->str = g_strdup("scroll_right");
-      break;
-    }
+    gtk_window_set_default_size(GTK_WINDOW(local->mainwin), width, height);
     break;
   }
 
@@ -934,13 +762,11 @@ static struct objtable gra2gtk[] = {
   {"BR", NINT, NREAD | NWRITE, gtkbg, NULL, 0},
   {"BG", NINT, NREAD | NWRITE, gtkbg, NULL, 0},
   {"BB", NINT, NREAD | NWRITE, gtkbg, NULL, 0},
-  {"wait_action",NSFUNC, NREAD | NEXEC, gtkwait_action, "", 0},
   {"_gtklocal", NPOINTER, 0, NULL, NULL, 0},
   {"_output", NVFUNC, 0, gtk_output, NULL, 0},
   {"_strwidth", NIFUNC, 0, gra2cairo_strwidth, NULL, 0},
   {"_charascent", NIFUNC, 0, gra2cairo_charheight, NULL, 0},
   {"_chardescent", NIFUNC, 0, gra2cairo_charheight, NULL, 0},
-  {"_evloop", NVFUNC, 0, gtk_evloop, NULL, 0},
 };
 
 #define TBLNUM (sizeof(gra2gtk) / sizeof(*gra2gtk))
