@@ -29,6 +29,7 @@
 #include <string.h>
 #include <ctype.h>
 
+#include "shell.h"
 #include "object.h"
 #include "mathfn.h"
 #include "nstring.h"
@@ -55,6 +56,32 @@ static char *dlgerrorlist[] = {
 
 static GtkWidget *DLGTopLevel = NULL;
 
+struct dialog_data {
+  char *title, *msg, *initial_text, *response_text, *defext, **files;
+  struct narray *buttons, *sarray;
+  int response, wait, *button, selected, *ival, quit_main_loop;
+  double min, max, inc, *val;
+};
+
+static int
+dialog_run(char *title, char *mes, GSourceOnceFunc func, struct dialog_data *data)
+{
+  data->wait = TRUE;
+  data->msg = mes;
+  data->title = title;
+  data->response_text = NULL;
+  if (main_loop_is_running()) {
+    data->quit_main_loop = FALSE;
+    g_idle_add_once(func, data);
+    dialog_wait(&data->wait);
+  } else {
+    data->quit_main_loop = TRUE;
+    func(data);
+    main_loop_run();
+  }
+  return data->response;
+}
+
 static GtkWidget *
 get_toplevel_window(void)
 {
@@ -63,7 +90,7 @@ get_toplevel_window(void)
   }
 
   if (DLGTopLevel == NULL) {
-    DLGTopLevel = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+    DLGTopLevel = gtk_window_new();
   }
 
   return DLGTopLevel;
@@ -72,8 +99,6 @@ get_toplevel_window(void)
 static int
 dlginit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
-  int pos = -1;
-
  if (_exeparent(obj, (char *)argv[1], inst, rval, argc, argv)) {
     return 1;
   }
@@ -83,9 +108,6 @@ dlginit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
     return 1;
   }
 
-  _putobj(obj, "x", inst, &pos);
-  _putobj(obj, "y", inst, &pos);
-
   return 0;
 }
 
@@ -97,11 +119,34 @@ dlgdone(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
   return 0;
 }
 
+static void
+dlg_response(int response, gpointer user_data)
+{
+  struct dialog_data *data;
+  data = (struct dialog_data *) user_data;
+  data->response = response;
+  data->wait = FALSE;
+  if (data->quit_main_loop) {
+    g_idle_add_once((GSourceOnceFunc) g_main_loop_quit, main_loop());
+  }
+}
+
+static void
+dlgconfirm_main(gpointer user_data)
+{
+  struct dialog_data *data;
+  data = (struct dialog_data *) user_data;
+  response_message_box(get_toplevel_window(), data->msg, data->title, RESPONS_YESNO, dlg_response, data);
+}
+
 static int
 dlgconfirm(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   char *mes, *title;
   int rcode, locksave;
+  struct dialog_data data;
+
+  memset(&data, 0, sizeof(data));
 
   if (_getobj(obj, "title", inst, &title)) {
     title = NULL;
@@ -111,7 +156,7 @@ dlgconfirm(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **a
   locksave = Globallock;
   Globallock = TRUE;
   mes = CHK_STR(mes);
-  rcode = message_box(get_toplevel_window(), mes, (title) ? title : _("Confirm"), RESPONS_YESNO);
+  rcode = dialog_run(title ? title : _("Confirm"), mes, dlgconfirm_main, &data);
   Globallock = locksave;
   if (rcode == IDYES) {
     rval->i = 1;
@@ -121,11 +166,22 @@ dlgconfirm(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **a
   return (rcode == IDYES)? 0 : 1;
 }
 
+static void
+dlgmessage_main(gpointer user_data)
+{
+  struct dialog_data *data;
+  data = (struct dialog_data *) user_data;
+  markup_message_box_full(get_toplevel_window(), CHK_STR(data->msg), (data->title) ? data->title : _("Message"), RESPONS_OK, FALSE, dlg_response, data);
+}
+
 static int
 dlgmessage(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   char *mes, *title;
   int locksave;
+  struct dialog_data data;
+
+  memset(&data, 0, sizeof(data));
 
   if (_getobj(obj, "title", inst, &title)) {
     title = NULL;
@@ -134,7 +190,7 @@ dlgmessage(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **a
   mes = (char *)argv[2];
   locksave = Globallock;
   Globallock = TRUE;
-  message_box(get_toplevel_window(), CHK_STR(mes), (title) ? title : _("Message"), RESPONS_OK);
+  dialog_run(title ? title : _("Message"), mes, dlgmessage_main, &data);
   Globallock = locksave;
 
   return 0;
@@ -162,14 +218,34 @@ dlg_get_buttons(struct objlist *obj, N_VALUE *inst)
   return sarray;
 }
 
+static void
+dlginput_response(int response, const char *str, gpointer user_data)
+{
+  struct dialog_data *data;
+  data = (struct dialog_data *) user_data;
+  dlg_response(response, data);
+  data->response_text = g_strdup(str);
+}
+
+static void
+dlginput_main(gpointer user_data)
+{
+  struct dialog_data *data;
+  data = (struct dialog_data *) user_data;
+  input_dialog(get_toplevel_window(), data->title, data->msg, data->initial_text, _("_OK"), data->buttons, data->button, dlginput_response, data);
+}
+
 static int
 dlginput(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   char *mes, *title, *init_str;
-  int locksave, x, y, r;
+  int locksave, r;
   char *inputbuf;
   struct narray *buttons;
   int btn = -1;
+  struct dialog_data data;
+
+  memset(&data, 0, sizeof(data));
 
   locksave = Globallock;
   Globallock = TRUE;
@@ -186,19 +262,13 @@ dlginput(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
     mes = NULL;
   }
 
-  if (_getobj(obj, "x", inst, &x)) {
-    x = -1;
-  }
-
-  if (_getobj(obj, "y", inst, &y)) {
-    y = -1;
-  }
-
   buttons = dlg_get_buttons(obj, inst);
 
-  r = DialogInput(get_toplevel_window(), (title) ? title : _("Input"), mes, init_str, buttons, &btn, &inputbuf, &x, &y);
-  _putobj(obj, "x", inst, &x);
-  _putobj(obj, "y", inst, &y);
+  data.buttons = buttons;
+  data.button = &btn;
+  data.initial_text = init_str;
+  r = dialog_run(title ? title : _("Confirm"), mes, dlginput_main, &data);
+  inputbuf = data.response_text;
   _putobj(obj, "response_button", inst, &btn);
   if (r == IDOK && inputbuf != NULL) {
     rval->str = inputbuf;
@@ -264,12 +334,23 @@ get_sarray_argument(struct narray *sarray)
   return sarray;
 }
 
+static void
+dlgbutton_main(gpointer user_data)
+{
+  struct dialog_data *data;
+  data = (struct dialog_data *) user_data;
+  button_dialog(get_toplevel_window(), data->title, data->msg, data->buttons, dlg_response, data);
+}
+
 static int
 dlgbutton(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   char *title, *caption;
-  int rcode, x, y;
+  int rcode;
   struct narray *sarray;
+  struct dialog_data data;
+
+  memset(&data, 0, sizeof(data));
 
   sarray = get_sarray_argument((struct narray *) argv[2]);
   if (arraynum(sarray) == 0) {
@@ -284,21 +365,12 @@ dlgbutton(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **ar
     caption = NULL;
   }
 
-  if (_getobj(obj, "x", inst, &x)) {
-    x = -1;
-  }
-
-  if (_getobj(obj, "y", inst, &y)) {
-    y = -1;
-  }
 
   g_free(rval->str);
   rval->str = NULL;
 
-  rcode = DialogButton(get_toplevel_window(), (title) ? title : _("Select"), caption, sarray, &x, &y);
-
-  _putobj(obj, "x", inst, &x);
-  _putobj(obj, "y", inst, &y);
+  data.buttons = sarray;
+  rcode = dialog_run(title ? title : _("Select"), caption, dlgbutton_main, &data);
 
   if (rcode > 0) {
     const char *str;
@@ -311,14 +383,26 @@ dlgbutton(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **ar
   return 0;
 }
 
+static void
+dlgradio_main(gpointer user_data)
+{
+  struct dialog_data *data;
+  data = (struct dialog_data *) user_data;
+  radio_dialog(get_toplevel_window(), data->title, data->msg, data->sarray, _("_OK"), data->buttons, data->button, data->selected, dlg_response, data);
+}
+
+
 static int
 dlgradio(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   char *title, *caption;
-  int locksave, r, x, y, ret;
+  int locksave, r, ret;
   struct narray *iarray, *sarray;
   struct narray *buttons;
   int btn = -1;
+  struct dialog_data data;
+
+  memset(&data, 0, sizeof(data));
 
   sarray = get_sarray_argument((struct narray *) argv[2]);
   if (arraynum(sarray) == 0)
@@ -339,20 +423,15 @@ dlgradio(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
     iarray = NULL;
   }
 
-  if (_getobj(obj, "x", inst, &x)) {
-    x = -1;
-  }
-
-  if (_getobj(obj, "y", inst, &y)) {
-    y = -1;
-  }
-
   r = arraylast_int(iarray);
 
   buttons = dlg_get_buttons(obj, inst);
-  ret = DialogRadio(get_toplevel_window(), (title) ? title : _("Select"), caption, sarray, buttons, &btn, &r, &x, &y);
-  _putobj(obj, "x", inst, &x);
-  _putobj(obj, "y", inst, &y);
+  data.buttons = buttons;
+  data.button = &btn;
+  data.selected = r;
+  data.sarray = sarray;
+  r = dialog_run(title ? title : _("Select"), caption, dlgradio_main, &data);
+  ret = (r < 0) ? IDCANCEL : IDOK;
   _putobj(obj, "response_button", inst, &btn);
   if (ret != IDOK) {
     Globallock = locksave;
@@ -365,14 +444,42 @@ dlgradio(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
   return 0;
 }
 
+static void
+dlgcombo_response(int response, const char *str, gpointer user_data)
+{
+  struct dialog_data *data;
+  data = (struct dialog_data *) user_data;
+  dlg_response(response, data);
+  data->response_text = g_strdup(str);
+}
+
+static void
+dlgcombo_main(gpointer user_data)
+{
+  struct dialog_data *data;
+  data = (struct dialog_data *) user_data;
+  combo_dialog(get_toplevel_window(), data->title, data->msg, data->sarray, data->buttons, data->button, data->selected, dlgcombo_response, data);
+}
+
+static void
+dlgcombo_entry_main(gpointer user_data)
+{
+  struct dialog_data *data;
+  data = (struct dialog_data *) user_data;
+  combo_entry_dialog(get_toplevel_window(), data->title, data->msg, data->sarray, data->buttons, data->button, data->selected, dlgcombo_response, data);
+}
+
 static int
 dlgcombo(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
-  int locksave, sel, ret, x, y;
+  int locksave, sel, ret;
   char *r, *title, *caption;
   struct narray *iarray, *sarray;
   struct narray *buttons;
   int btn = -1;
+  struct dialog_data data;
+
+  memset(&data, 0, sizeof(data));
 
   sarray = get_sarray_argument((struct narray *) argv[2]);
   if (arraynum(sarray) == 0)
@@ -396,24 +503,18 @@ dlgcombo(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
     iarray = NULL;
   }
 
-  if (_getobj(obj, "x", inst, &x)) {
-    x = -1;
-  }
-
-  if (_getobj(obj, "y", inst, &y)) {
-    y = -1;
-  }
-
   sel = arraylast_int(iarray);
   buttons = dlg_get_buttons(obj, inst);
+  data.buttons = buttons;
+  data.button = &btn;
+  data.selected = sel;
+  data.sarray = sarray;
   if (strcmp(argv[1], "combo") == 0) {
-    ret = DialogCombo(get_toplevel_window(), (title) ? title : _("Select"), caption, sarray, buttons, &btn, sel, &r, &x, &y);
+    ret = dialog_run(title ? title : _("Select"), caption, dlgcombo_main, &data);
   } else {
-    ret = DialogComboEntry(get_toplevel_window(), (title) ? title : _("Input"), caption, sarray, buttons, &btn, sel, &r, &x, &y);
+    ret = dialog_run(title ? title : _("Select"), caption, dlgcombo_entry_main, &data);
   }
-
-  _putobj(obj, "x", inst, &x);
-  _putobj(obj, "y", inst, &y);
+  r = data.response_text;
   _putobj(obj, "response_button", inst, &btn);
   if (ret != IDOK) {
     Globallock = locksave;
@@ -426,14 +527,25 @@ dlgcombo(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
   return 0;
 }
 
+static void
+dlgspin_main(gpointer user_data)
+{
+  struct dialog_data *data;
+  data = (struct dialog_data *) user_data;
+  spin_dialog(get_toplevel_window(), data->title, data->msg, data->min, data->max, data->inc, data->buttons, data->button, data->val, dlg_response, data);
+}
+
 static int
 dlgspin(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
-  int locksave, ret, type, x, y;
+  int locksave, ret, type;
   char *title, *caption;
   double min, max, inc, r;
   struct narray *buttons;
   int btn = -1;
+  struct dialog_data data;
+
+  memset(&data, 0, sizeof(data));
 
   locksave = Globallock;
   Globallock = TRUE;
@@ -446,14 +558,6 @@ dlgspin(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
     caption = NULL;
   }
 
-  if (_getobj(obj, "x", inst, &x)) {
-    x = -1;
-  }
-
-  if (_getobj(obj, "y", inst, &y)) {
-    y = -1;
-  }
-
   type = argv[1][0];
   switch (type) {
   case 'd':
@@ -474,10 +578,13 @@ dlgspin(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
   }
 
   buttons = dlg_get_buttons(obj, inst);
-  ret = DialogSpinEntry(get_toplevel_window(), (title) ? title : _("Input"), caption, min, max, inc, buttons, &btn, &r, &x, &y);
-
-  _putobj(obj, "x", inst, &x);
-  _putobj(obj, "y", inst, &y);
+  data.buttons = buttons;
+  data.button = &btn;
+  data.val = &r;
+  data.min = min;
+  data.max = max;
+  data.inc = inc;
+  ret = dialog_run(title ? title : _("Input"), caption, dlgspin_main, &data);
   _putobj(obj, "response_button", inst, &btn);
   if (ret != IDOK) {
     Globallock = locksave;
@@ -500,14 +607,25 @@ dlgspin(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
   return 0;
 }
 
+static void
+dlgcheck_main(gpointer user_data)
+{
+  struct dialog_data *data;
+  data = (struct dialog_data *) user_data;
+  check_dialog(get_toplevel_window(), data->title, data->msg, data->sarray, data->buttons, data->button, data->ival, dlg_response, data);
+}
+
 static int
 dlgcheck(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
-  int locksave, *r, i, n, inum, x, y, ret;
+  int locksave, *r, i, n, inum, ret;
   struct narray *array, *sarray, *iarray;
   char *title, *caption;
   struct narray *buttons;
   int btn = -1;
+  struct dialog_data data;
+
+  memset(&data, 0, sizeof(data));
 
   sarray = get_sarray_argument((struct narray *) argv[2]);
   n = arraynum(sarray);
@@ -534,14 +652,6 @@ dlgcheck(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
     iarray = NULL;
   }
 
-  if (_getobj(obj, "x", inst, &x)) {
-    x = -1;
-  }
-
-  if (_getobj(obj, "y", inst, &y)) {
-    y = -1;
-  }
-
   r = g_malloc(n * sizeof(int));
   if (r == NULL) {
     arrayfree(array);
@@ -554,16 +664,18 @@ dlgcheck(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
 
   inum = arraynum(iarray);
   for (i = 0; i < inum; i++) {
-    int *ptr;
+    const int *ptr;
     ptr = (int *) arraynget(iarray, i);
     if (ptr && *ptr >= 0 && *ptr < n)
       r[*ptr] = 1;
   }
 
   buttons = dlg_get_buttons(obj, inst);
-  ret = DialogCheck(get_toplevel_window(), (title) ? title : _("Select"), caption, sarray, buttons, &btn, r, &x, &y);
-  _putobj(obj, "x", inst, &x);
-  _putobj(obj, "y", inst, &y);
+  data.buttons = buttons;
+  data.button = &btn;
+  data.sarray = sarray;
+  data.ival = r;
+  ret = dialog_run(title ? title : _("Select"), caption, dlgcheck_main, &data);
   _putobj(obj, "response_button", inst, &btn);
   if (ret != IDOK) {
     arrayfree(array);
@@ -598,6 +710,23 @@ dlgbeep(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
   return 0;
 }
 
+static void
+dlggetopenfile_response(char *file, gpointer user_data)
+{
+  struct dialog_data *data;
+  data = (struct dialog_data *) user_data;
+  dlg_response((file) ? IDOK : IDCANCEL, data);
+  data->response_text = file;
+}
+
+static void
+dlggetopenfile_main(gpointer user_data)
+{
+  struct dialog_data *data;
+  data = (struct dialog_data *) user_data;
+  nGetOpenFileName(get_toplevel_window(), data->title, data->defext, NULL, data->initial_text, FALSE, dlggetopenfile_response, data);
+}
+
 static int
 dlggetopenfile(struct objlist *obj, N_VALUE *inst, N_VALUE *rval,
 	       int argc, char **argv)
@@ -609,6 +738,9 @@ dlggetopenfile(struct objlist *obj, N_VALUE *inst, N_VALUE *rval,
   int locksave;
   int ret;
   char *file;
+  struct dialog_data data;
+
+  memset(&data, 0, sizeof(data));
 
   locksave = Globallock;
   Globallock = TRUE;
@@ -630,20 +762,36 @@ dlggetopenfile(struct objlist *obj, N_VALUE *inst, N_VALUE *rval,
     filter = d[0];
     initfile = d[1];
   }
-  ret = nGetOpenFileName(get_toplevel_window(), _("Open file"),
-			 filter, NULL, initfile,
-			 &file, TRUE, TRUE);
-  if (ret == IDOK) {
-    if (file) {
-      changefilename(file);
-      rval->str = file;
-    }
+  data.initial_text = initfile;
+  data.defext = filter;
+  ret = dialog_run(_("Open file"), NULL, dlggetopenfile_main, &data);
+  file = data.response_text;
+  if (file) {
+    changefilename(file);
+    rval->str = file;
   }
 
   Globallock = locksave;
   return (ret == IDOK)? 0 : 1;
 }
 
+static void
+dlggetopenfiles_response(char **files, gpointer user_data)
+{
+  struct dialog_data *data;
+  data = (struct dialog_data *) user_data;
+  dlg_response((files) ? IDOK : IDCANCEL, data);
+  data->files = files;
+}
+
+static void
+dlggetopenfiles_main(gpointer user_data)
+{
+  struct dialog_data *data;
+  data = (struct dialog_data *) user_data;
+  nGetOpenFileNameMulti(get_toplevel_window(), data->title, data->defext, NULL, data->initial_text, FALSE, dlggetopenfiles_response, data);
+}
+
 static int
 dlggetopenfiles(struct objlist *obj, N_VALUE *inst, N_VALUE *rval,
 		int argc, char **argv)
@@ -656,6 +804,9 @@ dlggetopenfiles(struct objlist *obj, N_VALUE *inst, N_VALUE *rval,
   int ret;
   char **file = NULL;
   struct narray *farray;
+  struct dialog_data data;
+
+  memset(&data, 0, sizeof(data));
 
   locksave = Globallock;
   Globallock = TRUE;
@@ -677,9 +828,11 @@ dlggetopenfiles(struct objlist *obj, N_VALUE *inst, N_VALUE *rval,
     filter = d[0];
     initfile = d[1];
   }
-  ret = nGetOpenFileNameMulti(get_toplevel_window(), _("Open files"),
-			      filter, NULL, initfile, &file, TRUE);
-  if (ret == IDOK) {
+  data.initial_text = initfile;
+  data.defext = filter;
+  ret = dialog_run(_("Open file"), NULL, dlggetopenfiles_main, &data);
+  file = data.files;
+  if (file) {
     int i;
     farray = arraynew(sizeof(char *));
     for (i = 0; file[i]; i++) {
@@ -687,14 +840,22 @@ dlggetopenfiles(struct objlist *obj, N_VALUE *inst, N_VALUE *rval,
       arrayadd(farray, file + i);
     }
     rval->array = farray;
+    g_free(file);
   }
-  g_free(file);
 
   Globallock = locksave;
 
   return (ret == IDOK)? 0 : 1;
 }
 
+static void
+dlggetsavefile_main(gpointer user_data)
+{
+  struct dialog_data *data;
+  data = (struct dialog_data *) user_data;
+  nGetSaveFileName(get_toplevel_window(), data->title, data->defext, NULL, data->initial_text, FALSE, dlggetopenfile_response, data);
+}
+
 static int
 dlggetsavefile(struct objlist *obj, N_VALUE *inst, N_VALUE *rval,
 	       int argc, char **argv)
@@ -706,6 +867,9 @@ dlggetsavefile(struct objlist *obj, N_VALUE *inst, N_VALUE *rval,
   int locksave;
   int ret;
   char *file;
+  struct dialog_data data;
+
+  memset(&data, 0, sizeof(data));
 
   locksave = Globallock;
   Globallock = TRUE;
@@ -727,14 +891,13 @@ dlggetsavefile(struct objlist *obj, N_VALUE *inst, N_VALUE *rval,
     filter = d[0];
     initfile = d[1];
   }
-  ret = nGetSaveFileName(get_toplevel_window(), _("Save file"),
-			 filter, NULL, initfile,
-			 &file, FALSE, TRUE);
-  if (ret == IDOK) {
-    if (file) {
-      changefilename(file);
-      rval->str = file;
-    }
+  data.initial_text = initfile;
+  data.defext = filter;
+  ret = dialog_run(_("Save file"), NULL, dlggetsavefile_main, &data);
+  file = data.response_text;
+  if (file) {
+    changefilename(file);
+    rval->str = file;
   }
 
   Globallock = locksave;
@@ -745,8 +908,6 @@ static struct objtable dialog[] = {
   {"init", NVFUNC, NEXEC, dlginit, NULL, 0},
   {"done", NVFUNC, NEXEC, dlgdone, NULL, 0},
   {"next", NPOINTER, 0, NULL, NULL, 0},
-  {"x", NINT, NREAD | NWRITE, NULL, NULL, 0},
-  {"y", NINT, NREAD | NWRITE, NULL, NULL, 0},
   {"title", NSTR, NREAD | NWRITE, NULL, NULL, 0},
   {"caption", NSTR, NREAD | NWRITE, NULL, NULL, 0},
   {"select", NIARRAY, NREAD | NWRITE, NULL, NULL, 0},
