@@ -68,6 +68,7 @@
 #include "x11file.h"
 #include "x11cood.h"
 #include "x11info.h"
+#include "x11commn.h"
 
 #define NAME "menu"
 #define ALIAS "winmenu:gtkmenu"
@@ -162,7 +163,6 @@ static struct menu_config MenuConfigMisc[] = {
   {"history_size",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.hist_size},
   {"infowin_size",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.info_size},
   {"data_head_lines",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.data_head_lines},
-  {"use_opacity",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.use_opacity},
   {"select_data_on_export",	MENU_CONFIG_TYPE_BOOL,    NULL, &Menulocal.select_data},
   {"use_custom_palette",	MENU_CONFIG_TYPE_BOOL,    NULL, &Menulocal.use_custom_palette},
   {"custom_palette",		MENU_CONFIG_TYPE_COLOR_ARY, menu_config_set_custom_palette, NULL},
@@ -198,9 +198,6 @@ static struct menu_config MenuConfigToggleView[] = {
 
 static struct menu_config MenuConfigOthers[] = {
   {"png_dpi",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.png_dpi},
-#if WINDOWS
-  {"emf_dpi",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.emf_dpi},
-#endif
   {"ps_version",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.ps_version},
   {"svg_version",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.svg_version},
   {"main_pane",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.main_pane_pos},
@@ -218,6 +215,7 @@ static struct menu_config MenuConfigOthers[] = {
   {"text_tab",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.text_tab},
   {"parameter_tab",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.parameter_tab},
   {"math_input_mode",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.math_input_mode},
+  {"icon_size",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.icon_size_local}, /* this setting should be saved automatically */
   {NULL},
 };
 
@@ -248,6 +246,16 @@ static struct menu_config *MenuConfigArrray[] = {
 
 static NHASH MenuConfigHash = NULL;
 
+typedef void (* void_func) (void);
+
+static void
+idle_call_func(gpointer user_data)
+{
+  void_func func;
+  func = (void_func) user_data;
+  func();
+}
+
 static void
 add_str_with_int_to_array(struct menu_config *cfg, struct narray *conf)
 {
@@ -302,7 +310,7 @@ add_color_ary_to_array(struct menu_config *cfg, struct narray *conf)
   char *buf;
   struct narray *palette;
   int i, n;
-  GdkRGBA *color;
+  const GdkRGBA *color;
   palette = &(Menulocal.custom_palette);
   n = arraynum(palette);
   if (n < 1) {
@@ -711,7 +719,7 @@ mgtkloadconfig(void)
 }
 
 void
-menuadddrawrable(struct objlist *parent, struct narray *drawrable)
+menuadddrawrable(const struct objlist *parent, struct narray *drawrable)
 {
   struct objlist *ocur;
   const char *name;
@@ -882,7 +890,7 @@ init_custom_palette(void)
 {
   struct narray *palette;
   int i, j, k;
-  unsigned int c[] = {0xff, 0xcc, 0x99};
+  const unsigned int c[] = {0xff, 0xcc, 0x99};
   GdkRGBA color;
 
   palette = &(Menulocal.custom_palette);
@@ -899,7 +907,6 @@ init_custom_palette(void)
       }
     }
   }
-  add_default_gray(palette);
 }
 
 #define DEFAULT_AXIS_LENGTH 14000
@@ -1016,6 +1023,7 @@ menuinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
   Menulocal.mark_tab = 4;
   Menulocal.text_tab = 5;
   Menulocal.math_input_mode = 1;
+  Menulocal.icon_size_local = GTK_ICON_SIZE_NORMAL;
   menu_default_axis_size(&Menulocal);
 
   arrayinit(&(Menulocal.drawrable), sizeof(char *));
@@ -1032,20 +1040,17 @@ menuinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
   Menulocal.grid = 200;
   Menulocal.show_grid = TRUE;
   Menulocal.data_head_lines = 100;
-  Menulocal.use_opacity = FALSE;
   Menulocal.select_data = TRUE;
   Menulocal.local = local;
 
   Menulocal.png_dpi = 72;
-#if WINDOWS
-  Menulocal.emf_dpi = 576;
-#endif
   Menulocal.ps_version = 0;
   Menulocal.svg_version = 0;
 
   if (mgtkloadconfig())
     goto errexit;
 
+  Menulocal.icon_size = Menulocal.icon_size_local;
   Menulocal.Decimalsign = get_gra_decimalsign_type(Menulocal.default_decimalsign);
   gra_set_default_decimalsign(Menulocal.Decimalsign);
 
@@ -1077,10 +1082,6 @@ menuinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
   if (_putobj(obj, "redraw_num", inst, &(Menulocal.redrawf_num)))
     goto errexit;
 
-  Menulocal.local->use_opacity = Menulocal.use_opacity;
-  if (_putobj(obj, "use_opacity", inst, &Menulocal.use_opacity))
-    goto errexit;
-
   Menulocal.modified = 0;
   if (_putobj(obj, "modified", inst, &Menulocal.modified))
     goto errexit;
@@ -1136,11 +1137,67 @@ mgtkdisplaystatus(const char *str)
   DisplayDialog(str);
 }
 
+#define USE_BLOCK_PUTSTDERR 1
+#if USE_BLOCK_PUTSTDERR
+struct dialog_data {
+  const char *msg;
+  int wait;
+};
+
+static void
+dialog_run(const char *mes, GSourceOnceFunc func, struct dialog_data *data)
+{
+  data->wait = TRUE;
+  data->msg = mes;
+  g_idle_add_once(func, data);
+  dialog_wait(&data->wait);
+}
+
+static void
+putstderr_response(int response, gpointer user_data)
+{
+  struct dialog_data *data;
+  data = (struct dialog_data *) user_data;
+  if (data) {
+    data->wait = FALSE;
+  }
+}
+
+static void
+error_message(const char *msg, gpointer data)
+{
+  markup_message_box_full(get_current_window(), CHK_STR(msg), _("Error:"), RESPONS_ERROR, FALSE, putstderr_response, data);
+}
+
+static void
+putstderr_main(gpointer user_data)
+{
+  struct dialog_data *data;
+  data = (struct dialog_data *) user_data;
+  error_message(data->msg, data);
+}
+
+int
+mgtkputstderr(const char *s)
+{
+  struct dialog_data data;
+
+  if (ProgressDialogIsActive()) {
+    ProgressDialog_append_text(s);
+  } else if (is_main_thread()) {
+    error_message(s, NULL);
+  } else {
+    dialog_run(s, putstderr_main, &data);
+  }
+  return strlen(s) + 1;
+}
+#else
 int
 mgtkputstderr(const char *s)
 {
   return PutStderr(s);
 }
+#endif
 
 int
 mgtkputstdout(const char *s)
@@ -1161,7 +1218,11 @@ mgtkprintfstderr(const char *fmt, ...)
   va_start(ap, fmt);
   len = vsnprintf(buf, sizeof(buf), fmt, ap);
   va_end(ap);
+#if USE_BLOCK_PUTSTDERR
+  mgtkputstderr(buf);
+#else
   PutStderr(buf);
+#endif
   return len;
 }
 
@@ -1246,13 +1307,6 @@ menumenu(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
   return r;
 }
 
-static int
-mx_evloop(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
-{
-  reset_event();
-  return 0;
-}
-
 static int
 mxredrawflag(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc,
 	     char **argv)
@@ -1278,19 +1332,6 @@ mxredraw_num(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc,
   return 0;
 }
 
-static int
-mxuse_opacity(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc,
-	      char **argv)
-{
-  int n;
-
-  n = *(int *) argv[2];
-
-  Menulocal.local->use_opacity = Menulocal.use_opacity = n;
-
-  return 0;
-}
-
 static int
 mx_data_head_lines(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc,
 	     char **argv)
@@ -1311,17 +1352,11 @@ mx_data_head_lines(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc,
 void
 main_window_redraw(void)
 {
-  GdkWindow *win;
   if (NgraphApp.Viewer.Win == NULL) {
     return;
   }
 
-  win = gtk_widget_get_window(NgraphApp.Viewer.Win);
-  if(win == NULL) {
-    return;
-  }
-
-  gdk_window_invalidate_rect(win, NULL, FALSE);
+  gtk_widget_queue_draw(NgraphApp.Viewer.Win);
 }
 
 void
@@ -1358,15 +1393,35 @@ mx_redraw(struct objlist *obj, N_VALUE *inst, char const **objects)
   main_window_redraw();
 }
 
+struct mxredraw_data
+{
+  struct objlist *obj;
+  N_VALUE *inst;
+};
+
+static void
+mxredraw_main(gpointer user_data)
+{
+  struct mxredraw_data *data;
+  data = (struct mxredraw_data *) user_data;
+  mx_redraw(data->obj, data->inst, NULL);
+  g_free(data);
+}
+
 static int
 mxredraw(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
+  struct mxredraw_data *data;
+
   if (TopLevel == NULL) {
     error(obj, ERR_MENU_GUI);
     return 1;
   }
 
-  mx_redraw(obj, inst, NULL);
+  data = g_malloc0(sizeof(*data));
+  data->obj = obj;
+  data->inst = inst;
+  g_idle_add_once(mxredraw_main, data);
   return 0;
 }
 
@@ -1438,14 +1493,9 @@ clear_region(cairo_t *cr, cairo_region_t *region)
   cairo_restore(cr);
 }
 
-static int
-mxflush(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
+static void
+flush_window(gpointer user_data)
 {
-  if (TopLevel == NULL) {
-    error(obj, ERR_MENU_GUI);
-    return 1;
-  }
-
   if (Menulocal.local->cairo) {
     cairo_surface_t *surface;
     gra2cairo_draw_path(Menulocal.local);
@@ -1463,6 +1513,16 @@ mxflush(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
     flush_layers(cr);
     cairo_destroy(cr);
   }
+}
+
+static int
+mxflush(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
+{
+  if (TopLevel == NULL) {
+    error(obj, ERR_MENU_GUI);
+    return 1;
+  }
+  flush_window(NULL);
 
   return 0;
 }
@@ -1496,7 +1556,7 @@ mx_clear(cairo_region_t *region, char const **objects)
     array = &Menulocal.drawrable;
     n = arraynum(array);
     for (i = 0; i < n; i++) {
-      char *obj;
+      const char *obj;
       obj = arraynget_str(array, i);
       clear_layer(obj, region);
     }
@@ -1516,7 +1576,7 @@ mxclear(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
 
   mx_clear(NULL, NULL);
 
-  main_window_redraw();
+  g_idle_add_once(idle_call_func, main_window_redraw);
 
   return 0;
 }
@@ -1537,7 +1597,7 @@ mxfullpathngp(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc,
 }
 
 static int
-check_object_name(struct objlist *obj, struct narray *array)
+check_object_name(const struct objlist *obj, struct narray *array)
 {
   int i, n;
   char **adata;
@@ -1614,36 +1674,80 @@ mx_get_focused(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char
 
 }
 
+static void
+print_dialog_main_quit(gpointer user_data)
+{
+  g_main_loop_quit(main_loop());
+}
+
+static void
+print_dialog_quit(int res, gpointer user_data)
+{
+  gtk_window_destroy(GTK_WINDOW(TopLevel));
+  TopLevel = NULL;
+  g_idle_add_once(print_dialog_main_quit, NULL);
+}
+
+static void
+print_dialog(int flag, response_cb cb)
+{
+  int lock;
+  int show_dialog, select_file;
+
+  select_file = (flag & 1);
+  show_dialog = (flag >> 1);
+  lock = Menulock;
+  menu_lock(FALSE);
+  CmOutputPrinter(select_file, show_dialog, cb, NULL);
+  /* to be implemented correctly */
+  menu_lock(lock);
+}
+
+static void
+print_dialog_cb(gpointer user_data)
+{
+  print_dialog(GPOINTER_TO_INT(user_data), print_dialog_quit);
+}
+
+static void
+create_toplevel_window(int flag)
+{
+  GtkWidget *label;
+
+  TopLevel = gtk_window_new();
+  gtk_window_set_deletable(GTK_WINDOW(TopLevel), FALSE);
+  gtk_window_set_resizable(GTK_WINDOW(TopLevel), FALSE);
+
+  g_signal_connect(TopLevel, "close_request", G_CALLBACK(gtk_true), NULL);
+  label = gtk_label_new(" Ngraph ");
+  gtk_window_set_child(GTK_WINDOW(TopLevel), label);
+  gtk_window_present(GTK_WINDOW (TopLevel));
+  g_idle_add_once(print_dialog_cb, GINT_TO_POINTER(flag));
+  g_main_loop_run(main_loop());
+}
+
+static void
+print_dialog_run(gpointer user_data)
+{
+  print_dialog(GPOINTER_TO_INT(user_data), NULL);
+}
+
 static int
 mx_print(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
-  int show_dialog, select_file, create_window = FALSE, lock;
+  int show_dialog, select_file, flag;
 
   select_file = * (int *) argv[2];
   show_dialog = * (int *) argv[3];
 
+  flag = (show_dialog << 1 | (select_file));
+
   if (TopLevel == NULL) {
-    GtkWidget *label;
-    create_window = TRUE;
-    TopLevel = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-    gtk_window_set_type_hint(GTK_WINDOW(TopLevel), GDK_WINDOW_TYPE_HINT_DIALOG);
-    g_signal_connect(TopLevel, "delete-event", G_CALLBACK(gtk_true), NULL);
-    label = gtk_label_new(" Ngraph ");
-    gtk_container_add(GTK_CONTAINER(TopLevel), label);
-    gtk_widget_show_all(TopLevel);
-    reset_event();
+    create_toplevel_window(flag);
+    return 0;
   }
 
-  lock = Menulock;
-  menu_lock(FALSE);
-  CmOutputPrinter(select_file, show_dialog);
-  menu_lock(lock);
-
-  if (create_window) {
-    gtk_widget_destroy(TopLevel);
-    TopLevel = NULL;
-    reset_event();
-  }
+  g_idle_add_once(print_dialog_run, GINT_TO_POINTER(flag));
   return 0;
 }
 
@@ -1706,7 +1810,8 @@ mx_clear_info(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char
     return 1;
   }
 
-  InfoWinClear();
+  g_idle_add_once(idle_call_func, InfoWinClear);
+
   return 0;
 }
 
@@ -1754,7 +1859,8 @@ mx_get_accel_map(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, ch
 static int
 mx_show_lib_version(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
-  char *s, h[256];
+  const char *s;
+  char h[256];
   int i, n;
   GString *str;
 
@@ -1785,9 +1891,9 @@ mx_show_lib_version(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc,
 	       GTK_MINOR_VERSION,
 	       GTK_MICRO_VERSION,
 	       h,
-	       gtk_major_version,
-	       gtk_minor_version,
-	       gtk_micro_version);
+	       gtk_get_major_version(),
+	       gtk_get_minor_version(),
+	       gtk_get_micro_version());
 
   g_string_append_printf(str, "%sGLib\n"
 	       "%s compile: %d.%d.%d\n"
@@ -1899,15 +2005,32 @@ mx_show_source_view_search_path(struct objlist *obj, N_VALUE *inst, N_VALUE *rva
   return 0;
 }
 
+static void
+draw_finalise(gpointer user_data)
+{
+  int *drawing;
+  drawing = (int *) user_data;
+  *drawing = FALSE;
+}
+
+static void
+draw_main(gpointer user_data)
+{
+  Draw(FALSE, draw_finalise, user_data);
+}
+
 static int
 mxdraw(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
+  int drawing;
   if (TopLevel == NULL) {
     error(obj, ERR_MENU_GUI);
     return 1;
   }
 
-  Draw(FALSE);
+  drawing = TRUE;
+  g_idle_add_once(draw_main, &drawing);
+  dialog_wait(&drawing);
   return 0;
 }
 
@@ -1948,13 +2071,35 @@ mxmodified(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **a
   return 0;
 }
 
+struct focus_obj_data {
+  struct narray *iarray;
+  struct objlist *lobj;
+  int n, wait;
+};
+
+static void
+focus_obj_main(gpointer user_data)
+{
+  struct focus_obj_data *data;
+  int i, *id_array;
+
+  data = (struct focus_obj_data *) user_data;
+  id_array = arraydata(data->iarray);
+
+  for (i = 0; i < data->n; i++) {
+    Focus(data->lobj, id_array[i], TRUE);
+  }
+  data->wait = FALSE;
+}
+
 static int
 mx_focus_obj(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
-  int n, i, *id_array;
+  int n;
   char *legend;
   struct objlist *lobj;
   struct narray iarray;
+  struct focus_obj_data data;
 
   legend = (char *) argv[2];
   if (legend == NULL) {
@@ -1972,12 +2117,12 @@ mx_focus_obj(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char *
     return 0;
   }
 
-  id_array = arraydata(&iarray);
-
-  for (i = 0; i < n; i++) {
-    Focus(lobj, id_array[i], TRUE);
-  }
-
+  data.iarray = &iarray;
+  data.lobj = lobj;
+  data.n = n;
+  data.wait = TRUE;
+  g_idle_add_once(focus_obj_main, &data);
+  dialog_wait(&data.wait);
   arraydel(&iarray);
 
   return 0;
@@ -1987,7 +2132,7 @@ static int
 mx_unfocus_obj(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
 
-  UnFocus();
+  g_idle_add_once(idle_call_func, UnFocus);
 
   return 0;
 }
@@ -2135,7 +2280,8 @@ mx_addin_list_append(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc
 static int
 mx_output(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
-  char code, *cstr, *layer;
+  char code;
+  const char *cstr, *layer;
   struct gra2cairo_local *local;
 
   local = (struct gra2cairo_local *)argv[2];
@@ -2161,6 +2307,46 @@ mx_output(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **ar
   return 0;
 }
 
+static int
+mx_configuration(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
+{
+  char *id;
+  struct menu_config *cfg;
+
+  if (rval->str) {
+    g_free(rval->str);
+  }
+  rval->str = NULL;
+
+  id = (char *) argv[2];
+  if (id == NULL) {
+    return 0;
+  }
+
+  if (nhash_get_ptr(MenuConfigHash, id, (void *) &cfg)) {
+    return 0;
+  }
+  switch (cfg->type) {
+  case MENU_CONFIG_TYPE_NUMERIC:
+    rval->str = g_strdup_printf("%d", * (int *) cfg->data);
+    break;
+  case MENU_CONFIG_TYPE_BOOL:
+    rval->str = g_strdup_printf("%s", * (int *) cfg->data ? "true" : "false");
+    break;
+  case MENU_CONFIG_TYPE_STRING:
+    rval->str = g_strdup_printf("%s", * (char **) cfg->data);
+    break;
+  case MENU_CONFIG_TYPE_CHARMAP:
+  case MENU_CONFIG_TYPE_COLOR:
+  case MENU_CONFIG_TYPE_COLOR_ARY:
+  case MENU_CONFIG_TYPE_SCRIPT:
+  case MENU_CONFIG_TYPE_WINDOW:
+    break;
+  }
+
+  return 0;
+}
+
 static int
 mx_exeparent(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
@@ -2178,7 +2364,6 @@ static struct objtable gtkmenu[] = {
   {"dpi", NINT, NREAD | NWRITE, mxdpi, NULL, 0},
   {"redraw_flag", NBOOL, NREAD | NWRITE, mxredrawflag, NULL, 0},
   {"redraw_num", NINT, NREAD | NWRITE, mxredraw_num, NULL, 0},
-  {"use_opacity", NBOOL, NREAD | NWRITE, mxuse_opacity, NULL,0},
   {"redraw", NVFUNC, NREAD | NEXEC, mxredraw, "", 0},
   {"draw", NVFUNC, NREAD | NEXEC, mxdraw, "", 0},
   {"flush", NVFUNC, NREAD | NEXEC, mxflush, "", 0},
@@ -2196,7 +2381,7 @@ static struct objtable gtkmenu[] = {
   {"locale", NSFUNC, NREAD | NEXEC, mx_get_locale, "", 0},
   {"active", NBFUNC, NREAD | NEXEC, mx_get_active, "", 0},
   {"addin_list_append", NVFUNC, NREAD | NEXEC, mx_addin_list_append, "o", 0},
-  {"_evloop", NVFUNC, 0, mx_evloop, NULL, 0},
+  {"configuration", NSFUNC, NREAD | NEXEC, mx_configuration, NULL, 0},
   {"_output", NVFUNC, 0, mx_output, NULL, 0},
   {"_strwidth", NIFUNC, 0, mx_exeparent, NULL, 0},
   {"_charascent", NIFUNC, 0, mx_exeparent, NULL, 0},
