@@ -56,14 +56,26 @@
 
 #define COMMENT_BUF_SIZE 1024
 
+struct GraphSave_data
+{
+  int storedata, storemerge, path;
+  char *file, *current_wd, *prev_wd;
+  response_cb cb;
+  gpointer data;
+};
+
 static GtkWidget *ProgressDialog = NULL;
-static GtkProgressBar *ProgressBar, *ProgressBar2;
+#define PROGRESSBAR_N 2
+static GtkProgressBar *ProgressBar[PROGRESSBAR_N];
+static GtkWidget *ProgressFrame, *ProgressText, *ProgressButton;
 static unsigned int SaveCursor;
 
 static void AddNgpFileList(const char *file);
 static void ToFullPath(void);
 static void ToBasename(void);
 static void ToRalativePath(void);
+static void ProgressDialogFinalize(void);
+static void GraphSave_response(int ret, struct GraphSave_data *d);
 
 void
 OpenGRA(void)
@@ -1084,7 +1096,7 @@ save_inst(int hFile, struct objlist *ocur)
 }
 
 static void
-SaveParent(int hFile, struct objlist *parent, int storedata,
+SaveParent(int hFile, const struct objlist *parent, int storedata,
 	   int storemerge)
 {
   struct objlist *ocur, *odata, *omerge;
@@ -1121,7 +1133,7 @@ SaveParent(int hFile, struct objlist *parent, int storedata,
 }
 
 int
-SaveDrawrable(char *name, int storedata, int storemerge, int save_decimalsign)
+SaveDrawrable(const char *name, int storedata, int storemerge, int save_decimalsign)
 {
   int hFile;
   int error;
@@ -1190,15 +1202,46 @@ SaveDrawrable(char *name, int storedata, int storemerge, int save_decimalsign)
   return error;
 }
 
-static int
-get_save_opt(int *sdata, int *smerge, int *path)
+static void
+get_save_opt_response(struct response_callback *cb)
 {
-  int ret, fnum, mnum, i, src;
   struct objlist *fobj, *mobj;
+  int fnum, mnum, path;
+  int i;
+  struct GraphSave_data *save_data;
 
-  *path = SAVE_PATH_UNCHANGE;
-  *sdata = FALSE;
-  *smerge = FALSE;
+  save_data = (struct GraphSave_data *) cb->data;
+
+  if (cb->return_value != IDOK) {
+    GraphSave_response(cb->return_value, save_data);
+    return;
+  }
+
+  fobj = chkobject("data");
+  mobj = chkobject("merge");
+
+  fnum = chkobjlastinst(fobj) + 1;
+  mnum = chkobjlastinst(mobj) + 1;
+
+  path = DlgSave.Path;
+  for (i = 0; i < fnum; i++) {
+    putobj(fobj, "save_path", i, &path);
+  }
+
+  for (i = 0; i < mnum; i++) {
+    putobj(mobj, "save_path", i, &path);
+  }
+  save_data->path = path;
+  save_data->storedata = DlgSave.SaveData;
+  save_data->storemerge = DlgSave.SaveMerge;
+  GraphSave_response(cb->return_value, save_data);
+}
+
+static void
+get_save_opt(struct GraphSave_data *save_data)
+{
+  int fnum, mnum, i, src;
+  struct objlist *fobj, *mobj;
 
   fobj = chkobject("data");
   mobj = chkobject("merge");
@@ -1208,7 +1251,8 @@ get_save_opt(int *sdata, int *smerge, int *path)
 
   /* there are no instances of data and merge objects */
   if (fnum < 1 && mnum < 1) {
-    return IDOK;
+    GraphSave_response(IDOK, save_data);
+    return;
   }
 
   /* check source field of data objects */
@@ -1219,103 +1263,139 @@ get_save_opt(int *sdata, int *smerge, int *path)
     }
   }
   if (fnum > 0 && mnum < 1 && i == fnum) {
-    return IDOK;
+    GraphSave_response(IDOK, save_data);
+    return;
   }
 
-  SaveDialog(&DlgSave, sdata, smerge);
-  ret = DialogExecute(TopLevel, &DlgSave);
-  if (ret != IDOK)
-    return IDCANCEL;
+  SaveDialog(&DlgSave);
+  response_callback_add(&DlgSave, get_save_opt_response, NULL, save_data);
+  DialogExecute(TopLevel, &DlgSave);
+}
 
-  *path = DlgSave.Path;
-  for (i = 0; i < fnum; i++) {
-    putobj(fobj, "save_path", i, path);
+static void
+GraphSave_free(struct GraphSave_data *data)
+{
+  g_free(data->file);
+  g_free(data->current_wd);
+  g_free(data->prev_wd);
+  g_free(data);
+}
+
+static void
+check_graph_save_data_callback (int ret, struct GraphSave_data *save_data)
+{
+  if (save_data == NULL) {
+    return;
+  }
+  if (save_data->cb == NULL) {
+    return;
+  }
+  save_data->cb (ret, save_data->data);
+}
+
+static void
+GraphSave_response(int ret, struct GraphSave_data *d)
+{
+  if (ret == IDOK) {
+    char mes[256];
+    snprintf(mes, sizeof(mes), _("Saving `%.128s'."), d->file);
+    SetStatusBar(mes);
+    if(! SaveDrawrable(d->file, d->storedata, d->storemerge, TRUE)) {
+      switch (d->path) {
+      case SAVE_PATH_BASE:
+	  ToBasename();
+	  break;
+      case SAVE_PATH_RELATIVE:
+        ToRalativePath();
+        break;
+      case SAVE_PATH_FULL:
+        ToFullPath();
+        break;
+      }
+      changefilename(d->file);
+      AddNgpFileList(d->file);
+      SetFileName(d->file);
+      reset_graph_modified();
+    }
+    ResetStatusBar();
   }
 
-  for (i = 0; i < mnum; i++) {
-    putobj(mobj, "save_path", i, path);
+  if (d->current_wd && nchdir(d->current_wd)) {
+    ErrorMessage();
   }
+  check_graph_save_data_callback (IDOK, d);
+  GraphSave_free(d);
+}
 
-  return IDOK;
+static void
+GraphSaveSub(struct GraphSave_data *data)
+{
+  char *prev_wd;
+  char *file;
+  file = data->file;
+  if (file == NULL) {
+    if (data->cb) {
+      data->cb(IDCANCEL, data->data);
+    }
+    GraphSave_free(data);
+    return;
+  }
+  prev_wd = data->prev_wd;
+  if (prev_wd && nchdir(prev_wd)) {
+    ErrorMessage();
+  }
+  get_save_opt(data);
+}
+
+static void
+graph_save_response(char *file, gpointer user_data)
+{
+  struct GraphSave_data *data;
+  char *prev_wd, *current_wd;
+  data = (struct GraphSave_data *) user_data;
+  prev_wd = data->prev_wd;
+  current_wd = ngetcwd();
+  if (prev_wd && current_wd && strcmp(prev_wd, current_wd) == 0) {
+    g_free(prev_wd);
+    g_free(current_wd);
+    prev_wd = NULL;
+    current_wd = NULL;
+    data->prev_wd = NULL;
+  }
+  data->current_wd = current_wd;
+  data->file = file;
+  GraphSaveSub(data);
 }
 
 int
-GraphSave(int overwrite)
+GraphSave(int overwrite, response_cb cb, gpointer user_data)
 {
-  int path;
-  int sdata, smerge;
-  int ret;
-  char *initfil;
-  char *file, *prev_wd, *current_wd;
+  struct GraphSave_data *data;
+  const char *initfil;
 
+  data = g_malloc0 (sizeof (*data));
+  data->cb = cb;
+  data->data = user_data;
+  data->prev_wd = NULL;
   if (NgraphApp.FileName != NULL) {
     initfil = NgraphApp.FileName;
   } else {
     initfil = NULL;
     overwrite = FALSE;
   }
-  prev_wd = current_wd = NULL;
   if ((initfil == NULL) || (! overwrite || (naccess(initfil, 04) == -1))) {
-    prev_wd = ngetcwd();
-    ret = nGetSaveFileName(TopLevel, _("Save NGP file"), "ngp",
-			   &(Menulocal.graphloaddir), initfil,
-			   &file, overwrite, Menulocal.changedirectory);
-    current_wd = ngetcwd();
-    if (prev_wd && current_wd && strcmp(prev_wd, current_wd) == 0) {
-      g_free(prev_wd);
-      g_free(current_wd);
-      prev_wd = NULL;
-      current_wd = NULL;
-    }
+    int chd;
+    initfil = (initfil) ? initfil : "untitled.ngp";
+    data->prev_wd = ngetcwd();
+    chd = Menulocal.changedirectory;
+    nGetSaveFileName(TopLevel, _("Save NGP file"), "ngp",
+                     &(Menulocal.graphloaddir), initfil, chd,
+                     graph_save_response, data);
   } else {
-    file = g_strdup(initfil);
-    if (file == NULL)
-      return IDCANCEL;
-    ret = IDOK;
+    data->file = g_strdup (initfil);
+    GraphSaveSub(data);
   }
-
-  if (ret == IDOK) {
-    if (prev_wd && nchdir(prev_wd)) {
-      ErrorMessage();
-    }
-
-    ret = get_save_opt(&sdata, &smerge, &path);
-    if (ret == IDOK) {
-      char mes[256];
-      snprintf(mes, sizeof(mes), _("Saving `%.128s'."), file);
-      SetStatusBar(mes);
-      if(SaveDrawrable(file, sdata, smerge, TRUE)) {
-	ret = IDCANCEL;
-      } else {
-	switch (path) {
-	case SAVE_PATH_BASE:
-	  ToBasename();
-	  break;
-	case SAVE_PATH_RELATIVE:
-	  ToRalativePath();
-	  break;
-	case SAVE_PATH_FULL:
-	  ToFullPath();
-	  break;
-	}
-	changefilename(file);
-	AddNgpFileList(file);
-	SetFileName(file);
-	reset_graph_modified();
-      }
-      ResetStatusBar();
-    }
-    g_free(file);
-
-    if (current_wd && nchdir(current_wd)) {
-      ErrorMessage();
-    }
-  }
-
-  g_free(prev_wd);
-  g_free(current_wd);
-
-  return ret;
+  return IDOK;
 }
 
 static void
@@ -1381,112 +1461,52 @@ ToBasename(void)
   change_filename(get_basename);
 }
 
-
-int
-LoadNgpFile(char *file, int console, char *option)
+struct load_dialog_data
 {
-  struct objlist *sys;
-  char *expanddir;
+  char *file, *cwd;
+  int console;
+  char *option;
+};
+
+static void
+load_dialog_cb_free(struct load_dialog_data *data)
+{
+  g_free(data->file);
+  g_free(data->option);
+  g_free(data->cwd);
+  g_free(data);
+}
+
+static void
+draw_callback(gpointer user_data)
+{
+  UpdateAll2(NULL, FALSE);
+  menu_clear_undo();
+}
+
+struct LoadNgpFile_data {
   struct objlist *obj;
-  char *name;
-  int r, newid, allocnow = FALSE, tmp;
-  char *s;
-  int len;
-  char *argv[2];
-  struct narray sarray;
-  char mes[256];
   N_VALUE *inst;
-  struct objlist *robj;
-  int idn;
-  int loadpath, expand;
+  char *argv[2];
+  int r;
+  GThread *thread;
+  char *file, *name;
+  int loadpath, allocnow, newid, console;
+  struct narray sarray;
+  struct load_dialog_data *d;
+};
 
-  LoadDialog(&DlgLoad);
-  if (DialogExecute(TopLevel, &DlgLoad) != IDOK) {
-    return 1;
-  }
-  changefilename(file);
+static void
+LoadNgpFile_response_finalize(gpointer user_data)
+{
+  struct LoadNgpFile_data *data;
 
-  if (naccess(file, R_OK)) {
-    ErrorMessage();
-    return 1;
-  }
+  data = (struct LoadNgpFile_data *) user_data;
 
-  sys = chkobject("system");
-  if (sys == NULL) {
-    return 1;
-  }
-
-  loadpath = DlgLoad.loadpath;
-  expand = DlgLoad.expand;
-  expanddir = DlgLoad.exdir;
-  DlgLoad.exdir = NULL;
-  if (expanddir == NULL)
-    return 1;
-
-  putobj(sys, "expand_dir", 0, expanddir);
-  putobj(sys, "expand_file", 0, &expand);
-
-  tmp = FALSE;
-  putobj(sys, "ignore_path", 0, &tmp);
-
-  obj = chkobject("shell");
-  if (obj == NULL)
-    return 1;
-
-  newid = newobj(obj);
-  if (newid < 0)
-    return 1;
-
-  inst = chkobjinst(obj, newid);
-  arrayinit(&sarray, sizeof(char *));
-  while ((s = getitok2(&option, &len, " \t")) != NULL) {
-    if (arrayadd(&sarray, &s) == NULL) {
-      g_free(s);
-      arraydel2(&sarray);
-      delobj(obj, newid);
-      return 1;
-    }
-  }
-
-  name = g_strdup(file);
-
-  if (name == NULL) {
-    arraydel2(&sarray);
-    delobj(obj, newid);
-    return 1;
-  }
-
-  if (arrayadd(&sarray, &name) == NULL) {
-    g_free(name);
-    arraydel2(&sarray);
-    delobj(obj, newid);
-    return 1;
-  }
-
-  DeleteDrawable();
-
-  if (console) {
-    allocnow = allocate_console();
-  }
-
-  exeobj(obj, "set_security", newid, 0, NULL);
-
-  argv[0] = (char *) &sarray;
-  argv[1] = NULL;
-
-  snprintf(mes, sizeof(mes), _("Loading `%.128s'."), name);
-  SetStatusBar(mes);
-
-  menu_lock(TRUE);
-  idn = getobjtblpos(Menulocal.obj, "_evloop", &robj);
-  registerevloop(chkobjectname(Menulocal.obj), "_evloop", robj, idn, Menulocal.inst, NULL);
-
-  r = _exeobj(obj, "shell", inst, 1, argv);
-
-  unregisterevloop(robj, idn, Menulocal.inst);
+  g_thread_join(data->thread);
   menu_lock(FALSE);
 
-  if (r == 0) {
+  if (data->r == 0) {
     struct objlist *aobj;
     int i;
     if ((aobj = getobject("axis")) != NULL) {
@@ -1499,11 +1519,11 @@ LoadNgpFile(char *file, int console, char *option)
 	exeobj(aobj, "tight", i, 0, NULL);
     }
 
-    SetFileName(file);
-    AddNgpFileList(name);
+    SetFileName(data->file);
+    AddNgpFileList(data->name);
     reset_graph_modified();
 
-    switch (loadpath) {
+    switch (data->loadpath) {
     case LOAD_PATH_BASE:
       ToBasename();
       break;
@@ -1516,20 +1536,179 @@ LoadNgpFile(char *file, int console, char *option)
 
   AxisNameToGroup();
   ResetStatusBar();
-  arraydel2(&sarray);
+  arraydel2(&data->sarray);
 
-  if (console) {
-    free_console(allocnow);
+  if (data->console) {
+    free_console(data->allocnow);
   }
 
   set_axis_undo_button_sensitivity(FALSE);
   GetPageSettingsFromGRA();
-  CmViewerDraw(NULL, GINT_TO_POINTER(FALSE));
-  UpdateAll2(NULL, FALSE);
-  delobj(obj, newid);
-  menu_clear_undo();
+  Draw(FALSE, draw_callback, NULL);
+  delobj(data->obj, data->newid);
+  load_dialog_cb_free(data->d);
+  g_free(data);
+}
 
-  return 0;
+static gpointer
+LoadNgpFile_response_main(gpointer user_data)
+{
+  struct LoadNgpFile_data *data;
+  data = (struct LoadNgpFile_data *) user_data;
+  data->r = _exeobj(data->obj, "shell", data->inst, 1, data->argv);
+  data->thread = g_thread_self();
+  g_idle_add_once(LoadNgpFile_response_finalize, data);
+  return NULL;
+}
+
+static void
+LoadNgpFile_response(struct response_callback *cb)
+{
+  char *file;
+  char *option;
+  int console;
+  struct load_dialog_data *d;
+  struct objlist *sys;
+  char *expanddir;
+  struct objlist *obj;
+  char *name;
+  int newid, allocnow = FALSE, tmp;
+  char *s;
+  int len;
+  char mes[256];
+  N_VALUE *inst;
+  int loadpath, expand;
+  struct LoadNgpFile_data *data;
+
+  d = (struct load_dialog_data *) cb->data;
+
+  file = d->file;
+  console = d->console;
+  option = d->option;
+
+  data = g_malloc0(sizeof(*data));
+  if (data == NULL) {
+    goto ErrorExit;
+  }
+
+  if (DlgLoad.ret != IDOK) {
+    goto ErrorExit;
+  }
+  changefilename(file);
+
+  if (naccess(file, R_OK)) {
+    ErrorMessage();
+    goto ErrorExit;
+  }
+
+  sys = chkobject("system");
+  if (sys == NULL) {
+    goto ErrorExit;
+  }
+
+  loadpath = DlgLoad.loadpath;
+  expand = DlgLoad.expand;
+  expanddir = DlgLoad.exdir;
+  DlgLoad.exdir = NULL;
+  if (expanddir == NULL) {
+    goto ErrorExit;
+  }
+
+  putobj(sys, "expand_dir", 0, expanddir);
+  putobj(sys, "expand_file", 0, &expand);
+
+  tmp = FALSE;
+  putobj(sys, "ignore_path", 0, &tmp);
+
+  obj = chkobject("shell");
+  if (obj == NULL) {
+    goto ErrorExit;
+  }
+
+  newid = newobj(obj);
+  if (newid < 0) {
+    goto ErrorExit;
+  }
+
+  inst = chkobjinst(obj, newid);
+  arrayinit(&data->sarray, sizeof(char *));
+  while ((s = getitok2(&option, &len, " \t")) != NULL) {
+    if (arrayadd(&data->sarray, &s) == NULL) {
+      g_free(s);
+      arraydel2(&data->sarray);
+      delobj(obj, newid);
+      goto ErrorExit;
+    }
+  }
+
+  name = g_strdup(file);
+
+  if (name == NULL) {
+    arraydel2(&data->sarray);
+    delobj(obj, newid);
+    goto ErrorExit;
+  }
+
+  if (arrayadd(&data->sarray, &name) == NULL) {
+    g_free(name);
+    arraydel2(&data->sarray);
+    delobj(obj, newid);
+    goto ErrorExit;
+  }
+
+  DeleteDrawable();
+
+  if (console) {
+    allocnow = allocate_console();
+  }
+
+  exeobj(obj, "set_security", newid, 0, NULL);
+
+  data->argv[0] = (char *) &data->sarray;
+  data->argv[1] = NULL;
+
+  snprintf(mes, sizeof(mes), _("Loading `%.128s'."), name);
+  SetStatusBar(mes);
+
+  menu_lock(TRUE);
+
+  data->obj = obj;
+  data->inst = inst;
+  data->file = file;
+  data->name = name;
+  data->loadpath = loadpath;
+  data->allocnow = allocnow;
+  data->console = console;
+  data->newid = newid;
+  data->d = d;
+  g_thread_new(NULL, LoadNgpFile_response_main, data);
+  return;
+
+ ErrorExit:
+  g_free(data);
+  if (d->cwd) {
+    nchdir(d->cwd);
+  }
+  load_dialog_cb_free(d);
+}
+
+void
+LoadNgpFile(const char *file, int console, const char *option, const char *cwd)
+{
+  struct load_dialog_data *data;
+
+  data = g_malloc0(sizeof(* data));
+  if (data == NULL) {
+    return;
+  }
+  data->file = g_strdup(file);
+  data->console = console;
+  data->option = g_strdup(option);
+  data->cwd = g_strdup(cwd);
+
+  response_callback_add(&DlgLoad, LoadNgpFile_response, NULL, data);
+  LoadDialog(&DlgLoad, file);
+  DialogExecute(TopLevel, &DlgLoad);
 }
 
 static int
@@ -1657,20 +1836,60 @@ AdjustAxis(void)
     exeobj(aobj, "adjust", i, 0, NULL);
 }
 
-int
-CheckSave(void)
+struct CheckSave_data {
+  response_cb cb;
+  gpointer data;
+};
+
+static void
+CheckSave_response_response (int ret, gpointer user_data)
 {
+  struct CheckSave_data *data;
+  int r = TRUE;
+
+  data = (struct CheckSave_data *) user_data;
+  if (ret == IDCANCEL) {
+    r = FALSE;
+  }
+  data->cb(r, data->data);
+  g_free(data);
+}
+
+static void
+CheckSave_response(int ret, gpointer user_data)
+{
+  struct CheckSave_data *data;
+  int r = TRUE;
+
+  data = (struct CheckSave_data *) user_data;
+  if (ret == IDYES) {
+    GraphSave(TRUE, CheckSave_response_response, user_data);
+    return;
+  } else if (ret != IDNO) {
+    r = FALSE;
+  }
+  data->cb(r, data->data);
+  g_free(data);
+}
+
+void
+CheckSave(response_cb cb, gpointer user_data)
+{
+  struct CheckSave_data *data;
+  data = g_malloc0(sizeof(*data));
+  if (data == NULL) {
+    return;
+  }
+  data->cb = cb;
+  data->data = user_data;
   if (get_graph_modified()) {
-    int ret;
-    ret = message_box(TopLevel, _("This graph is modified.\nSave this graph?"),
-		      _("Confirm"), RESPONS_YESNOCANCEL);
-    if (ret == IDYES) {
-      if (GraphSave(TRUE) == IDCANCEL)
-	return FALSE;
-    } else if (ret != IDNO)
-      return FALSE;
+    response_message_box(TopLevel,
+			 _("This graph is modified.\nSave this graph?"),
+			 _("Confirm"), RESPONS_YESNOCANCEL,
+			 CheckSave_response, data);
+  } else {
+    CheckSave_response(IDNO, data);
   }
-  return TRUE;
 }
 
 static void
@@ -1701,6 +1920,7 @@ add_hist(const char *file, char *mime)
   uri = g_filename_to_uri(full_name, NULL, NULL);
   g_free(full_name);
 
+  gtk_recent_manager_remove_item (NgraphApp.recent_manager, uri, NULL);
   gtk_recent_manager_add_full(NgraphApp.recent_manager, uri, &recent_data);
   g_free(uri);
 }
@@ -1824,150 +2044,331 @@ PlotFileCB(struct objlist *obj, int id)
   return get_plot_cb_str(obj, id, source);
 }
 
-int
-SetFileHidden(void)
+struct set_file_hidden_data {
+  response_cb cb;
+  gpointer user_data;
+};
+
+static void
+set_file_hidden_response(struct response_callback *cb)
 {
-  struct objlist *fobj;
-  int lastinst;
-  struct narray farray, ifarray;
-  int i, a, r;
-
-  fobj = chkobject("data");
-  if (fobj == NULL) {
-    return 1;
-  }
-
-  lastinst = chkobjlastinst(fobj);
-  if (lastinst < 0) {
-    return 1;
-  }
-
-  arrayinit(&ifarray, sizeof(int));
-  for (i = 0; i <= lastinst; i++) {
-    getobj(fobj, "hidden", i, 0, NULL, &a);
-    if (!a) {
-      arrayadd(&ifarray, &i);
-    }
-  }
+  struct SelectDialog *d;
+  struct narray *farray, *ifarray;
+  int a, r;
 
+  d = (struct SelectDialog *) cb->dialog;
+  farray = d->sel;
+  ifarray = d->isel;
   r = 0;
-  SelectDialog(&DlgSelect, fobj, NULL, FileCB, &farray, &ifarray);
-  if (DialogExecute(TopLevel, &DlgSelect) == IDOK) {
-    int num, inum, *array, undo, modified = FALSE;
-    a = TRUE;
+  if (cb->return_value == IDOK) {
+    int num, inum, *array, lastinst, undo, modified = FALSE, i;
+    struct objlist *fobj;
     undo = menu_save_undo_single(UNDO_TYPE_EDIT, "data");
+    fobj = chkobject("data");
+    lastinst = chkobjlastinst(fobj);
+    a = TRUE;
     for (i = 0; i <= lastinst; i++) {
       putobj(fobj, "hidden", i, &a);
     }
-    num = arraynum(&farray);
-    array = arraydata(&farray);
+    num = arraynum(farray);
+    arraysort_int(farray);
+    array = arraydata(farray);
     a = FALSE;
     for (i = 0; i < num; i++) {
       putobj(fobj, "hidden", array[i], &a);
     }
 
-    inum = arraynum(&ifarray);
+    inum = arraynum(ifarray);
     if (inum != num) {
       modified = TRUE;
     } else {
       for (i = 0; i < num; i++) {
-	if (arraynget_int(&ifarray, i) != array[i]) {
+	if (arraynget_int(ifarray, i) != array[i]) {
 	  modified = TRUE;
 	  break;
 	}
       }
     }
-    r = 1;
     if (modified) {
       set_graph_modified();
     } else {
       menu_undo_internal(undo);
     }
+    r = 1;
   }
 
-  arraydel(&ifarray);
-  arraydel(&farray);
-  return r;
+  arrayfree(ifarray);
+  arrayfree(farray);
+
+  if (cb->data) {
+    struct set_file_hidden_data *data;
+    data = (struct set_file_hidden_data *) cb->data;
+    if (data->cb) {
+      data->cb(r, data->user_data);
+    }
+    g_free(data);
+  }
 }
 
-int
-CheckIniFile(void)
+void
+SetFileHidden(response_cb cb, gpointer user_data)
+{
+  struct objlist *fobj;
+  int lastinst;
+  struct narray *farray, *ifarray;
+  int i, a;
+  struct set_file_hidden_data *data;
+
+  fobj = chkobject("data");
+  if (fobj == NULL) {
+    if (cb) {
+      cb(1, user_data);
+    }
+    return;
+  }
+
+  lastinst = chkobjlastinst(fobj);
+  if (lastinst < 0) {
+    if (cb) {
+      cb(1, user_data);
+    }
+    return;
+  }
+
+  data = g_malloc0(sizeof(*data));
+  if (data == NULL) {
+    if (cb) {
+      cb(1, user_data);
+    }
+    return;
+  }
+  data->cb = cb;
+  data->user_data = user_data;
+
+  ifarray = arraynew(sizeof(int));
+  if (ifarray == NULL) {
+    if (cb) {
+      cb(1, user_data);
+    }
+    g_free(data);
+    return;
+  }
+  farray = arraynew(sizeof(int));
+  if (farray == NULL) {
+    if (cb) {
+      cb(1, user_data);
+    }
+    g_free(data);
+    arrayfree(ifarray);
+    return;
+  }
+  for (i = 0; i <= lastinst; i++) {
+    getobj(fobj, "hidden", i, 0, NULL, &a);
+    if (!a) {
+      arrayadd(ifarray, &i);
+    }
+  }
+
+  SelectDialog(&DlgSelect, fobj, NULL, FileCB, farray, ifarray);
+  response_callback_add(&DlgSelect, set_file_hidden_response, NULL, data);
+  DialogExecute(TopLevel, &DlgSelect);
+}
+
+struct CheckIniFile_data {
+  struct objlist *obj;
+  int id, modified;
+  obj_response_cb cb;
+};
+
+static void
+CheckIniFile_response(int ret, gpointer user_data)
+{
+  struct objlist *obj;
+  int id, modified;
+  obj_response_cb cb;
+  struct CheckIniFile_data *data;
+
+  data = (struct CheckIniFile_data *) user_data;
+  id = data->id;
+  obj = data->obj;
+  cb = data->cb;
+  modified = data->modified;
+  g_free(data);
+
+  if (ret != IDYES) {
+    cb(FALSE, obj, id, modified);
+    return;
+  }
+  if (!copyconfig()) {
+    message_box(TopLevel, _("Ngraph.ini could not be copied."), "Ngraph.ini", RESPONS_ERROR);
+    cb(FALSE, obj, id, modified);
+    return;
+  }
+  cb(TRUE, obj, id, modified);
+}
+
+void
+CheckIniFile(obj_response_cb cb, struct objlist *obj, int id, int modified)
 {
   int ret;
 
   ret = writecheckconfig();
   if (ret == 0) {
     message_box(TopLevel, _("Ngraph.ini is not found."), "Ngraph.ini", RESPONS_ERROR);
-    return FALSE;
+    cb(FALSE, obj, id, modified);
+    return;
   } else if ((ret == -1) || (ret == -3)) {
     message_box(TopLevel, _("Ngraph.ini is write protected."), "Ngraph.ini", RESPONS_ERROR);
-    return FALSE;
+    cb(FALSE, obj, id, modified);
+    return;
   } else if ((ret == -2) || (ret == 2)) {
+    struct CheckIniFile_data *data;
     struct objlist *sys;
     char *homedir, *buf;
 
     sys = getobject("system");
     if (sys == NULL) {
-      return FALSE;
+      cb(FALSE, obj, id, modified);
+      return;
     }
 
     if (getobj(sys, "home_dir", 0, 0, NULL, &homedir) == -1) {
-      return FALSE;
+      cb(FALSE, obj, id, modified);
+      return;
     }
 
+    data = g_malloc0(sizeof(*data));
+    if (data == NULL) {
+      cb(FALSE, obj, id, modified);
+      return;
+    }
+    data->cb = cb;
+    data->obj = obj;
+    data->id = id;
+    data->modified = modified;
+
     buf = g_strdup_printf(_("Install `Ngraph.ini' to %s ?"), homedir);
-    if (message_box(TopLevel, buf, "Ngraph.ini", RESPONS_YESNO) == IDYES) {
-      g_free(buf);
-      if (!copyconfig()) {
-	message_box(TopLevel, _("Ngraph.ini could not be copied."), "Ngraph.ini", RESPONS_ERROR);
-	return FALSE;
-      }
-    } else {
-      g_free(buf);
-      return FALSE;
-    }
+    response_message_box(TopLevel, buf, "Ngraph.ini", RESPONS_YESNO, CheckIniFile_response, data);
+    g_free(buf);
+  } else {
+    cb(TRUE, obj, id, modified);
   }
-  return TRUE;
 }
 
+struct progress_dialog_data {
+  GString *msg[2], *title;
+  double fraction[2];
+  guint tag;
+  GThread *thread;
+  progress_func update, finalize;
+  int finish;
+};
+static struct progress_dialog_data * ProgressDialogData = NULL;
+
 void
-ProgressDialogSetTitle(char *title)
+ProgressDialogSetTitle(const char *title)
 {
-  if (ProgressDialog)
-    gtk_window_set_title(GTK_WINDOW(ProgressDialog), title);
+  if (ProgressDialogData) {
+    g_string_assign(ProgressDialogData->title, title);
+  }
 }
 
 static void
 show_progress(int pos, const char *msg, double fraction)
 {
-  GtkProgressBar *bar;
+  int i;
 
   if (! ProgressDialog)
     return;
 
-  if (pos) {
-    bar = ProgressBar2;
-  } else {
-    bar = ProgressBar;
-  }
+  if (! ProgressDialogData)
+    return;
 
-  if (fraction < 0) {
-    gtk_progress_bar_pulse(bar);
-  } else {
-    gtk_progress_bar_set_fraction(bar, fraction);
-  }
+  i = (pos) ? 1 : 0;
 
-  gtk_progress_bar_set_text(bar, msg);
+  g_string_assign(ProgressDialogData->msg[i], msg);
+  ProgressDialogData->fraction[i] = fraction;
 }
 
 static void
 stop_btn_clicked(GtkButton *button, gpointer user_data)
 {
+  if (ProgressDialogData == NULL) {
+    return;
+  }
+  if (ProgressDialogData->finish) {
+    ProgressDialogFinalize();
+    return;
+  }
   set_interrupt();
 }
 
-void
-ProgressDialogCreate(char *title)
+static void
+progress_dialog_finalize(gpointer user_data)
+{
+  g_thread_join(ProgressDialogData->thread);
+  if (ProgressDialogData->finalize) {
+    ProgressDialogData->finalize(user_data);
+  }
+  if (gtk_widget_get_visible(GTK_WIDGET(ProgressFrame))) {
+    gtk_progress_bar_set_fraction(ProgressBar[0], 1);
+    gtk_progress_bar_set_fraction(ProgressBar[1], 1);
+    gtk_window_set_title(GTK_WINDOW(ProgressDialog), _("Drawing end"));
+    gtk_button_set_label(GTK_BUTTON(ProgressButton), _("_OK"));
+  } else {
+    ProgressDialogFinalize();
+  }
+}
+
+static gboolean
+progress_dialog_update(gpointer user_data)
+{
+  int i;
+  if (ProgressDialogData == NULL) {
+    return FALSE;
+  } else if (ProgressDialogData->finish) {
+    progress_dialog_finalize(user_data);
+    return FALSE;
+  }
+  for (i = 0; i < PROGRESSBAR_N; i++) {
+    GtkProgressBar *bar;
+    double fraction;
+    const char *msg, *title;
+    bar = ProgressBar[i];
+    fraction = ProgressDialogData->fraction[i];
+    msg = ProgressDialogData->msg[i]->str;
+    title = ProgressDialogData->title->str;
+    if (fraction < 0) {
+      gtk_progress_bar_pulse(bar);
+    } else {
+      gtk_progress_bar_set_fraction(bar, fraction);
+    }
+    gtk_progress_bar_set_text(bar, msg);
+    gtk_window_set_title(GTK_WINDOW(ProgressDialog), title);
+  }
+  return TRUE;
+}
+
+static void
+progress_dialog_add_text_view(GtkWidget *vbox)
+{
+  GtkWidget *swin;
+  swin = gtk_scrolled_window_new();
+  gtk_widget_set_vexpand(swin, TRUE);
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+
+  ProgressText = gtk_text_view_new();
+  gtk_text_view_set_editable(GTK_TEXT_VIEW(ProgressText), FALSE);
+  gtk_scrolled_window_set_child(GTK_SCROLLED_WINDOW(swin), ProgressText);
+
+  ProgressFrame = gtk_frame_new(_("Error messages"));
+  gtk_frame_set_child(GTK_FRAME(ProgressFrame), swin);
+
+  gtk_box_append(GTK_BOX(vbox), ProgressFrame);
+}
+
+static void
+create_progress_dialog(const char *title)
 {
   GtkWidget *btn, *vbox, *hbox;
 
@@ -1979,71 +2380,161 @@ ProgressDialogCreate(char *title)
   SaveCursor = NGetCursor();
   NSetCursor(GDK_WATCH);
 
-  set_draw_lock(DrawLockDraw);
-
   if (ProgressDialog) {
+    GtkTextBuffer *buf;
+    buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ProgressText));
+    gtk_text_buffer_set_text(buf, "", -1);
+    gtk_widget_set_visible(ProgressDialog, FALSE);
+
     ProgressDialogSetTitle(title);
     show_progress(0, "", 0);
     show_progress(1, "", 0);
     set_progress_func(show_progress);
-    gtk_widget_show_all(ProgressDialog);
     return;
   }
 
-  ProgressDialog = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-  g_signal_connect(ProgressDialog, "delete-event", G_CALLBACK(gtk_true), NULL);
+  ProgressDialog = gtk_window_new();
+  g_signal_connect(ProgressDialog, "close_request", G_CALLBACK(gtk_true), NULL);
   gtk_window_set_title(GTK_WINDOW(ProgressDialog), title);
+  gtk_window_set_deletable(GTK_WINDOW(ProgressDialog), FALSE);
 
   gtk_window_set_transient_for(GTK_WINDOW(ProgressDialog), GTK_WINDOW(TopLevel));
   gtk_window_set_modal(GTK_WINDOW(ProgressDialog), TRUE);
-  gtk_window_set_position(GTK_WINDOW(ProgressDialog), GTK_WIN_POS_CENTER);
-  gtk_window_set_type_hint(GTK_WINDOW(ProgressDialog), GDK_WINDOW_TYPE_HINT_DIALOG);
 
   vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
 
-  ProgressBar = GTK_PROGRESS_BAR(gtk_progress_bar_new());
-  gtk_progress_bar_set_ellipsize(ProgressBar, PANGO_ELLIPSIZE_MIDDLE);
-  gtk_progress_bar_set_show_text(ProgressBar, TRUE);
-  gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(ProgressBar), FALSE, FALSE, 4);
+  ProgressBar[0] = GTK_PROGRESS_BAR(gtk_progress_bar_new());
+  gtk_progress_bar_set_ellipsize(ProgressBar[0], PANGO_ELLIPSIZE_MIDDLE);
+  gtk_progress_bar_set_show_text(ProgressBar[0], TRUE);
+  gtk_box_append(GTK_BOX(vbox), GTK_WIDGET(ProgressBar[0]));
 
-  ProgressBar2 = GTK_PROGRESS_BAR(gtk_progress_bar_new());
-  gtk_progress_bar_set_ellipsize(ProgressBar2, PANGO_ELLIPSIZE_MIDDLE);
-  gtk_progress_bar_set_show_text(ProgressBar2, TRUE);
-  gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(ProgressBar2), FALSE, FALSE, 4);
+  ProgressBar[1] = GTK_PROGRESS_BAR(gtk_progress_bar_new());
+  gtk_progress_bar_set_ellipsize(ProgressBar[1], PANGO_ELLIPSIZE_MIDDLE);
+  gtk_progress_bar_set_show_text(ProgressBar[1], TRUE);
+  gtk_box_append(GTK_BOX(vbox), GTK_WIDGET(ProgressBar[1]));
+
+  progress_dialog_add_text_view(vbox);
 
   btn = gtk_button_new_with_mnemonic(_("_Stop"));
-  set_button_icon(btn, "process-stop");
   g_signal_connect(btn, "clicked", G_CALLBACK(stop_btn_clicked), NULL);
+  ProgressButton = btn;
 #if USE_HEADER_BAR
   hbox = gtk_header_bar_new();
-  gtk_header_bar_set_title(GTK_HEADER_BAR(hbox), title);
   gtk_header_bar_pack_end(GTK_HEADER_BAR(hbox), btn);
   gtk_window_set_titlebar(GTK_WINDOW(ProgressDialog), hbox);
 #else
   hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
 
-  gtk_box_pack_end(GTK_BOX(hbox), btn, FALSE, FALSE, 4);
-
-  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 4);
+  gtk_box_append(GTK_BOX(hbox), btn);
+  gtk_box_append(GTK_BOX(vbox), hbox);
 #endif
-  gtk_container_add(GTK_CONTAINER(ProgressDialog), vbox);
+  gtk_window_set_child(GTK_WINDOW(ProgressDialog), vbox);
 
   gtk_window_set_default_size(GTK_WINDOW(ProgressDialog), 400, -1);
-  gtk_widget_show_all(ProgressDialog);
 
   set_progress_func(show_progress);
 }
 
+static gpointer
+ProgressDialog_thread(gpointer user_data)
+{
+  if (ProgressDialogData->update) {
+    ProgressDialogData->update(user_data);
+  }
+  ProgressDialogData->finish = TRUE;
+  return NULL;
+}
+
+static void
+progress_dialog_set_text(gpointer user_data)
+{
+  GtkTextBuffer *buf;
+  GtkTextIter iter;
+  char *text;
+
+  text = (char *) user_data;
+
+  if (text == NULL) {
+    return;
+  }
+
+  if (ProgressText == NULL) {
+    g_free(text);
+    return;
+  }
+
+  gtk_widget_set_visible(ProgressFrame, TRUE);
+
+  buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ProgressText));
+  gtk_text_buffer_get_end_iter(buf, &iter);
+  gtk_text_buffer_insert(buf, &iter, text, -1);
+
+  g_free(text);
+}
+
+int
+ProgressDialogIsActive(void)
+{
+  if (ProgressDialogData == NULL) {
+    return FALSE;
+  }
+  return ! ProgressDialogData->finish;
+}
+
 void
+ProgressDialog_append_text(const char *text)
+{
+  char *str;
+  str = g_strdup(text);
+  g_idle_add_once(progress_dialog_set_text, str);
+}
+
+void
+ProgressDialogCreate(const char *title, progress_func update, progress_func finalize, gpointer data)
+{
+  int i;
+  if (ProgressDialogData) {
+    finalize(data);
+    return;
+  }
+  ProgressDialogData = g_malloc0(sizeof(*ProgressDialogData));
+  if (ProgressDialogData == NULL) {
+    return;
+  }
+  for (i = 0; i < PROGRESSBAR_N; i++) {
+    ProgressDialogData->msg[i] = g_string_new("");
+    ProgressDialogData->fraction[i] = 0;
+  }
+  ProgressDialogData->title = g_string_new("");
+  ProgressDialogData->update = update;
+  ProgressDialogData->finalize = finalize;
+  ProgressDialogData->finish = FALSE;
+  create_progress_dialog(title);
+  gtk_window_present(GTK_WINDOW(ProgressDialog));
+  gtk_widget_set_visible(ProgressFrame, FALSE);
+  gtk_button_set_label(GTK_BUTTON(ProgressButton), _("_Stop"));
+  ProgressDialogData->thread = g_thread_new(NULL, ProgressDialog_thread, data);
+
+  ProgressDialogData->tag = g_timeout_add(100, progress_dialog_update, data);
+}
+
+static void
 ProgressDialogFinalize(void)
 {
+  int i;
   if (TopLevel == NULL)
     return;
 
-  gtk_widget_hide(ProgressDialog);
+  gtk_widget_set_visible(ProgressDialog, FALSE);
   NSetCursor(SaveCursor);
   set_progress_func(NULL);
-  set_draw_lock(DrawLockNone);
+
+  for (i = 0; i < PROGRESSBAR_N; i++) {
+    g_string_free(ProgressDialogData->msg[i], TRUE);
+  }
+  g_string_free(ProgressDialogData->title, TRUE);
+  g_free(ProgressDialogData);
+  ProgressDialogData = NULL;
 }
 
 void
