@@ -57,8 +57,9 @@ enum AXIS_COMBO_BOX_FLAGS {
 #define N_RESPONSE_ALL 1
 
 void initdialog(void);
-int CopyClick(GtkWidget *parent, struct objlist *obj, int Id,
-	      char *(*callback) (struct objlist *, int));
+void CopyClick(GtkWidget *parent, struct objlist *obj, int Id,
+	       char *(*callback) (struct objlist *, int),
+               response_cb response_cb, gpointer user_data);
 int SetObjFieldFromWidget(GtkWidget *w, struct objlist *Obj, int Id, char *field);
 void SetWidgetFromObjField(GtkWidget *w, struct objlist *Obj, int Id, char *field);
 int SetObjPointsFromText(GtkWidget *w, struct objlist *Obj, int Id, char *field);
@@ -70,24 +71,44 @@ void axis_combo_box_setup(GtkWidget *cbox, struct objlist *obj, int id, const ch
 int SetObjAxisFieldFromWidget(GtkWidget *w, struct objlist *obj, int id, char *field);
 struct compatible_font_info *SetFontListFromObj(GtkWidget *w, struct objlist *obj, int id, const char *name);
 void SetObjFieldFromFontList(GtkWidget *w, struct objlist *obj, int id, char *name);
-void set_color(GtkWidget *w, struct objlist *obj, int id, char *prefix);
-void set_color2(GtkWidget *w, struct objlist *obj, int id);
-void set_fill_color(GtkWidget *w, struct objlist *obj, int id);
-void set_stroke_color(GtkWidget *w, struct objlist *obj, int id);
-int putobj_color(GtkWidget *w, struct objlist *obj, int id, char *prefix);
-int putobj_color2(GtkWidget *w, struct objlist *obj, int id);
-int putobj_fill_color(GtkWidget *w, struct objlist *obj, int id);
-int putobj_stroke_color(GtkWidget *w, struct objlist *obj, int id);
+void set_color(GtkWidget *w, GtkWidget *aw, struct objlist *obj, int id, char *prefix);
+void set_color2(GtkWidget *w, GtkWidget *aw, struct objlist *obj, int id);
+void set_fill_color(GtkWidget *w, GtkWidget *aw, struct objlist *obj, int id);
+void set_stroke_color(GtkWidget *w, GtkWidget *aw, struct objlist *obj, int id);
+int putobj_color(GtkWidget *w, GtkWidget *aw, struct objlist *obj, int id, const char *prefix);
+int putobj_color2(GtkWidget *w, GtkWidget *aw, struct objlist *obj, int id);
+int putobj_fill_color(GtkWidget *w, GtkWidget *aw, struct objlist *obj, int id);
+int putobj_stroke_color(GtkWidget *w, GtkWidget *aw, struct objlist *obj, int id);
 int chk_sputobjfield(struct objlist *obj, int id, char *field, char *str);
 int get_style_index(struct objlist *obj, int id, char *field);
 const char *get_style_string(struct objlist *obj, int id, char *field);
 
-#define DIALOG_PROTOTYPE GtkWidget *parent, *widget, *focus;	\
-  GtkBox *vbox;\
-  int ret, show_cancel;\
-  char *resource, *ok_button;\
-  void (*SetupWindow) (GtkWidget *w, void *data, int makewidget);\
-  void (*CloseWindow) (GtkWidget *w, void *data);\
+struct DialogType;
+struct response_callback;
+
+typedef void (* response_callback_func) (struct response_callback *);
+typedef void (* response_callback_free_func) (struct response_callback *);
+
+struct response_callback
+{
+  response_callback_func cb;
+  response_callback_free_func free;
+  gpointer data;
+  int return_value;
+  struct DialogType *dialog;
+};
+
+void response_callback_add(void *dialog, response_callback_func cb, response_callback_free_func free, gpointer data);
+
+#define DIALOG_PROTOTYPE GtkWidget *parent, *widget, *focus;            \
+  GtkBox *vbox;                                                         \
+  struct response_callback *response_cb;                                \
+  int ret, show_cancel;                                                 \
+  char *resource, *ok_button;                                           \
+  void (*SetupWindow) (GtkWidget *w, void *data, int makewidget);       \
+  void (*CloseWindow) (GtkWidget *w, void *data);                       \
+  int lockstate, menulock, modified;                                    \
+  GtkWidget *win_ptr, *ok;						\
 
 
 struct DialogType
@@ -95,15 +116,6 @@ struct DialogType
   DIALOG_PROTOTYPE;
 };
 
-struct MarkDialog
-{
-  DIALOG_PROTOTYPE;
-  /****** local member *******/
-  GtkWidget *toggle[MARK_TYPE_NUM];
-  int Type, cb_respond;
-};
-void MarkDialog(struct MarkDialog *data, GtkWidget *parent, int type);
-
 struct FileMath
 {
   GtkWidget *xsmooth, *x, *ysmooth, *averaging_type, *y, *f, *g, *h, *text_x, *text_y, *text_f, *text_g, *text_h;
@@ -133,14 +145,13 @@ struct FileDialog
   DIALOG_PROTOTYPE;
   /****** local member *******/
   GtkWidget *file, *load_settings, *fit, *xcol, *xaxis, *ycol, *yaxis,
-    *type, *mark_btn, *curve, *col1, *col2, *math_input_tab,
+    *type, *mark_btn, *curve, *col1, *col2, *alpha1, *alpha2, *math_input_tab,
     *clip, *style, *size, *miter, *join, *min, *max, *div,
-    *comment_box, *file_box, *fit_table, *width, *apply_all, *comment_view, *comment_table;
+    *comment_box, *file_box, *fit_table, *width, *comment_view, *comment_table;
   GtkNotebook *tab, *math_tab;
   struct objlist *Obj;
   int Id, source, math_page;
-  struct MarkDialog mark;
-  int R, G, B, R2, G2, B2, alpha, multi_open, fit_row, initialized;
+  int multi_open, fit_row, initialized;
   struct FileMath math;
   struct FileLoad load;
   struct FileMask mask;
@@ -168,7 +179,7 @@ struct MathDialog
   /****** local member *******/
   GtkWidget *list, *func[MATH_FNC_NUM];
   struct objlist *Obj;
-  int Mode, modified;
+  int Mode;
 };
 void MathDialog(struct MathDialog *data, struct objlist *obj);
 
@@ -177,12 +188,11 @@ struct MathTextDialog
   DIALOG_PROTOTYPE;
   /****** local member *******/
   GtkWidget *list, *text, *tree, *input_tab;
-  GList *id_list;
   struct objlist *Obj;
   char *Text;
-  int Mode, modified, page;
+  int Mode, page;
 };
-void MathTextDialog(struct MathTextDialog *data, char *text, int mode, struct objlist *obj, GList *list, GtkWidget *tree);
+void MathTextDialog(struct MathTextDialog *data, char *text, int mode, struct objlist *obj, GtkWidget *tree);
 
 #define FIT_PARM_NUM FIT_DIMENSION_MAX
 
@@ -200,17 +210,6 @@ struct FitDialog
 };
 void FitDialog(struct FitDialog *data, struct objlist *obj, int id);
 
-struct FitLoadDialog
-{
-  DIALOG_PROTOTYPE;
-  /****** local member *******/
-  GtkWidget *list;
-  struct objlist *Obj;
-  int Sid;
-  int sel;
-};
-void FitLoadDialog(struct FitLoadDialog *data, struct objlist *obj, int sid);
-
 struct FitSaveDialog
 {
   DIALOG_PROTOTYPE;
@@ -230,14 +229,14 @@ struct SectionDialog
     *width, *height, *xaxis, *yaxis, *uaxis, *raxis, *grid;
   int Section;
   int X, Y, LenX, LenY, X0, Y0, LenX0, LenY0;
-  int IDX, IDY, IDU, IDR, *IDG;
+  int IDX, IDY, IDU, IDR, IDG;
   struct objlist *Obj, *Obj2;
   int MaxX, MaxY;
 };
 void SectionDialog(struct SectionDialog *data,
 		   int x, int y, int lenx, int leny,
 		   struct objlist *obj, int idx, int idy, int idu, int idr,
-		   struct objlist *obj2, int *idg, int section);
+		   struct objlist *obj2, int idg, int section);
 
 struct CrossDialog
 {
@@ -257,9 +256,9 @@ void CrossDialog(struct CrossDialog *data,
 
 struct AxisBase
 {
-  GtkWidget *style, *width, *color, *arrow, *arrowlen, *arrowwid,
+  GtkWidget *style, *width, *color, *alpha, *arrow, *arrowlen, *arrowwid,
     *wave, *wavelen, *wavewid, *baseline;
-  int R, G, B, alpha, tab_id;
+  int tab_id;
 };
 
 struct AxisPos
@@ -270,8 +269,8 @@ struct AxisPos
 
 struct AxisFont
 {
-  GtkWidget *space, *pt, *script, *font, *color, *font_bold, *font_italic;
-  int R, G, B, alpha, tab_id;
+  GtkWidget *space, *pt, *script, *font, *color, *alpha, *font_bold, *font_italic;
+  int tab_id;
 };
 
 struct AxisNumbering
@@ -286,8 +285,8 @@ struct AxisNumbering
 struct AxisGauge
 {
   GtkWidget *length[GAUGE_STYLE_NUM], *width[GAUGE_STYLE_NUM],
-    *gauge, *min, *max, *style, *color;
-  int R, G, B, alpha, tab_id;
+    *gauge, *min, *max, *style, *color, *alpha;
+  int tab_id;
 };
 
 struct AxisDialog
@@ -314,7 +313,7 @@ struct GridDialog
   /****** local member *******/
   struct objlist *Obj;
   GtkWidget *style[GRID_DIALOG_STYLE_NUM], *width[GRID_DIALOG_STYLE_NUM],
-    *axisx, *axisy, *background, *color, *bcolor, *draw_x, *draw_y;
+    *axisx, *axisy, *background, *color, *bcolor, *alpha, *balpha, *draw_x, *draw_y;
   int Id;
   int R, G, B, R2, G2, B2, A;
 };
@@ -355,7 +354,7 @@ struct LegendDialog
   /****** local member *******/
   char *(* prop_cb) (struct objlist *obj, int id);
   GtkWidget *path_type, *style, *points, *interpolation, *width,
-    *miter, *join, *color, *color2, *stroke_color, *fill_color,
+    *miter, *join, *color, *color2, *alpha, *alpha2, *stroke_color, *stroke_alpha, *fill_color, *fill_alpha,
     *x, *y, *x1, *y1, *x2, *y2, *rx, *ry, *angle1, *angle2,
     *pieslice, *close_path, *stroke, *fill, *fill_rule,
     *marker_begin, *marker_end, *arrow_length, *arrow_width,
@@ -364,8 +363,7 @@ struct LegendDialog
     *font_italic;
   struct objlist *Obj;
   int Id;
-  int R, G, B, R2, G2, B2, fill_R, fill_G, fill_B, alpha, wid, ang;
-  struct MarkDialog mark, mark_begin, mark_end;
+  int R, G, B, R2, G2, B2, fill_R, fill_G, fill_B, A, A2, fill_A, wid, ang;
   cairo_surface_t *arrow_pixmap;
 };
 
@@ -385,7 +383,7 @@ struct LegendGaussDialog
   DIALOG_PROTOTYPE;
   /****** local member *******/
   GSList *func_list, *dir_list;
-  GtkWidget *style, *div, *color, *width, *miter, *join, *sch, *scv, *view;
+  GtkWidget *style, *div, *color, *alpha, *width, *miter, *join, *sch, *scv, *view;
   int R, G, B, A;
   struct objlist *Obj;
   int Id;
@@ -433,11 +431,11 @@ struct LoadDialog
   /****** local member *******/
   GtkWidget *expand_file, *load_path, *dir;
   int expand;
-  char *exdir;
+  char *exdir, *file;
   int loadpath;
   int Id;
 };
-void LoadDialog(struct LoadDialog *data);
+void LoadDialog(struct LoadDialog *data, const char *file);
 
 struct SaveDialog
 {
@@ -445,9 +443,9 @@ struct SaveDialog
   /****** local member *******/
   GtkWidget *path, *include_data, *include_merge;
   int Path;
-  int *SaveData, *SaveMerge;
+  int SaveData, SaveMerge;
 };
-void SaveDialog(struct SaveDialog *data, int *sdata, int *smerge);
+void SaveDialog(struct SaveDialog *data);
 
 struct OutputDataDialog
 {
@@ -505,11 +503,13 @@ struct MiscDialog
 {
   DIALOG_PROTOTYPE;
   /****** local member *******/
-  GtkWidget *editor, *directory, *path, *datafile,
+  GtkWidget *editor, *path, *datafile,
     *expand, *expanddir, *loadpath, *mergefile, *coordwin_font, *infowin_font,
     *file_preview_font, *hist_size, *info_size, *data_head_lines, *help_browser,
-    *browser, *use_opacity, *select_data, *use_custom_palette, *use_dark_theme, *source_style,
-    *decimalsign;
+    *browser, *select_data, *use_custom_palette, *use_dark_theme, *source_style,
+    *decimalsign, *icon_size;
+  GtkWidget **palette;
+  GtkWidget *directory;
   struct objlist *Obj;
   struct narray tmp_palette;
   int Id;
@@ -566,6 +566,7 @@ struct CopyDialog
   GtkWidget *list;
   const char *title;
   char *(*cb) (struct objlist * obj, int id);
+  response_cb rcb;
 };
 
 void CopyDialog(struct CopyDialog *data,
@@ -580,7 +581,6 @@ struct OutputImageDialog
   GtkWidget *version, *t2p, *dpi, *use_opacity;
   int Version, text2path, Dpi, DlgType, UseOpacity;
 };
-void OutputImageDialog(struct OutputImageDialog *data, int type);
 
 
 
