@@ -65,12 +65,7 @@ static void LegendGaussDialogDiv(GtkWidget *w, gpointer client_data);
 static void
 clear_view(struct LegendGaussDialog *d)
 {
-  GdkWindow *win;
-
-  win = gtk_widget_get_window(d->view);
-  if (win) {
-    gdk_window_invalidate_rect(win, NULL, TRUE);
-  }
+  gtk_widget_queue_draw(d->view);
 }
 
 static void
@@ -86,21 +81,21 @@ LegendGaussDialogSetupItem(GtkWidget *w, struct LegendGaussDialog *d, int id)
 
   SetWidgetFromObjField(d->miter, d->Obj, id, "miter_limit");
 
-  set_stroke_color(d->color, d->Obj, id);
+  set_stroke_color(d->color, d->alpha, d->Obj, id);
 
   n = d->Dir;
   if (n >= 0 && n < LEGEND_DIRECTION_NUM) {
-    GtkToggleButton *btn;
-    btn = GTK_TOGGLE_BUTTON(g_slist_nth_data(d->dir_list, n));
-    gtk_toggle_button_set_active(btn, TRUE);
+    GtkCheckButton *btn;
+    btn = GTK_CHECK_BUTTON(g_slist_nth_data(d->dir_list, n));
+    gtk_check_button_set_active(btn, TRUE);
   }
 
 
   n = d->Mode;
   if (n >=0 && n < 4) {
-    GtkToggleButton *btn;
-    btn = GTK_TOGGLE_BUTTON(g_slist_nth_data(d->func_list, n));
-    gtk_toggle_button_set_active(btn, TRUE);
+    GtkCheckButton *btn;
+    btn = GTK_CHECK_BUTTON(g_slist_nth_data(d->func_list, n));
+    gtk_check_button_set_active(btn, TRUE);
   }
 
   gtk_range_set_value(GTK_RANGE(d->div), d->Div);
@@ -115,7 +110,6 @@ LegendGaussDialogPaint(GtkWidget *w, cairo_t *cr, gpointer client_data)
   int pw, dw, minx, miny, maxx, maxy,
     amp, wd, output, found;
   double ppd, dashes[] = {4.0};
-  GdkWindow *win;
   struct objlist *gobj, *robj;
   N_VALUE *inst;
   struct gra2cairo_local *local;
@@ -123,11 +117,6 @@ LegendGaussDialogPaint(GtkWidget *w, cairo_t *cr, gpointer client_data)
 
   d = (struct LegendGaussDialog *) client_data;
 
-  win = gtk_widget_get_window(w);
-  if (win == NULL) {
-    return FALSE;
-  }
-
   found = find_gra2gdk_inst(&gobj, &inst, &robj, &output, &local);
   if (! found) {
     return FALSE;
@@ -292,9 +281,9 @@ get_radio_index(GSList *top)
 
   n = g_slist_length(top);
   for (i = 0, list = top; i < n; i++, list = list->next) {
-    GtkToggleButton *btn;
-    btn = GTK_TOGGLE_BUTTON(list->data);
-    if (gtk_toggle_button_get_active(btn)) {
+    GtkCheckButton *btn;
+    btn = GTK_CHECK_BUTTON(list->data);
+    if (gtk_check_button_get_active(btn)) {
       return i;
     }
   }
@@ -308,7 +297,7 @@ LegendGaussDialogMode(GtkWidget *w, gpointer client_data)
   int i;
   gboolean active;
 
-  active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
+  active = gtk_check_button_get_active(GTK_CHECK_BUTTON(w));
   if (! active)
     return;
 
@@ -329,7 +318,7 @@ LegendGaussDialogDir(GtkWidget *w, gpointer client_data)
   struct LegendGaussDialog *d;
   gboolean active;
 
-  active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
+  active = gtk_check_button_get_active(GTK_CHECK_BUTTON(w));
   if (! active)
     return;
 
@@ -354,6 +343,12 @@ LegendGaussDialogDiv(GtkWidget *w, gpointer client_data)
   clear_view(d);
 }
 
+static void
+draw_function(GtkDrawingArea* drawing_area, cairo_t* cr, int width, int height, gpointer user_data)
+{
+  LegendGaussDialogPaint(GTK_WIDGET(drawing_area), cr, user_data);
+}
+
 static void
 LegendGaussDialogSetup(GtkWidget *wi, void *data, int makewidget)
 {
@@ -368,29 +363,36 @@ LegendGaussDialogSetup(GtkWidget *wi, void *data, int makewidget)
   if (makewidget) {
     GtkWidget *w, *button, *hbox, *hbox2, *vbox, *table;
     int i;
+    GtkWidget *group = NULL;
     hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
 
     button = NULL;
 
-    button = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(button), _("_Sin"));
+    button = gtk_check_button_new_with_mnemonic(_("_Sin"));
+    group = button;
+    func_list = g_slist_prepend(NULL, button);
     g_signal_connect(button, "toggled", G_CALLBACK(LegendGaussDialogMode), d);
-    gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+    gtk_box_prepend(GTK_BOX(hbox), button);
 
-    button = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(button), _("_Parabola"));
+    button = gtk_check_button_new_with_mnemonic(_("_Parabola"));
+    gtk_check_button_set_group(GTK_CHECK_BUTTON(button), GTK_CHECK_BUTTON(group));
+    func_list = g_slist_prepend(func_list, button);
     g_signal_connect(button, "toggled", G_CALLBACK(LegendGaussDialogMode), d);
-    gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+    gtk_box_prepend(GTK_BOX(hbox), button);
 
-    button = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(button), _("_Lorentz"));
+    button = gtk_check_button_new_with_mnemonic(_("_Lorentz"));
+    gtk_check_button_set_group(GTK_CHECK_BUTTON(button), GTK_CHECK_BUTTON(group));
+    func_list = g_slist_prepend(func_list, button);
     g_signal_connect(button, "toggled", G_CALLBACK(LegendGaussDialogMode), d);
-    gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+    gtk_box_prepend(GTK_BOX(hbox), button);
 
-    button = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(button), _("_Gauss"));
+    button = gtk_check_button_new_with_mnemonic(_("_Gauss"));
+    gtk_check_button_set_group(GTK_CHECK_BUTTON(button), GTK_CHECK_BUTTON(group));
+    func_list = g_slist_prepend(func_list, button);
     g_signal_connect(button, "toggled", G_CALLBACK(LegendGaussDialogMode), d);
-    gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+    gtk_box_prepend(GTK_BOX(hbox), button);
 
-    func_list = gtk_radio_button_get_group(GTK_RADIO_BUTTON(button));
-
-    gtk_box_pack_start(GTK_BOX(d->vbox), hbox, FALSE, FALSE, 0);
+    gtk_box_append(GTK_BOX(d->vbox), hbox);
     d->func_list = func_list;
 
 
@@ -419,8 +421,13 @@ LegendGaussDialogSetup(GtkWidget *wi, void *data, int makewidget)
     add_widget_to_table(table, w, _("_Color:"), FALSE, i++);
     d->color = w;
 
+    w = create_spin_entry_type(SPIN_BUTTON_TYPE_ALPHA, FALSE, TRUE);
+    add_widget_to_table(table, w, _("_Alpha:"), FALSE, i++);
+    d->alpha = w;
+
     w = gtk_scale_new_with_range(GTK_ORIENTATION_HORIZONTAL, 10, DIV_MAX, 1);
     set_scale_mark(w, GTK_POS_BOTTOM, 20, 20);
+    gtk_scale_set_draw_value(GTK_SCALE(w), TRUE);
     add_widget_to_table(table, w, _("_Division:"), TRUE, i++);
     g_signal_connect(w, "value-changed", G_CALLBACK(LegendGaussDialogDiv), d);
     d->div = w;
@@ -429,54 +436,60 @@ LegendGaussDialogSetup(GtkWidget *wi, void *data, int makewidget)
     hbox2 = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
     button = NULL;
 
-    button = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(button), "_T");
+    button = gtk_check_button_new_with_mnemonic("_T");
+    group = button;
+    dir_list = g_slist_prepend(NULL, button);
     g_signal_connect(button, "toggled", G_CALLBACK(LegendGaussDialogDir), d);
-    gtk_box_pack_start(GTK_BOX(hbox2), button, FALSE, FALSE, 0);
+    gtk_box_append(GTK_BOX(hbox2), button);
 
-    button = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(button), "_B");
+    button = gtk_check_button_new_with_mnemonic("_B");
+    gtk_check_button_set_group(GTK_CHECK_BUTTON(button), GTK_CHECK_BUTTON(group));
+    dir_list = g_slist_prepend(dir_list, button);
     g_signal_connect(button, "toggled", G_CALLBACK(LegendGaussDialogDir), d);
-    gtk_box_pack_start(GTK_BOX(hbox2), button, FALSE, FALSE, 0);
+    gtk_box_append(GTK_BOX(hbox2), button);
 
-    button = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(button), "_L");
+    button = gtk_check_button_new_with_mnemonic("_L");
+    gtk_check_button_set_group(GTK_CHECK_BUTTON(button), GTK_CHECK_BUTTON(group));
+    dir_list = g_slist_prepend(dir_list, button);
     g_signal_connect(button, "toggled", G_CALLBACK(LegendGaussDialogDir), d);
-    gtk_box_pack_start(GTK_BOX(hbox2), button, FALSE, FALSE, 0);
+    gtk_box_append(GTK_BOX(hbox2), button);
 
-    button = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(button), "_R");
+    button = gtk_check_button_new_with_mnemonic("_R");
+    gtk_check_button_set_group(GTK_CHECK_BUTTON(button), GTK_CHECK_BUTTON(group));
+    dir_list = g_slist_prepend(dir_list, button);
     g_signal_connect(button, "toggled", G_CALLBACK(LegendGaussDialogDir), d);
-    gtk_box_pack_start(GTK_BOX(hbox2), button, FALSE, FALSE, 0);
-
-    dir_list = gtk_radio_button_get_group(GTK_RADIO_BUTTON(button));
+    gtk_box_append(GTK_BOX(hbox2), button);
     d->dir_list = dir_list;
 
     add_widget_to_table(table, hbox2, _("Direction:"), TRUE, i++);
 
-    gtk_box_pack_start(GTK_BOX(hbox), table, TRUE, TRUE, 0);
+    gtk_box_append(GTK_BOX(hbox), table);
 
 
     vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
 
     w = gtk_scale_new_with_range(GTK_ORIENTATION_HORIZONTAL, 0, SCALE_V_MAX, 1);
     set_scale_mark(w, GTK_POS_BOTTOM, 100, 200);
+    gtk_scale_set_draw_value(GTK_SCALE(w), TRUE);
     g_signal_connect(w, "value-changed", G_CALLBACK(LegendGaussDialogScaleV), d);
     d->scv = w;
-    gtk_box_pack_start(GTK_BOX(vbox), w, FALSE, FALSE, 0);
+    gtk_box_append(GTK_BOX(vbox), w);
 
     w = gtk_drawing_area_new();
     d->view = w;
     gtk_widget_set_size_request(w, VIEW_SIZE, VIEW_SIZE);
 
-    g_signal_connect(w, "draw", G_CALLBACK(LegendGaussDialogPaint), d);
-    gtk_box_pack_start(GTK_BOX(vbox), w, FALSE, FALSE, 0);
+    gtk_drawing_area_set_draw_func(GTK_DRAWING_AREA(w), draw_function, d, NULL);
+    gtk_box_append(GTK_BOX(vbox), w);
 
     w = gtk_scale_new_with_range(GTK_ORIENTATION_HORIZONTAL, - SCALE_H_MAX, SCALE_H_MAX, 1);
     set_scale_mark(w, GTK_POS_TOP, -100, 25);
+    gtk_scale_set_draw_value(GTK_SCALE(w), TRUE);
     g_signal_connect(w, "value-changed", G_CALLBACK(LegendGaussDialogScaleH), d);
     d->sch = w;
-    gtk_box_pack_start(GTK_BOX(vbox), w, FALSE, FALSE, 0);
-
-    gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
-    gtk_box_pack_start(GTK_BOX(d->vbox), hbox, TRUE, TRUE, 0);
-    gtk_widget_show_all(GTK_WIDGET(d->vbox));
+    gtk_box_append(GTK_BOX(vbox), w);
+    gtk_box_append(GTK_BOX(hbox), vbox);
+    gtk_box_append(GTK_BOX(d->vbox), hbox);
   }
   LegendGaussDialogSetupItem(wi, d, d->Id);
 }
@@ -508,7 +521,7 @@ LegendGaussDialogClose(GtkWidget *w, void *data)
   if (SetObjFieldFromWidget(d->miter, d->Obj, d->Id, "miter_limit"))
     return;
 
-  if (putobj_stroke_color(d->color, d->Obj, d->Id))
+  if (putobj_stroke_color(d->color, d->alpha, d->Obj, d->Id))
     return;
 
   switch (d->Dir) {
