@@ -22,7 +22,7 @@
  */
 
 void MergeWinUpdate(struct obj_list_data *d, int clear, int draw);
-void CmMergeOpen(void *w, gpointer client_data);
-void CmMergeClose(void *w, gpointer client_data);
-void CmMergeUpdate(void *w, gpointer client_data);
+void CmMergeOpen(GSimpleAction *action, GVariant *parameter, gpointer client_data);
+void CmMergeClose(void);
+void CmMergeUpdate(void);
 GtkWidget *create_merge_list(struct SubWin *d);
