@@ -33,7 +33,6 @@
 #include "shell.h"
 #include "oparameter.h"
 
-#include "gtk_liststore.h"
 #include "gtk_subwin.h"
 #include "gtk_combo.h"
 #include "gtk_widget.h"
@@ -54,8 +53,11 @@ struct parameter_data
   int playing;
   GtkWidget *scale, *repeat;
   struct objlist *obj;
-  int id;
+  int id, undo;
   struct obj_list_data *obj_list_data;
+  /* for animation */
+  double start, stop, step, prm;
+  GtkButton *btn;
 };
 
 static void
@@ -102,19 +104,21 @@ ParameterCB(struct objlist *obj, int id)
 }
 
 static void
-ParameterDialogCopy(GtkWidget *w, gpointer data)
+parameter_dialog_copy_response(int sel, gpointer user_data)
 {
   struct ParameterDialog *d;
-  int sel;
-
-  d = (struct ParameterDialog *) data;
-
-  sel = CopyClick(d->widget, d->Obj, d->Id, ParameterCB);
+  d = (struct ParameterDialog *) user_data;
   if (sel != -1) {
     ParameterDialogSetupItem(d, sel);
   }
 }
 
+static void
+ParameterDialogCopy(struct ParameterDialog *d)
+{
+  CopyClick(d->widget, d->Obj, d->Id, ParameterCB, parameter_dialog_copy_response, d);
+}
+
 #define TYPE_SPIN_NAME  "spin"
 #define TYPE_CHECK_NAME "check"
 #define TYPE_COMBO_NAME "combo"
@@ -161,20 +165,20 @@ exchange_start_stop(GtkButton *btn, gpointer user_data)
 
   d = user_data;
 
-  str = gtk_entry_get_text(GTK_ENTRY(d->start));
+  str = gtk_editable_get_text(GTK_EDITABLE(d->start));
   if (str == NULL) {
     str = "";
   }
   start = g_strdup(str);
 
-  str = gtk_entry_get_text(GTK_ENTRY(d->stop));
+  str = gtk_editable_get_text(GTK_EDITABLE(d->stop));
   if (str == NULL) {
     str = "";
   }
   stop = g_strdup(str);
 
-  gtk_entry_set_text(GTK_ENTRY(d->start), stop);
-  gtk_entry_set_text(GTK_ENTRY(d->stop), start);
+  gtk_editable_set_text(GTK_EDITABLE(d->start), stop);
+  gtk_editable_set_text(GTK_EDITABLE(d->stop), start);
   g_free(start);
   g_free(stop);
 }
@@ -239,9 +243,9 @@ add_page_combo(struct ParameterDialog *d)
   buf = gtk_text_buffer_new(NULL);
   w = gtk_text_view_new_with_buffer(buf);
   gtk_widget_set_vexpand(w, TRUE);
-  swin = gtk_scrolled_window_new(NULL, NULL);
+  swin = gtk_scrolled_window_new();
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-  gtk_container_add(GTK_CONTAINER(swin), w);
+  gtk_scrolled_window_set_child(GTK_SCROLLED_WINDOW(swin), w);
   add_widget_to_table(table, swin, _("_Items:"), TRUE, i++);
   d->items = w;
 
@@ -253,23 +257,23 @@ add_page_combo(struct ParameterDialog *d)
 }
 
 static void
-parameter_type_changed(GtkComboBox *combo, gpointer user_data)
+parameter_type_changed(GtkWidget *combo, GParamSpec *spec, gpointer user_data)
 {
   struct ParameterDialog *d;
   int type;
 
   d = user_data;
-  type = gtk_combo_box_get_active(combo);
+  type = combo_box_get_active(combo);
   if (type < 0) {
     return;
   }
   switch (type) {
   case PARAMETER_TYPE_SPIN:
-    gtk_widget_show(d->wrap);
+    gtk_widget_set_visible(d->wrap, TRUE);
     gtk_stack_set_visible_child_name(GTK_STACK(d->stack), TYPE_SPIN_NAME);
     break;
   case PARAMETER_TYPE_SCALE:
-    gtk_widget_hide(d->wrap);
+    gtk_widget_set_visible(d->wrap, FALSE);
     gtk_stack_set_visible_child_name(GTK_STACK(d->stack), TYPE_SPIN_NAME);
     break;
   case PARAMETER_TYPE_SWITCH:
@@ -308,7 +312,7 @@ ParameterDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
     w = combo_box_create();
     add_widget_to_table(table, w, _("_Type:"), TRUE, i++);
-    g_signal_connect(w, "changed", G_CALLBACK(parameter_type_changed), d);
+    g_signal_connect(w, "notify::selected", G_CALLBACK(parameter_type_changed), d);
     d->type = w;
 
     w = gtk_stack_new();
@@ -325,12 +329,10 @@ ParameterDialogSetup(GtkWidget *wi, void *data, int makewidget)
     d->redraw = w;
 
     frame = gtk_frame_new(NULL);
-    gtk_container_add(GTK_CONTAINER(frame), table);
-    gtk_box_pack_start(GTK_BOX(d->vbox), frame, TRUE, TRUE, 4);
+    gtk_frame_set_child(GTK_FRAME(frame), table);
+    gtk_box_append(GTK_BOX(d->vbox), frame);
 
     add_copy_button_to_box(GTK_WIDGET(d->vbox), G_CALLBACK(ParameterDialogCopy), d, "parameter");
-
-    gtk_widget_show_all(GTK_WIDGET(d->vbox));
   }
   ParameterDialogSetupItem(d, d->Id);
 }
@@ -383,7 +385,7 @@ ParameterDialogClose(GtkWidget *w, void *data)
   if (SetObjFieldFromWidget(d->transition, d->Obj, d->Id, "transition"))
     return;
 
-  type = gtk_combo_box_get_active(GTK_COMBO_BOX(d->type));
+  type = combo_box_get_active(d->type);
   if (type == PARAMETER_TYPE_TRANSITION) {
     if (SetObjFieldFromWidget(d->transition_step, d->Obj, d->Id, "step")) {
       return;
@@ -433,10 +435,25 @@ ParameterDialog(struct obj_list_data *data, int id, int user_data)
   d->Id = id;
 }
 
-void
-CmParameterAdd(void *w, gpointer client_data)
+static void
+parameter_add_response(struct response_callback *cb)
 {
-  int id, undo, ret;
+  struct obj_list_data *d;
+
+  d = NgraphApp.ParameterWin.data.data;
+  if (cb->return_value == IDCANCEL) {
+    int undo = GPOINTER_TO_INT(cb->data);
+    menu_undo_internal(undo);
+  } else {
+    set_graph_modified();
+  }
+  ParameterWinUpdate(d, FALSE, FALSE);
+}
+
+void
+CmParameterAdd(void)
+{
+  int id, undo;
   struct obj_list_data *d;
 
   if (Menulock || Globallock)
@@ -450,42 +467,57 @@ CmParameterAdd(void *w, gpointer client_data)
     return;
   }
   ParameterDialog(d, id, -1);
-  ret = DialogExecute(TopLevel, &DlgParameter);
-  if (ret == IDCANCEL) {
-    menu_undo_internal(undo);
-  } else {
-    set_graph_modified();
-  }
-  ParameterWinUpdate(d, FALSE, FALSE);
+  response_callback_add(&DlgParameter, parameter_add_response, NULL, GINT_TO_POINTER(undo));
+  DialogExecute(TopLevel, &DlgParameter);
 }
 
-void
-CmParameterDelete(void *w, gpointer client_data)
+static void
+parameter_delete_response(struct response_callback *cb)
 {
-  struct narray farray;
+  struct narray *farray;
   struct obj_list_data *d;
 
-  if (Menulock || Globallock)
-    return;
-
   d = NgraphApp.ParameterWin.data.data;
-  if (chkobjlastinst(d->obj) == -1)
-    return;
-  SelectDialog(&DlgSelect, d->obj, _("delete parameter (multi select)"), ParameterCB, (struct narray *) &farray, NULL);
-  if (DialogExecute(TopLevel, &DlgSelect) == IDOK) {
+  farray = (struct narray *) cb->data;
+  if (cb->return_value == IDOK) {
     int i, num, *array;
-    num = arraynum(&farray);
+    num = arraynum(farray);
     if (num > 0) {
       menu_save_undo_single(UNDO_TYPE_DELETE, d->obj->name);
     }
-    array = arraydata(&farray);
+    array = arraydata(farray);
     for (i = num - 1; i >= 0; i--) {
       delobj(d->obj, array[i]);
       set_graph_modified();
     }
     ParameterWinUpdate(d, FALSE, FALSE);
   }
-  arraydel(&farray);
+  arrayfree(farray);
+}
+
+void
+CmParameterDelete(void)
+{
+  struct narray *farray;
+  struct obj_list_data *d;
+
+  if (Menulock || Globallock)
+    return;
+
+  d = NgraphApp.ParameterWin.data.data;
+  if (chkobjlastinst(d->obj) == -1) {
+    return;
+  }
+
+  farray = arraynew(sizeof(int));
+  if (farray == NULL) {
+    return;
+
+  }
+
+  SelectDialog(&DlgSelect, d->obj, _("delete parameter (multi select)"), ParameterCB, farray, NULL);
+  response_callback_add(&DlgSelect, parameter_delete_response, NULL, farray);
+  DialogExecute(TopLevel, &DlgSelect);
 }
 
 static void
@@ -496,13 +528,86 @@ update_parameter(struct obj_list_data *d)
   ViewerWinUpdate(objects);
 }
 
-void
-CmParameterUpdate(void *w, gpointer client_data)
+struct parameter_update_data
 {
-  struct narray farray;
-  int modified;
   struct obj_list_data *d;
-  int i, *array, num, undo;
+  struct narray *farray;
+  int undo, modified;
+};
+
+static void
+cm_parameter_update_response_response(struct response_callback *cb)
+{
+  struct narray *farray;
+  int num, id;
+  struct parameter_update_data *data;
+
+  data = (struct parameter_update_data *) cb->data;
+  farray = data->farray;
+
+  num = arraynum (farray);
+  if (num < 1) {
+    if (data->modified) {
+      update_parameter(data->d);
+    } else if (data->undo >= 0) {
+      menu_undo_internal(data->undo);
+    }
+    arrayfree(farray);
+    g_free(data);
+    return;
+  }
+
+  if (cb->return_value == IDOK) {
+    data->modified = TRUE;
+  }
+  id = arraypop_int (farray);
+  ParameterDialog(data->d, id, -1);
+  response_callback_add(&DlgParameter, cm_parameter_update_response_response, NULL, data);
+  DialogExecute(TopLevel, &DlgParameter);
+}
+
+static void
+cm_parameter_update_response(struct response_callback *cb)
+{
+  struct obj_list_data *d;
+  struct narray *farray;
+  int num;
+  struct parameter_update_data *data;
+  int id;
+
+  farray = (struct narray *) cb->data;
+  d = NgraphApp.ParameterWin.data.data;
+  if (cb->return_value != IDOK) {
+    arrayfree(farray);
+    return;
+  }
+  num = arraynum(farray);
+  if (num <= 0) {
+    arrayfree(farray);
+    return;
+  }
+  data = g_malloc0(sizeof(*data));
+  if (data == NULL) {
+    arrayfree(farray);
+    return;
+  }
+
+  data->modified = FALSE;
+  data->farray = farray;
+  data->d = d;
+  data->undo = menu_save_undo_single(UNDO_TYPE_EDIT, d->obj->name);
+
+  id = arraypop_int (farray);
+  ParameterDialog(d, id, -1);
+  response_callback_add(&DlgParameter, cm_parameter_update_response_response, NULL, data);
+  DialogExecute(TopLevel, &DlgParameter);
+}
+
+void
+CmParameterUpdate(void)
+{
+  struct narray *farray;
+  struct obj_list_data *d;
 
   if (Menulock || Globallock)
     return;
@@ -511,52 +616,43 @@ CmParameterUpdate(void *w, gpointer client_data)
   if (chkobjlastinst(d->obj) == -1) {
     return;
   }
-  SelectDialog(&DlgSelect, d->obj, _("parameter property (multi select)"), ParameterCB, (struct narray *) &farray, NULL);
-  modified = FALSE;
-  if (DialogExecute(TopLevel, &DlgSelect) != IDOK) {
-    goto EndUpdate;
+
+  farray = arraynew(sizeof(int));
+  if (farray == NULL) {
+    return;
+
   }
-  num = arraynum(&farray);
-  if (num <= 0) {
-    goto EndUpdate;
+
+  SelectDialog(&DlgSelect, d->obj, _("parameter property (multi select)"), ParameterCB, farray, NULL);
+  response_callback_add(&DlgSelect, cm_parameter_update_response, NULL, farray);
+  DialogExecute(TopLevel, &DlgSelect);
+}
+
+static void
+parameter_update_response(struct response_callback *cb)
+{
+  struct parameter_data *d;
+  d = (struct parameter_data *) cb->data;
+  if (cb->return_value == IDCANCEL) {
+    menu_undo_internal(d->undo);
+  } else {
+    update_parameter(d->obj_list_data);
   }
-  undo = menu_save_undo_single(UNDO_TYPE_EDIT, d->obj->name);
-  array = arraydata(&farray);
-  for (i = 0; i < num; i++) {
-    int ret;
-    ParameterDialog(d, array[i], -1);
-    ret = DialogExecute(TopLevel, &DlgParameter);
-    if (ret != IDCANCEL) {
-      modified = TRUE;
-    }
-  }
-  if (modified) {
-    update_parameter(d);
-  } else if (undo >= 0) {
-    menu_undo_internal(undo);
-  }
- EndUpdate:
-  arraydel(&farray);
 }
 
 static void
 parameter_update(GtkButton *btn, gpointer data)
 {
-  int undo, ret;
   struct parameter_data *d;
 
   if (Menulock || Globallock)
     return;
 
   d = data;
-  undo = menu_save_undo_single(UNDO_TYPE_EDIT, d->obj->name);
+  d->undo = menu_save_undo_single(UNDO_TYPE_EDIT, d->obj->name);
   ParameterDialog(d->obj_list_data, d->id, -1);
-  ret = DialogExecute(TopLevel, &DlgParameter);
-  if (ret == IDCANCEL) {
-    menu_undo_internal(undo);
-  } else {
-    update_parameter(d->obj_list_data);
-  }
+  response_callback_add(&DlgParameter, parameter_update_response, NULL, d);
+  DialogExecute(TopLevel, &DlgParameter);
 }
 
 static void
@@ -608,18 +704,68 @@ static void
 set_pause_icon(GtkButton *btn)
 {
   GtkWidget *icon;
-  icon = gtk_image_new_from_icon_name("media-playback-pause-symbolic", GTK_ICON_SIZE_BUTTON);
+  icon = gtk_image_new_from_icon_name("media-playback-pause-symbolic");
+  gtk_image_set_icon_size(GTK_IMAGE(icon), Menulocal.icon_size);
   gtk_widget_set_tooltip_text(GTK_WIDGET(btn), _("Pause"));
-  gtk_button_set_image(btn, icon);
+  gtk_button_set_child(btn, icon);
 }
 
 static void
 set_play_icon(GtkButton *btn)
 {
   GtkWidget *icon;
-  icon = gtk_image_new_from_icon_name("media-playback-start-symbolic", GTK_ICON_SIZE_BUTTON);
+  icon = gtk_image_new_from_icon_name("media-playback-start-symbolic");
+  gtk_image_set_icon_size(GTK_IMAGE(icon), Menulocal.icon_size);
   gtk_widget_set_tooltip_text(GTK_WIDGET(btn), _("Play"));
-  gtk_button_set_image(btn, icon);
+  gtk_button_set_child(btn, icon);
+}
+
+static void
+parameter_play_finalize(struct parameter_data *data)
+{
+  double prm;
+  GtkWidget *scale;
+
+  scale = data->scale;
+  prm = gtk_range_get_value(GTK_RANGE(scale));
+  set_parameter(prm, data);
+  data->playing = FALSE;
+  set_play_icon(data->btn);
+  menu_lock(FALSE);
+}
+
+static int
+parameter_play_main(gpointer user_data)
+{
+  double start, stop, step, prm;
+  struct parameter_data *data;
+  GtkWidget *scale;
+
+  data = user_data;
+  start = data->start;
+  stop = data->stop;
+  step = data->step;
+  prm = data->prm;
+  scale = data->scale;
+
+  if (! data->playing) {
+    parameter_play_finalize(data);
+    return FALSE;
+  }
+  if (fabs(prm - start) > fabs(stop - start)) {
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(data->repeat))) {
+      data->prm = start;
+      return TRUE;
+    }
+    parameter_play_finalize(data);
+    return FALSE;
+  }
+  gtk_range_set_value(GTK_RANGE(scale), prm);
+  set_parameter(prm, data);
+  prm = gtk_range_get_value(GTK_RANGE(scale));
+  data->prm += step;
+
+  return TRUE;
 }
 
 static void
@@ -665,32 +811,12 @@ parameter_play(GtkButton *btn, gpointer user_data)
     }
   }
   data->playing = TRUE;
-  while (1) {
-    while (fabs(prm - start) <= fabs(stop - start)) {
-      int i;
-      gtk_range_set_value(GTK_RANGE(scale), prm);
-      set_parameter(prm, data);
-      for (i = 1; i < wait; i++) {
-	msleep(10);
-	reset_event();
-	if (! data->playing) {
-	  goto EndPlaying;
-	}
-      }
-      prm = gtk_range_get_value(GTK_RANGE(scale));
-      prm += step;
-    }
-    if (! gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(data->repeat))) {
-      break;
-    }
-    prm = start;
-  }
- EndPlaying:
-  prm = gtk_range_get_value(GTK_RANGE(scale));
-  set_parameter(prm, data);
-  data->playing = FALSE;
-  set_play_icon(btn);
-  menu_lock(FALSE);
+  data->start = start;
+  data->stop = stop;
+  data->step = step;
+  data->prm = prm;
+  data->btn = btn;
+  g_timeout_add(wait * 10, parameter_play_main, data);
 }
 
 static void
@@ -833,16 +959,16 @@ value_changed(GtkAdjustment *adjustment, gpointer user_data)
 }
 
 static void
-toggled(GtkToggleButton *toggle_button, gpointer user_data)
+toggled(GtkCheckButton *check_button, gpointer user_data)
 {
   int active;
   if (Menulock || Globallock) {
-    gtk_toggle_button_set_inconsistent(toggle_button, TRUE);
+    gtk_check_button_set_inconsistent(GTK_CHECK_BUTTON(check_button), FALSE);
     return;
   }
 
-  gtk_toggle_button_set_inconsistent(toggle_button, FALSE);
-  active = gtk_toggle_button_get_active(toggle_button);
+  gtk_check_button_set_inconsistent(GTK_CHECK_BUTTON(check_button), FALSE);
+  active = gtk_check_button_get_active(check_button);
   set_parameter(active, user_data);
 }
 
@@ -864,13 +990,13 @@ switch_set(GtkSwitch *sw, gboolean state, gpointer user_data)
 }
 
 static void
-combo_changed(GtkComboBox *combo_box, gpointer user_data)
+combo_changed(GtkWidget *combo_box, GParamSpec *spec, gpointer user_data)
 {
   int selected;
   if (Menulock || Globallock)
     return;
 
-  selected = gtk_combo_box_get_active(combo_box);
+  selected = combo_box_get_active(combo_box);
   set_parameter(selected, user_data);
 }
 
@@ -920,6 +1046,7 @@ create_scale(double min, double max, double inc, double value)
 
   inverted = check_min_max(&min, &max, &inc);
   scale = gtk_scale_new_with_range(GTK_ORIENTATION_HORIZONTAL, min, max, inc);
+  gtk_scale_set_draw_value(GTK_SCALE(scale), TRUE);
   gtk_widget_set_size_request(GTK_WIDGET(scale), 200, -1);
   gtk_widget_set_hexpand(GTK_WIDGET(scale), TRUE);
   gtk_range_set_value(GTK_RANGE(scale), value);
@@ -1019,13 +1146,13 @@ create_widget(struct obj_list_data *d, int id, int n)
     }
     width = 2;
     g_signal_connect(w, "toggled", G_CALLBACK(toggled), data);
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), checked);
+    gtk_check_button_set_active(GTK_CHECK_BUTTON(w), checked);
     break;
   case PARAMETER_TYPE_COMBO:
     w = create_combo_box(items, selected);
     gtk_widget_set_halign(GTK_WIDGET(w), GTK_ALIGN_START);
     gtk_widget_set_hexpand(w, FALSE);
-    g_signal_connect(w, "changed", G_CALLBACK(combo_changed), data);
+    g_signal_connect(w, "notify::selected", G_CALLBACK(combo_changed), data);
     break;
   case PARAMETER_TYPE_SWITCH:
     w = gtk_switch_new();
@@ -1078,12 +1205,6 @@ create_widget(struct obj_list_data *d, int id, int n)
     col += 3;
   }
 
-  col++;
-  separator = gtk_frame_new(NULL);
-  gtk_frame_set_shadow_type(GTK_FRAME(separator), GTK_SHADOW_NONE);
-  gtk_widget_set_hexpand(GTK_WIDGET(separator), TRUE);
-  gtk_grid_attach(GTK_GRID(d->text), separator, col, id, 1, 1);
-
   col++;
   separator = gtk_separator_new(GTK_ORIENTATION_HORIZONTAL);
   gtk_grid_attach(GTK_GRID(d->text), separator, col, id, 1, 1);
@@ -1103,13 +1224,6 @@ create_widget(struct obj_list_data *d, int id, int n)
   return col + 1;
 }
 
-static void
-remove_child(GtkWidget *widget, gpointer data)
-{
-  GtkContainer *container = data;
-  gtk_container_remove(container, widget);
-}
-
 static void
 save_as_default(GtkButton *button, gpointer user_data)
 {
@@ -1162,6 +1276,52 @@ save_as_default(GtkButton *button, gpointer user_data)
   }
 }
 
+static void
+reset_settings(GtkButton *button, gpointer user_data)
+{
+  struct obj_list_data *d;
+  int i, num;
+  GtkGrid *grid;
+  d = (struct obj_list_data *) user_data;
+  num = chkobjlastinst(d->obj) + 1;
+  grid = GTK_GRID (d->text);
+  for (i = 0; i < num; i++) {
+    double value;
+    int ival, type;
+    GtkWidget *w;
+    getobj(d->obj, "type", i, 0, NULL, &type);
+    switch (type) {
+    case PARAMETER_TYPE_SPIN:
+      w = gtk_grid_get_child_at (grid, 3, i);
+      getobj(d->obj, "value", i, 0, NULL, &value);
+      gtk_spin_button_set_value (GTK_SPIN_BUTTON (w), value);
+      break;
+    case PARAMETER_TYPE_SCALE:
+      w = gtk_grid_get_child_at (grid, 3, i);
+      getobj(d->obj, "value", i, 0, NULL, &value);
+      gtk_range_set_value (GTK_RANGE (w), value);
+      break;
+    case PARAMETER_TYPE_SWITCH:
+      w = gtk_grid_get_child_at (grid, 3, i);
+      getobj(d->obj, "active", i, 0, NULL, &ival);
+      gtk_switch_set_active (GTK_SWITCH (w), ival);
+      break;
+    case PARAMETER_TYPE_CHECK:
+      w = gtk_grid_get_child_at (grid, 2, i);
+      getobj(d->obj, "active", i, 0, NULL, &ival);
+      gtk_check_button_set_active (GTK_CHECK_BUTTON (w), ival);
+      break;
+    case PARAMETER_TYPE_COMBO:
+      w = gtk_grid_get_child_at (grid, 3, i);
+      getobj(d->obj, "selected", i, 0, NULL, &ival);
+      combo_box_set_active (w, ival);
+      break;
+    case PARAMETER_TYPE_TRANSITION:
+      break;
+    }
+  }
+}
+
 static void
 add_save_as_default_button(struct obj_list_data *d, int row, int width)
 {
@@ -1170,7 +1330,12 @@ add_save_as_default_button(struct obj_list_data *d, int row, int width)
   g_signal_connect(button, "clicked", G_CALLBACK(save_as_default), d);
 
   box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
-  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
+  gtk_box_append(GTK_BOX(box), button);
+
+  button = gtk_button_new_with_mnemonic(_("_Reset"));
+  g_signal_connect(button, "clicked", G_CALLBACK(reset_settings), d);
+  gtk_box_append(GTK_BOX(box), button);
+
   gtk_grid_attach(GTK_GRID(d->text), box, 1, row, width, 1);
 }
 
@@ -1187,7 +1352,10 @@ ParameterWinUpdate(struct obj_list_data *d, int clear, int draw)
   }
 
   add_save_button = FALSE;
-  gtk_container_foreach(GTK_CONTAINER(d->text), remove_child, d->text);
+  while (gtk_grid_get_child_at(GTK_GRID(d->text), 0, 0) ||
+         gtk_grid_get_child_at(GTK_GRID(d->text), 1, 0)) {
+    gtk_grid_remove_row(GTK_GRID(d->text), 0);
+  }
   num = chkobjlastinst(d->obj);
   for (i = 0; i <= num; i++) {
     int type;
@@ -1208,8 +1376,6 @@ ParameterWinUpdate(struct obj_list_data *d, int clear, int draw)
   if (add_save_button) {
     add_save_as_default_button(d, num + 1, col - 4);
   }
-
-  gtk_widget_show_all(GTK_WIDGET(d->text));
 }
 
 GtkWidget *
