@@ -32,7 +32,6 @@
 #include "object.h"
 #include "ioutil.h"
 
-#include "gtk_liststore.h"
 #include "gtk_combo.h"
 #include "gtk_widget.h"
 
@@ -68,6 +67,9 @@ struct print_obj {
 };
 
 static GtkPrintSettings *PrintSettings = NULL;
+#if WINDOWS
+static void output_emf(int type, char *file);
+#endif
 
 static void
 OutputDataDialogSetupItem(GtkWidget *w, struct OutputDataDialog *d)
@@ -87,8 +89,7 @@ OutputDataDialogSetup(GtkWidget *wi, void *data, int makewidget)
     d->div_entry = w;
     hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
     item_setup(hbox, w, _("_Div:"), TRUE);
-    gtk_box_pack_start(GTK_BOX(d->vbox), hbox, FALSE, FALSE, 4);
-    gtk_widget_show_all(GTK_WIDGET(d->vbox));
+    gtk_box_append(GTK_BOX(d->vbox), hbox);
   }
   OutputDataDialogSetupItem(wi, d);
 }
@@ -125,7 +126,6 @@ OutputImageDialogSetupItem(struct OutputImageDialog *d)
   vlabel = get_mnemonic_label(d->version);
 
   gtk_label_set_text(GTK_LABEL(vlabel), "");
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->use_opacity), FALSE);
 
   combo_box_clear(d->version);
   switch (d->DlgType) {
@@ -134,10 +134,12 @@ OutputImageDialogSetupItem(struct OutputImageDialog *d)
     for (i = 0; PsVersion[i]; i++) {
       combo_box_append_text(d->version, PsVersion[i]);
     }
-    set_widget_visibility_with_label(d->dpi, FALSE);
+    set_widget_visibility_with_label(d->dpi, TRUE);
     set_widget_visibility_with_label(d->version, TRUE);
+    set_widget_visibility_with_label(d->use_opacity, TRUE);
     gtk_widget_set_visible(d->t2p, TRUE);
 
+    gtk_check_button_set_active(GTK_CHECK_BUTTON(d->use_opacity), FALSE);
     gtk_label_set_markup_with_mnemonic(GTK_LABEL(vlabel), _("_PostScript Version:"));
 
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(d->dpi), 72);
@@ -146,19 +148,19 @@ OutputImageDialogSetupItem(struct OutputImageDialog *d)
   case MenuIdOutputPNGFile:
     set_widget_visibility_with_label(d->dpi, TRUE);
     set_widget_visibility_with_label(d->version, FALSE);
+    set_widget_visibility_with_label(d->use_opacity, FALSE);
     gtk_widget_set_visible(d->t2p, FALSE);
 
+    gtk_check_button_set_active(GTK_CHECK_BUTTON(d->use_opacity), TRUE);
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(d->dpi), Menulocal.png_dpi);
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->use_opacity), Menulocal.use_opacity);
-
     break;
   case MenuIdOutputPDFFile:
     set_widget_visibility_with_label(d->dpi, FALSE);
     set_widget_visibility_with_label(d->version, FALSE);
+    set_widget_visibility_with_label(d->use_opacity, FALSE);
     gtk_widget_set_visible(d->t2p, TRUE);
 
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(d->dpi), 72);
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->use_opacity), Menulocal.use_opacity);
     break;
   case MenuIdOutputSVGFile:
     for (i = 0; PsVersion[i]; i++) {
@@ -167,58 +169,52 @@ OutputImageDialogSetupItem(struct OutputImageDialog *d)
 
     set_widget_visibility_with_label(d->dpi, FALSE);
     set_widget_visibility_with_label(d->version, TRUE);
+    set_widget_visibility_with_label(d->use_opacity, FALSE);
     gtk_widget_set_visible(d->t2p, TRUE);
 
     gtk_label_set_markup_with_mnemonic(GTK_LABEL(vlabel), _("_SVG Version:"));
 
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(d->dpi), 72);
     combo_box_set_active(d->version, Menulocal.svg_version);
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->use_opacity), Menulocal.use_opacity);
     break;
-#ifdef CAIRO_HAS_WIN32_SURFACE
-  case MenuIdOutputCairoEMFFile:
-    set_widget_visibility_with_label(d->dpi, TRUE);
-    set_widget_visibility_with_label(d->version, FALSE);
-    gtk_widget_set_visible(d->t2p, FALSE);
-    gtk_spin_button_set_value(GTK_SPIN_BUTTON(d->dpi), Menulocal.emf_dpi);
-    break;
-#endif	/* CAIRO_HAS_WIN32_SURFACE */
   }
 
   window = gtk_widget_get_parent(GTK_WIDGET(d->vbox));
   if (GTK_IS_WINDOW(window)) {
     gtk_widget_get_preferred_size(GTK_WIDGET(d->vbox), &minimum_size, NULL);
-    gtk_window_resize(GTK_WINDOW(window), minimum_size.width, minimum_size.height);
+    gtk_window_set_default_size(GTK_WINDOW(window), minimum_size.width, minimum_size.height);
   }
 }
 
 static void
 OutputImageDialogSetup(GtkWidget *wi, void *data, int makewidget)
 {
-  GtkWidget *w;
   struct OutputImageDialog *d;
   char *title;
 
   d = (struct OutputImageDialog *) data;
   if (makewidget) {
+    GtkWidget *w, *box;
     w = gtk_check_button_new_with_mnemonic(_("_Convert texts to paths"));
     d->t2p = w;
-    gtk_box_pack_start(GTK_BOX(d->vbox), w, FALSE, FALSE, 4);
+    gtk_box_append(GTK_BOX(d->vbox), w);
 
     w = gtk_check_button_new_with_mnemonic(_("_Use opacity"));
     d->use_opacity = w;
-    gtk_box_pack_start(GTK_BOX(d->vbox), w, FALSE, FALSE, 4);
+    gtk_box_append(GTK_BOX(d->vbox), w);
 
     w = gtk_spin_button_new_with_range(1, DPI_MAX, 1);
     gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(w), TRUE);
-    gtk_entry_set_activates_default(GTK_ENTRY(w), TRUE);
+    spin_button_set_activates_default(w);
     d->dpi = w;
     item_setup(GTK_WIDGET(d->vbox), w, "_DPI:", FALSE);
 
+    box = gtk_widget_get_parent (d->dpi);
+    g_object_bind_property (d->use_opacity, "active", box, "sensitive", G_BINDING_SYNC_CREATE);
+
     w = combo_box_create();
     d->version = w;
     item_setup(GTK_WIDGET(d->vbox), w, "", FALSE);
-    gtk_widget_show_all(GTK_WIDGET(d->vbox));
   }
 
   switch (d->DlgType) {
@@ -237,11 +233,6 @@ OutputImageDialogSetup(GtkWidget *wi, void *data, int makewidget)
   case MenuIdOutputSVGFile:
     title = N_("Cairo SVG Output");
     break;
-#ifdef CAIRO_HAS_WIN32_SURFACE
-  case MenuIdOutputCairoEMFFile:
-    title = N_("Cairo EMF Output");
-    break;
-#endif	/* CAIRO_HAS_WIN32_SURFACE */
   default:
     title = NULL; /* not reachable */
   }
@@ -262,11 +253,11 @@ OutputImageDialogClose(GtkWidget *w, void *data)
     return;
 
   d->Dpi = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(d->dpi));
-  d->UseOpacity = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(d->use_opacity));
+  d->UseOpacity = gtk_check_button_get_active(GTK_CHECK_BUTTON(d->use_opacity));
 
   switch (d->DlgType) {
   case MenuIdOutputPSFile:
-    d->text2path = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(d->t2p));
+    d->text2path = gtk_check_button_get_active(GTK_CHECK_BUTTON(d->t2p));
     a = combo_box_get_active(d->version);
     switch (a) {
     case 0:
@@ -280,7 +271,7 @@ OutputImageDialogClose(GtkWidget *w, void *data)
     Menulocal.ps_version = a;
     break;
   case MenuIdOutputEPSFile:
-    d->text2path = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(d->t2p));
+    d->text2path = gtk_check_button_get_active(GTK_CHECK_BUTTON(d->t2p));
     a = combo_box_get_active(d->version);
     switch (a) {
     case 0:
@@ -297,18 +288,12 @@ OutputImageDialogClose(GtkWidget *w, void *data)
     d->Version = TYPE_PNG;
     Menulocal.png_dpi = d->Dpi;
     break;
-#ifdef CAIRO_HAS_WIN32_SURFACE
-  case MenuIdOutputCairoEMFFile:
-    d->Version = TYPE_EMF;
-    Menulocal.emf_dpi = d->Dpi;
-    break;
-#endif	/* CAIRO_HAS_WIN32_SURFACE */
   case MenuIdOutputPDFFile:
-    d->text2path = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(d->t2p));
+    d->text2path = gtk_check_button_get_active(GTK_CHECK_BUTTON(d->t2p));
     d->Version = TYPE_PDF;
     break;
   case MenuIdOutputSVGFile:
-    d->text2path = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(d->t2p));
+    d->text2path = gtk_check_button_get_active(GTK_CHECK_BUTTON(d->t2p));
     a = combo_box_get_active(d->version);
     switch (a) {
     case 0:
@@ -325,7 +310,7 @@ OutputImageDialogClose(GtkWidget *w, void *data)
 
 }
 
-void
+static void
 OutputImageDialog(struct OutputImageDialog *data, int type)
 {
   data->SetupWindow = OutputImageDialogSetup;
@@ -333,23 +318,65 @@ OutputImageDialog(struct OutputImageDialog *data, int type)
   data->DlgType = type;
 }
 
-static void
-draw_gra(struct objlist *graobj, int id, char *msg, int close)
-{
-  ProgressDialogCreate(msg);
-  SetStatusBar(msg);
+typedef void (* draw_gra_cb) (gpointer user_data);
 
+struct draw_gra_data {
+  struct objlist *graobj;
+  int id, close;
+  gpointer data;
+  draw_gra_cb cb;
+};
+
+static void
+draw_gra_finalize(gpointer user_data)
+{
+  struct draw_gra_data *data;
+  data = (struct draw_gra_data *) user_data;
+
+  ResetStatusBar();
+  main_window_redraw();
+  if (data->cb) {
+    data->cb(data->data);
+  }
+  g_free(user_data);
+}
+
+static void
+draw_gra_main(gpointer user_data)
+{
+  struct objlist *graobj;
+  int id;
+  struct draw_gra_data *data;
+  data = (struct draw_gra_data *) user_data;
+
+  graobj = data->graobj;
+  id = data->id;
   if (exeobj(graobj, "open", id, 0, NULL) == 0) {
     exeobj(graobj, "draw", id, 0, NULL);
     exeobj(graobj, "flush", id, 0, NULL);
-    if (close) {
+    if (data->close) {
       exeobj(graobj, "close", id, 0, NULL);
     }
   }
+}
 
-  ProgressDialogFinalize();
-  ResetStatusBar();
-  main_window_redraw();
+static void
+draw_gra(struct objlist *graobj, int id, char *msg, int close, draw_gra_cb cb, gpointer user_data )
+{
+  struct draw_gra_data *data;
+  data = g_malloc0(sizeof(*data));
+  data->graobj = graobj;
+  data->id = id;
+  data->close = close;
+  data->data = user_data;
+  data->cb = cb;
+  SetStatusBar(msg);
+  if (cb) {
+    ProgressDialogCreate(msg, draw_gra_main, draw_gra_finalize, data);
+  } else {
+    draw_gra_main(data);
+    draw_gra_finalize(data);
+  }
 }
 
 static void
@@ -372,12 +399,12 @@ draw_page(GtkPrintOperation *operation, GtkPrintContext *context, int page_nr, g
   r = _exeobj(g2wobj, "_context", g2winst, 1, argv);
 
   if (r == 0) {
-    draw_gra(graobj, id, _("Printing."), TRUE);
+    draw_gra(graobj, id, _("Printing."), TRUE, NULL, NULL);
   }
 }
 
 static void
-init_graobj(struct objlist *graobj, int id, char *dev_name, int dev_oid)
+init_graobj(struct objlist *graobj, int id, const char *dev_name, int dev_oid)
 {
   struct narray *drawrable;
   char *device;
@@ -406,8 +433,8 @@ init_graobj(struct objlist *graobj, int id, char *dev_name, int dev_oid)
   putobj(graobj, "device", id, device);
 }
 
-void
-CmOutputPrinter(int select_file, int show_dialog)
+static void
+output_printer(int select_file, int show_dialog)
 {
   GtkPrintOperation *print;
   GtkPrintOperationResult res;
@@ -418,12 +445,7 @@ CmOutputPrinter(int select_file, int show_dialog)
   struct print_obj pobj;
   GtkPaperSize *paper_size;
   GtkPageSetup *page_setup;
-
-  if (Menulock || Globallock)
-    return;
-
-  if (select_file && ! SetFileHidden())
-    return;
+  int use_opacity;
 
   FileAutoScale();
   AdjustAxis();
@@ -440,7 +462,8 @@ CmOutputPrinter(int select_file, int show_dialog)
   if (g2wid < 0)
     return;
 
-  putobj(g2wobj, "use_opacity", g2wid, &Menulocal.use_opacity);
+  use_opacity = TRUE;
+  putobj(g2wobj, "use_opacity", g2wid, &use_opacity);
 
   g2winst = chkobjinst(g2wobj, g2wid);
   _getobj(g2wobj, "oid", g2winst, &g2woid);
@@ -536,8 +559,75 @@ CmOutputPrinter(int select_file, int show_dialog)
   }
 }
 
+struct output_printer_data {
+  int select_file, show_dialog;
+  response_cb cb;
+  gpointer data;
+};
+
+static void
+output_printer_response(int res, gpointer user_data)
+{
+  struct output_printer_data *data;
+  data = (struct output_printer_data *) user_data;
+  if (res) {
+    output_printer(data->select_file, data->show_dialog);
+  }
+  if (data->cb) {
+    data->cb(res, data->data);
+  }
+  g_free(data);
+}
+
 void
-CmOutputViewerB(void *wi, gpointer client_data)
+CmOutputPrinter(int select_file, int show_dialog, response_cb cb, gpointer user_data)
+{
+  if (Menulock || Globallock) {
+    if (cb) {
+      cb(IDCANCEL, user_data);
+    }
+    return;
+  }
+
+  if (select_file) {
+    struct output_printer_data *data;
+    data = g_malloc0(sizeof(*data));
+    if (data) {
+      data->select_file = select_file;
+      data->show_dialog = show_dialog;
+      data->cb = cb;
+      data->data = user_data;
+      SetFileHidden(output_printer_response, data);
+      return;
+    }
+  }
+  output_printer(select_file, show_dialog);
+  if (cb) {
+    cb(IDOK, user_data);
+  }
+}
+
+struct previewer_data {
+  struct objlist *g2wobj;
+  N_VALUE *g2winst;
+  int g2wid;
+};
+
+static void
+previewer_cb(gpointer user_data)
+{
+  struct previewer_data *data;
+  int delgra;
+
+  data = (struct previewer_data *) user_data;
+  delgra = TRUE;
+  _putobj(data->g2wobj, "delete_gra", data->g2winst, &delgra);
+  exeobj(data->g2wobj, "present", data->g2wid, 0, NULL);
+  g_free(data);
+}
+
+void
+CmOutputViewerB(void)
 {
   if (Menulock || Globallock)
     return;
@@ -565,7 +655,8 @@ CmOutputViewerB(void *wi, gpointer client_data)
     struct objlist *graobj, *g2wobj;
     int id, g2wid, g2woid, c;
     N_VALUE *g2winst;
-    int delgra;
+    struct previewer_data *data;
+    int use_opacity;
 
     if ((graobj = chkobject("gra")) == NULL)
       return;
@@ -588,15 +679,15 @@ CmOutputViewerB(void *wi, gpointer client_data)
     putobj(g2wobj, "BG", g2wid, &c);
     c = Menulocal.bg_b * 255.0;
     putobj(g2wobj, "BB", g2wid, &c);
-    putobj(g2wobj, "use_opacity", g2wid, &Menulocal.use_opacity);
+    use_opacity = TRUE;
+    putobj(g2wobj, "use_opacity", g2wid, &use_opacity);
     id = newobj(graobj);
     init_graobj(graobj, id, "gra2gtk", g2woid);
-    draw_gra(graobj, id, _("Spawning external viewer."), FALSE);
-
-    delgra = TRUE;
-    _putobj(g2wobj, "delete_gra", g2winst, &delgra);
-
-    exeobj(g2wobj, "present", g2wid, 0, NULL);
+    data = g_malloc0(sizeof(*data));
+    data->g2wobj = g2wobj;
+    data->g2wid = g2wid;
+    data->g2winst = g2winst;
+    draw_gra(graobj, id, _("Spawning external viewer."), FALSE, previewer_cb, data);
   }
 }
 
@@ -606,7 +697,7 @@ get_base_ngp_name(void)
   char *ptr, *tmp;
 
   if (NgraphApp.FileName == NULL)
-    return NULL;
+    return g_strdup(_("untitled"));
 
   tmp = g_strdup(NgraphApp.FileName);
   if (tmp == NULL)
@@ -620,34 +711,46 @@ get_base_ngp_name(void)
   return tmp;
 }
 
-static void
-CmPrintGRAFile(void)
+struct gra_out_data
 {
   struct objlist *graobj, *g2wobj;
-  int id, g2wid, g2woid, ret;
-  N_VALUE *g2winst;
-  char *tmp, *file;
+  int id, g2wid;
+};
 
-  if (Menulock || Globallock)
-    return;
+static void
+gra_out_cb(gpointer user_data)
+{
+  struct gra_out_data *data;
+  data = (struct gra_out_data *) user_data;
 
-  tmp = get_base_ngp_name();
+  delobj(data->graobj, data->id);
+  delobj(data->g2wobj, data->g2wid);
 
-  ret = nGetSaveFileName(TopLevel, _("GRA file"), "gra", NULL, tmp,
-			 &file, FALSE, Menulocal.changedirectory);
-
-  if (tmp)
-    g_free(tmp);
-
-  if (ret != IDOK)
-    return;
-
-  if (file == NULL) {
-    return;
+  if (Menulocal.select_data) {
+    FileWinUpdate(NgraphApp.FileWin.data.data, TRUE, TRUE);
   }
+  g_free(data);
+}
 
-  if (Menulocal.select_data && ! SetFileHidden())
-    return;
+struct gra_out_data *
+create_gra_out_data(struct objlist *graobj, int id, struct objlist *g2wobj, int g2wid)
+{
+  struct gra_out_data *data;
+  data = g_malloc0(sizeof(*data));
+  data->graobj = graobj;
+  data->id = id;
+  data->g2wobj = g2wobj;
+  data->g2wid = g2wid;
+  return data;
+}
+
+static void
+print_gra_file(char *file)
+{
+  struct objlist *graobj, *g2wobj;
+  int id, g2wid, g2woid;
+  N_VALUE *g2winst;
+  struct gra_out_data *data;
 
   FileAutoScale();
   AdjustAxis();
@@ -673,86 +776,75 @@ CmPrintGRAFile(void)
   putobj(g2wobj, "file", g2wid, file);
   id = newobj(graobj);
   init_graobj(graobj, id, "gra2file", g2woid);
-  draw_gra(graobj, id, _("Making GRA file."), TRUE);
-  delobj(graobj, id);
-  delobj(g2wobj, g2wid);
 
-  if (Menulocal.select_data) {
-    FileWinUpdate(NgraphApp.FileWin.data.data, TRUE, TRUE);
+  data = create_gra_out_data(graobj, id, g2wobj, g2wid);
+  draw_gra(graobj, id, _("Making GRA file."), TRUE, gra_out_cb, data);
+}
+
+static void
+print_gra_file_response(int res, gpointer user_data)
+{
+  char *file;
+  file = (char *) user_data;
+  if (res) {
+    print_gra_file(file);
   }
 }
 
 static void
-CmOutputImage(int type)
+CmPrintGRAFile_response(char *file, gpointer user_data)
 {
-  struct objlist *graobj, *g2wobj;
-  int id, g2wid, g2woid;
-  N_VALUE *g2winst;
-  int ret;
-  char *title, *ext_str;
-  char *file, *tmp;
-
-  if (Menulock || Globallock)
-    return;
-
-  switch (type) {
-  case MenuIdOutputPSFile:
-    title = "Save as PostScript";
-    ext_str = "ps";
-    break;
-  case MenuIdOutputEPSFile:
-    title = "Save as Encapsulated PostScript";
-    ext_str = "eps";
-    break;
-  case MenuIdOutputPDFFile:
-    title = "Save as Portable Document Format (PDF)";
-    ext_str = "pdf";
-    break;
-  case MenuIdOutputPNGFile:
-    title = "Save as Portable Network Graphics (PNG)";
-    ext_str = "png";
-    break;
-  case MenuIdOutputSVGFile:
-    title = "Save as Scalable Vector Graphics (SVG)";
-    ext_str = "svg";
-    break;
-#ifdef CAIRO_HAS_WIN32_SURFACE
-  case MenuIdOutputCairoEMFFile:
-    title = "Save as Windows Enhanced Metafile (EMF)";
-    ext_str = "emf";
-    break;
-#endif	/* CAIRO_HAS_WIN32_SURFACE */
-  default:
-    /* not reachable */
-    title = NULL;
-    ext_str = NULL;
-  }
-
-  tmp = get_base_ngp_name();
-  ret = nGetSaveFileName(TopLevel, title, ext_str, NULL, tmp,
-			 &file, FALSE, Menulocal.changedirectory);
-  if (tmp) {
-    g_free(tmp);
-  }
-
-  if (ret != IDOK) {
-    return;
-  }
 
   if (file == NULL) {
     return;
   }
 
-  OutputImageDialog(&DlgImageOut, type);
-  ret = DialogExecute(TopLevel, &DlgImageOut);
-  if (ret != IDOK) {
-    g_free(file);
+  if (Menulocal.select_data) {
+    SetFileHidden(print_gra_file_response, file);
     return;
   }
+  print_gra_file(file);
+}
 
-  if (Menulocal.select_data && ! SetFileHidden())
+static void
+CmPrintGRAFile(void)
+{
+  char *tmp;
+  int chd;
+
+  if (Menulock || Globallock)
     return;
 
+  tmp = get_base_ngp_name();
+
+  chd = Menulocal.changedirectory;
+  nGetSaveFileName(TopLevel, _("GRA file"), "gra", NULL, tmp, chd, CmPrintGRAFile_response, NULL);
+
+  if (tmp)
+    g_free(tmp);
+}
+
+struct output_image_data
+{
+  int type, text2path;
+  char *file;
+};
+
+static void
+output_image(struct output_image_data *d)
+{
+  struct objlist *graobj, *g2wobj;
+  int id, g2wid, g2woid, use_opacity;
+  N_VALUE *g2winst;
+  struct gra_out_data *data;
+  int type, text2path;
+  char *file;
+
+  type = d->type;
+  file = d->file;
+  text2path = d->text2path;
+  g_free (d);
+
   FileAutoScale();
   AdjustAxis();
 
@@ -779,72 +871,177 @@ CmOutputImage(int type)
   id = newobj(graobj);
   putobj(g2wobj, "file", g2wid, file);
 
+  use_opacity = TRUE;
   switch (type) {
   case MenuIdOutputPSFile:
   case MenuIdOutputEPSFile:
+    use_opacity = DlgImageOut.UseOpacity;
+    /* fall through */
   case MenuIdOutputSVGFile:
   case MenuIdOutputPDFFile:
-#ifdef CAIRO_HAS_WIN32_SURFACE
-  case MenuIdOutputCairoEMFFile:
-#endif	/* CAIRO_HAS_WIN32_SURFACE */
-    putobj(g2wobj, "text2path", g2wid, &DlgImageOut.text2path);
+    putobj(g2wobj, "text2path", g2wid, &text2path);
     break;
   case MenuIdOutputPNGFile:
     break;
   }
 
-  putobj(g2wobj, "use_opacity", g2wid, &DlgImageOut.UseOpacity);
+  putobj(g2wobj, "use_opacity", g2wid, &use_opacity);
   putobj(g2wobj, "dpi", g2wid, &DlgImageOut.Dpi);
   putobj(g2wobj, "format", g2wid, &DlgImageOut.Version);
 
   init_graobj(graobj, id, "gra2cairofile", g2woid);
-  draw_gra(graobj, id, _("Drawing."), TRUE);
-  delobj(graobj, id);
-  delobj(g2wobj, g2wid);
+
+  data = create_gra_out_data(graobj, id, g2wobj, g2wid);
+  draw_gra(graobj, id, _("Making image file."), TRUE, gra_out_cb, data);
+}
+
+static void
+output_image_response_response(int res, gpointer user_data)
+{
+  struct output_image_data *data;
+  data = (struct output_image_data *) user_data;
+  if (res) {
+    output_image(data);
+  } else {
+    g_free(data->file);
+    g_free(user_data);
+  }
+}
+
+static void
+output_image_response(struct response_callback *cb)
+{
+  char *file;
+  struct output_image_data *data;
+
+  data = (struct output_image_data *) cb->data;
+  file = data->file;
+
+  if (cb->return_value != IDOK) {
+    g_free(file);
+    g_free(data);
+    return;
+  }
 
   if (Menulocal.select_data) {
-    FileWinUpdate(NgraphApp.FileWin.data.data, TRUE, TRUE);
+    SetFileHidden(output_image_response_response, data);
+    return;
+  }
+  output_image(data);
+}
+
+static void
+CmOutputImage_response(char *file, gpointer user_data)
+{
+  int type;
+  struct output_image_data *data;
+
+  type = GPOINTER_TO_INT(user_data);
+#if WINDOWS
+  switch (type) {
+  case MenuIdOutputEMFFile:
+  case MenuIdOutputEMFPlusFile:
+  case MenuIdOutputEMFClipboard:
+  case MenuIdOutputEMFPlusClipboard:
+    output_emf(type, file);
+    return;
+  }
+#endif
+  if (file == NULL) {
+    return;
+  }
+  data = g_malloc0(sizeof(*data));
+  if (data == NULL) {
+    g_free(file);
+    return;
+  }
+  data->type= type;
+  data->file = file;
+  OutputImageDialog(&DlgImageOut, type);
+  response_callback_add(&DlgImageOut, output_image_response, NULL, data);
+  DialogExecute(TopLevel, &DlgImageOut);
+}
+
+static void
+CmOutputImage(int type)
+{
+  char *title, *ext_str;
+  char *tmp;
+  int chd;
+
+  if (Menulock || Globallock)
+    return;
+
+  switch (type) {
+  case MenuIdOutputPSFile:
+    title = "Save as PostScript";
+    ext_str = "ps";
+    break;
+  case MenuIdOutputEPSFile:
+    title = "Save as Encapsulated PostScript";
+    ext_str = "eps";
+    break;
+  case MenuIdOutputPDFFile:
+    title = "Save as Portable Document Format (PDF)";
+    ext_str = "pdf";
+    break;
+  case MenuIdOutputPNGFile:
+    title = "Save as Portable Network Graphics (PNG)";
+    ext_str = "png";
+    break;
+  case MenuIdOutputSVGFile:
+    title = "Save as Scalable Vector Graphics (SVG)";
+    ext_str = "svg";
+    break;
+#if WINDOWS
+  case MenuIdOutputEMFPlusFile:
+  case MenuIdOutputEMFFile:
+    title = "Save as Windows Enhanced Metafile (EMF)";
+    ext_str = "emf";
+    break;
+  case MenuIdOutputEMFClipboard:
+  case MenuIdOutputEMFPlusClipboard:
+    CmOutputImage_response(NULL, GINT_TO_POINTER (type));
+    return;
+#endif	/* CAIRO_HAS_WIN32_SURFACE */
+  default:
+    /* not reachable */
+    title = NULL;
+    ext_str = NULL;
+  }
+
+  tmp = get_base_ngp_name();
+  chd = Menulocal.changedirectory;
+  nGetSaveFileName(TopLevel, title, ext_str, NULL, tmp, chd, CmOutputImage_response, GINT_TO_POINTER(type));
+  if (tmp) {
+    g_free(tmp);
   }
 }
 
 #if WINDOWS
 static void
-CmOutputEMF(int type)
+output_emf(int type, char *file)
 {
-  struct objlist *graobj, *g2wobj;
-  int id, g2wid, g2woid;
   N_VALUE *g2winst;
-  char *title, *file;
+  int id, g2wid, g2woid;
+  struct objlist *graobj, *g2wobj;
+  struct gra_out_data *data;
+  char *objname;
 
-  if (Menulock || Globallock)
+  switch (type) {
+  case MenuIdOutputEMFPlusFile:
+  case MenuIdOutputEMFPlusClipboard:
+    objname = "gra2emfplus";
+    break;
+  case MenuIdOutputEMFFile:
+  case MenuIdOutputEMFClipboard:
+    objname = "gra2emf";
+    break;
+  default:
+    g_free(file);
     return;
-
-  title = "Save as Windows Enhanced Metafile (EMF)";
-
-  file = NULL;
-  if (type == MenuIdOutputEMFFile) {
-    int ret;
-    char *ext_str, *tmp;
-    ext_str = "emf";
-    tmp = get_base_ngp_name();
-    ret = nGetSaveFileName(TopLevel, title, ext_str, NULL, tmp,
-			   &file, FALSE, Menulocal.changedirectory);
-    if (tmp) {
-      g_free(tmp);
-    }
-
-    if (ret != IDOK) {
-      return;
-    }
-
-    if (file == NULL) {
-      return;
-    }
   }
 
-  if (Menulocal.select_data && ! SetFileHidden())
-    return;
-
   FileAutoScale();
   AdjustAxis();
 
@@ -854,7 +1051,7 @@ CmOutputEMF(int type)
     return;
   }
 
-  g2wobj = chkobject("gra2emf");
+  g2wobj = chkobject(objname);
   if (g2wobj == NULL) {
     g_free(file);
     return;
@@ -870,27 +1067,22 @@ CmOutputEMF(int type)
   _getobj(g2wobj, "oid", g2winst, &g2woid);
   id = newobj(graobj);
   putobj(g2wobj, "file", g2wid, file);
-  init_graobj(graobj, id, "gra2emf", g2woid);
-  draw_gra(graobj, id, _("Drawing."), TRUE);
-  delobj(graobj, id);
-  delobj(g2wobj, g2wid);
-
-  if (Menulocal.select_data) {
-    FileWinUpdate(NgraphApp.FileWin.data.data, TRUE, TRUE);
-  }
+  init_graobj(graobj, id, objname, g2woid);
+  data = create_gra_out_data(graobj, id, g2wobj, g2wid);
+  draw_gra(graobj, id, _("Making image file."), TRUE, gra_out_cb, data);
 }
 #endif
 
 void
-CmOutputPrinterB(void *wi, gpointer client_data)
+CmOutputPrinterB(void)
 {
-  CmOutputPrinter(FALSE, PRINT_SHOW_DIALOG_DIALOG);
+  CmOutputPrinter(FALSE, PRINT_SHOW_DIALOG_DIALOG, NULL, NULL);
 }
 
 void
-CmOutputMenu(void *wi, gpointer client_data)
+CmOutputMenu(int menu_id)
 {
-  switch (GPOINTER_TO_INT(client_data)) {
+  switch (menu_id) {
   case MenuIdOutputGRAFile:
     CmPrintGRAFile();
     break;
@@ -899,16 +1091,13 @@ CmOutputMenu(void *wi, gpointer client_data)
   case MenuIdOutputPDFFile:
   case MenuIdOutputPNGFile:
   case MenuIdOutputSVGFile:
-#ifdef CAIRO_HAS_WIN32_SURFACE
-  case MenuIdOutputCairoEMFFile:
-#endif	/* CAIRO_HAS_WIN32_SURFACE */
-    CmOutputImage(GPOINTER_TO_INT(client_data));
-    break;
 #if WINDOWS
   case MenuIdOutputEMFFile:
+  case MenuIdOutputEMFPlusFile:
   case MenuIdOutputEMFClipboard:
-    CmOutputEMF(GPOINTER_TO_INT(client_data));
-    break;
+  case MenuIdOutputEMFPlusClipboard:
 #endif
+    CmOutputImage(menu_id);
+    break;
   }
 }
