@@ -1,11 +1,13 @@
-/* 
+/*
  * $Id: math_error.h,v 1.5 2009-11-21 11:39:10 hito Exp $
- * 
+ *
  */
 
 #ifndef MATH_ERROR_HEADER
 #define MATH_ERROR_HEADER
 
+#include "math_equation.h"
+
 #define MATH_ERROR_TYPE_SYNTAX	0x00010000
 #define MATH_ERROR_TYPE_PARSE	0x00020000
 #define MATH_ERROR_TYPE_RUNTIME	0x00040000
@@ -24,10 +26,12 @@ enum MATH_ERROR {
   MATH_ERROR_INVALID_FDEF = (MATH_ERROR_TYPE_SYNTAX | 0x0009),
   MATH_ERROR_INVALID_CDEF = (MATH_ERROR_TYPE_PARSE  | 0x000A),
   MATH_ERROR_UNEXP_TOKEN  = (MATH_ERROR_TYPE_SYNTAX | 0x000B),
+  MATH_ERROR_INVALID_ARG  = (MATH_ERROR_TYPE_SYNTAX | 0x000C),
 
   MATH_ERROR_INVALID_PRM  = (MATH_ERROR_TYPE_PARSE  | 0x0010),
   MATH_ERROR_PRM_IN_DEF   = (MATH_ERROR_TYPE_PARSE  | 0x0020),
   MATH_ERROR_CONST_EXIST  = (MATH_ERROR_TYPE_PARSE  | 0x0040),
+  MATH_ERROR_UNTERMINATED_STRING  = (MATH_ERROR_TYPE_PARSE  | 0x0080),
 
   MATH_ERROR_CALCULATION  = (MATH_ERROR_TYPE_RUNTIME | 0x0010),
 
