@@ -51,6 +51,11 @@ enum MATH_EXPRESSION_TYPE {
   MATH_EXPRESSION_TYPE_STRING_ARRAY_ARGUMENT,
   MATH_EXPRESSION_TYPE_BLOCK,
   MATH_EXPRESSION_TYPE_EOEQ,
+  MATH_EXPRESSION_TYPE_BIT_AND,
+  MATH_EXPRESSION_TYPE_BIT_OR,
+  MATH_EXPRESSION_TYPE_BIT_SHFT_L,
+  MATH_EXPRESSION_TYPE_BIT_SHFT_R,
+  MATH_EXPRESSION_TYPE_BIT_NOT,
 };
 
 struct math_func_arg_list {
