@@ -16,7 +16,7 @@ enum ngraph_object_field_type {
   NSARRAY = 9,
   NENUM = 10,
   NOBJ = 11,
-#if USE_LABEL
+#ifdef USE_LABEL
   NLABEL = 12,
 #endif
   NVFUNC = 20,
@@ -103,24 +103,24 @@ const char *ngraph_get_object_name(struct objlist *obj);
 const char *ngraph_get_object_alias(struct objlist *obj);
 const char *ngraph_get_object_field_args(struct objlist *obj, const char *field);
 const char *ngraph_get_object_field(struct objlist *obj, int i);
-const char *ngraph_get_object_version(struct objlist *obj);
+const char *ngraph_get_object_version(const struct objlist *obj);
 struct objlist *ngraph_get_object(const char *name);
 struct objlist *ngraph_get_parent_object(struct objlist *obj);
 struct objlist *ngraph_get_root_object(void);
-struct objlist *ngraph_get_next_object(struct objlist *obj);
-struct objlist *ngraph_get_child_object(struct objlist *obj);
+struct objlist *ngraph_get_next_object(const struct objlist *obj);
+struct objlist *ngraph_get_child_object(const struct objlist *obj);
 struct objlist *ngraph_get_object_instances_by_str(const char *str, int *n, int **ids);
-int ngraph_get_object_id(struct objlist *obj);
-int ngraph_get_object_size(struct objlist *obj);
-int ngraph_get_object_current_id(struct objlist *obj);
-int ngraph_get_object_last_id(struct objlist *obj);
+int ngraph_get_object_id(const struct objlist *obj);
+int ngraph_get_object_size(const struct objlist *obj);
+int ngraph_get_object_current_id(const struct objlist *obj);
+int ngraph_get_object_last_id(const struct objlist *obj);
 int ngraph_get_object_field_num(struct objlist *obj);
 int ngraph_get_object_field_permission(struct objlist *obj, const char *field);
 enum ngraph_object_field_type ngraph_get_object_field_type(struct objlist *obj, const char *field);
 
 int ngraph_puts(const char *s);
 int ngraph_err_puts(const char *s);
-void ngraph_sleep(int t);
+void ngraph_sleep(double t);
 void *ngraph_malloc(size_t size);
 void ngraph_free(void *ptr);
 char *ngraph_strdup(const char *str);
