@@ -272,7 +272,7 @@ static int
 arcgeometry(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,
                  int argc,char **argv)
 {
-  char *field;
+  const char *field;
   int val;
 
   field = (char *) (argv[1]);
