@@ -83,12 +83,15 @@ struct gra_info
 struct gra_cache
 {
   struct GRAdata *data;
+  int lines;
   GStatBuf gstat;
 };
 
 static NHASH GraCache = NULL;
 #endif
 
+#define UPDATE_PROGRESS_LINE_NUM 0xfff
+
 static void
 set_bbox(struct objlist *obj,N_VALUE *inst, struct narray *array, int l, int t, int zx, int zy)
 {
@@ -234,7 +237,7 @@ mergedone(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 
 #if USE_MERGE_CACHE
 static void
-gra_cache_init(struct gra_cache *cache, struct gra_info *info)
+gra_cache_init(struct gra_cache *cache, const struct gra_info *info)
 {
   if (cache->data) {
     gra_data_free(cache->data);
@@ -246,6 +249,7 @@ gra_cache_init(struct gra_cache *cache, struct gra_info *info)
     cache->gstat.st_mtime = 0;
     cache->gstat.st_size = 0;
   }
+  cache->lines = 0;
 }
 
 static int *
@@ -277,12 +281,25 @@ draw_gra_data(struct gra_info *info, struct GRAdata *data)
   return rcode;
 }
 
+static void
+set_merge_progress(int cur, int n)
+{
+  char msgbuf[32];
+
+  snprintf(msgbuf, sizeof(msgbuf), "%d", cur);
+  if (n >= 0) {
+    n++;
+  }
+  set_progress(0, msgbuf, (cur * 1.0) / n);
+}
+
 static int
 read_new_gra_file(struct gra_cache *cache, struct gra_info *info, FILE *fd)
 {
   int r;
   char *buf;
   int rcode;
+  int line = 0;
   struct GRAdata *prev, *data;
 
   gra_cache_init(cache, info);
@@ -290,6 +307,7 @@ read_new_gra_file(struct gra_cache *cache, struct gra_info *info, FILE *fd)
   r = 0;
   prev = NULL;
   while ((rcode = fgetline(fd, &buf)) != 1) {
+    line++;
     if (rcode == -1) {
       return 1;
     }
@@ -303,6 +321,9 @@ read_new_gra_file(struct gra_cache *cache, struct gra_info *info, FILE *fd)
       error2(info->obj, ERRGRAFM, buf);
       goto errexit;
     }
+    if ((line & UPDATE_PROGRESS_LINE_NUM) == 0) {
+      set_merge_progress(line, -1);
+    }
     rcode = draw_gra_data(info, data);
     if (! rcode) {
       GRAdata_free(data);
@@ -317,6 +338,7 @@ read_new_gra_file(struct gra_cache *cache, struct gra_info *info, FILE *fd)
     }
     prev = data;
   }
+  cache->lines = line;
   return r;
 
 errexit:
@@ -331,6 +353,7 @@ read_old_gra_file(struct gra_cache *cache, struct gra_info *info, FILE *fd, cons
   int r;
   char *buf;
   int rcode;
+  int line = 0;
 
   gra_cache_init(cache, NULL);
   rcode = fgetline(fd, &buf);
@@ -344,9 +367,13 @@ read_old_gra_file(struct gra_cache *cache, struct gra_info *info, FILE *fd, cons
 
   r = 0;
   while ((rcode = fgetline(fd, &buf)) != 1) {
+    line++;
     if (rcode == -1) {
       return 1;
     }
+    if ((line & UPDATE_PROGRESS_LINE_NUM) == 0) {
+      set_merge_progress(line, -1);
+    }
     rcode = GRAinputold(info->GC, buf, info->lm, info->tm, info->zmx, info->zmy);
     if (!rcode) {
       error2(info->obj, ERRGRAFM, buf);
@@ -355,6 +382,7 @@ read_old_gra_file(struct gra_cache *cache, struct gra_info *info, FILE *fd, cons
     }
     g_free(buf);
   }
+  cache->lines = line;
   return r;
 }
 
@@ -405,6 +433,7 @@ static int
 draw_gra(struct gra_cache *cache, struct gra_info *info)
 {
   struct GRAdata *data;
+  int line;
 
   if (cache == NULL) {
     return 0;
@@ -414,12 +443,17 @@ draw_gra(struct gra_cache *cache, struct gra_info *info)
   if (data == NULL) {
     return 0;
   }
+  line = 0;
   while (data) {
     int rcode;
+    line++;
     rcode = draw_gra_data(info, data);
     if (! rcode) {
       return 1;
     }
+    if ((line & UPDATE_PROGRESS_LINE_NUM) == 0) {
+      set_merge_progress(line, cache->lines);
+    }
     data = data->next;
   }
   return 0;
@@ -950,7 +984,7 @@ static int
 mergegeometry(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,
                  int argc,char **argv)
 {
-  char *field;
+  const char *field;
   int zoom, val;
   struct narray *array;
 
