@@ -123,11 +123,10 @@ void sherror2(int code,char *mes);
 void sherror3(char *cmd,int code,char *mes);
 void sherror4(char *cmd,int code);
 void shellsavestdio(struct nshell *nshell);
-void shellrestorestdio(struct nshell *nshell);
+void shellrestorestdio(const struct nshell *nshell);
 int setshelloption(struct nshell *nshell,char *opt);
 int getshelloption(struct nshell *nshell,char opt);
 int set_shell_args(struct nshell *nshell, int j, const char *argv0, int argc, char **argv);
-void setshellargument(struct nshell *nshell,int argc,char **argv);
 int printfconsole(const char *fmt,...);
 void ngraphenvironment(struct nshell *nshell);
 int msleep(int ms);
